# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode


from LIB import *
import matplotlib


# uncomment the following to use wx rather than wxagg
# matplotlib.use('WX')
# from matplotlib.backends.backend_wx import FigureCanvasWx as FigureCanvas
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
#from matplotlib.transforms import Value, Bbox, Point as Pt
from matplotlib.backends.backend_agg import RendererAgg
from matplotlib._image import frombyte


class PseudoEvent(object):
    u"Cette classe est destine  maintenir en vie un vnement prim."
    
    _methodes = ("AltDown", "ControlDown", "ShiftDown", "GetPositionTuple", 
                "RightIsDown", "RightIsUp", "LeftIsDown", "LeftIsUp", "GetWheelRotation")
    
    def __init__(self, event):
        self._dict = {}
        for methode in self._methodes:
            if hasattr(event, methode):
                self._dict[methode] = getattr(event, methode)()
                
    def __getattr__(self, name):
        try:
            return lambda:self._dict[name]
        except KeyError:
            print(u"La mthode " + name + " n'est actuellement pas dfinie pour les pseudo-vnements.")
            raise
                    
    def Skip(self):
        pass


class MiniEditeur:
    def __init__(self, parent):
        self.parent = parent
        self.objet = None

        
    def init(self, objet, mode = 0):
        u"""Edition d'un nouvel objet.
        mode = 0: dition du nom de l'objet
        mode = 1: dition de l'tiquette de l'objet"""
        self.close() # finalise l'ventuelle dition en cours
        self.texte = ""
        self.objet = objet
        self.old_label = objet.label()
        self.mode = mode # 
        self.display()
    
    def display(self):
        self.objet.label(self.texte + "...")
    
    def cancel(self):
        self.objet.label(self.old_label)
        self.objet = None


    def ok(self):
        if self.texte:
            try: # si possible, on change le nom de l'objet ; sinon, on change son label.
                nom = self.objet.nom
                if not re.match("""[A-Za-z_][A-Za-z0-9_]*['"]?['"]?[']?$""", self.texte) or self.texte == nom:
                    self.mode = 1
                if self.mode:
                    self.objet.label(self.texte)
                    self.parent.historique.enregistrer(u"%s.label(%s)" %(nom, repr(self.texte)))
                else:
                    self.objet.style(label = self.old_label)
                    self.objet.renommer(self.texte, legende = wxgeo.NOM)
                    self.parent.historique.enregistrer(u"%s.renommer(%s, legende = %s)" %(nom, repr(self.texte), wxgeo.NOM))
                self.objet = None
            except RuntimeError: # on reste en mode edition
                if not param.nom_multiple:
                    self.display()
                    raise
                self.objet.label(self.texte) # par dfaut, si A est rattribu  un point, il sera trait comme tiquette.
                self.parent.historique.enregistrer(u"%s.label(%s)" %(nom, repr(self.texte)))
                self.parent.message(u"Attention : ce nom est dj attribu.")
                self.objet = None
            except:
                self.display()
                raise
        else: # si aucun nom n'est rentre, [ENTREE] est un equivalent de [ECHAP].
            self.cancel()

    def key(self, key):
        if self:
            if key == wx.WXK_BACK:
                self.texte = self.texte[:-1]
                self.display()
            elif key == wx.WXK_ESCAPE:
                self.cancel()
            elif key == wx.WXK_RETURN or key == wx.WXK_NUMPAD_ENTER:
                self.ok()
            elif key == 10: # Ctrl + Entree ( tester sous Linux !)
                self.texte += "\n"
                self.display()
            elif key >= 32 and key <= 255 and key <> wx.WXK_DELETE:
                key = chr(key) # a-z, A-Z, 0-9 ou _
                self.texte += key
                self.display()
    
    def close(self):
        u"Ferme l'diteur. Ne renvoie pas d'erreur s'il est dj ferm."
        if self:
            try:
                self.ok()
            except:
                self.cancel()
                print_error()
                
        
    def __nonzero__(self):
        return self.objet is not None






class Moteur_graphique:
    def __init__(self, parent):
        self.parent = parent
        self.axes = self.parent.axes
        self.__empty_plot__ = self.axes.plot([0,1], [1,0])[0]
        self.__empty_fill__ = self.axes.fill([0, 1, 1, 0], [1, 0, 0, 1])[0]
        self.__empty_text__ = self.axes.text(0, 0, '')
        self.mise_en_cache_axes = [[],[], [], []] # plot, fill, text, artist
        #if self.parent.transformation is not None:
            #self.__empty_plot__.set_transform(self.parent.transformation)
            #self.__empty_fill__.set_transform(self.parent.transformation)
            #self.__empty_text__.set_transform(self.parent.transformation)
        
    def fast_plot(self, x, y, couleur = None, **kwargs):
        ligne = module_copy.copy(self.__empty_plot__)
        if self.parent.transformation is not None:
            x, y = zip(*(zip(x, y)*self.parent.transformation).array)
        ligne.set_data(x, y)
        if couleur:
            ligne._color = couleur
        if kwargs:
            ligne.set(**kwargs)
        #ligne.set_transform(matplotlib.transforms.Affine(1,0,0,1,0,0))
        self.axes.add_artist(ligne)
        return ligne
    
    def fast_plots(self, *liste, **kwargs):
        for elt in liste:
            return [self.fast_plot(*elt, **kwargs) for elt in liste]

    def fast_fill(self, x, y, couleur = None, **kwargs):
        polygone = module_copy.copy(self.__empty_fill__)
        if self.parent.transformation is not None:
            x, y = zip(*(zip(x, y)*self.parent.transformation).array)
        polygone.xy = zip(x, y)
        if couleur:
            polygone._facecolor = couleur
        if kwargs:
            polygone.set(**kwargs)
        self.axes.add_artist(polygone)
        return polygone
    
    def fast_fills(self, *liste, **kwargs):
        for elt in liste:
            return [self.fast_fill(*elt, **kwargs) for elt in liste]

    def fast_text(self, x, y, txt, **kwargs):
        texte = module_copy.copy(self.__empty_text__)
        texte.set_x(x)
        texte.set_y(y)
        if self.parent.transformation is not None:
            x, y = ((x, y)*self.parent.transformation).array[0]
        if param.unicode and not isinstance(txt, unicode):
            txt = unicode(txt, param.encodage)
        texte._text = txt
        if kwargs:
            texte.set(**kwargs)
        self.axes.add_artist(texte)
        return texte
    
    def fast_texts(self, *liste):
        for elt in liste:
            return [self.fast_text(*elt, **kwargs) for elt in liste]

    def fast_clear(self, axes = True):
        self.axes.lines = []
        self.axes.patches = []
        self.axes.texts = []
        self.axes.tables = []
        self.axes.artists = []
        self.axes.images = []
        self.axes.legend_ = None
        self.axes.collections = []  # collection.Collection instances
        self.axes.axison = False
        if axes and self.parent.feuille_actuelle:
            self.draw_axes()

    def fast_draw(self):
        # bug dans matplotlib ?? Je suis oblige de faire marcher le zorder manuellement.
        self.axes.artists.sort(key=lambda x:getattr(x,u"zorder"))
        # contournation d'un bug de matplotlib si un texte est tres excentre de la zone d'affichage.
        xmin, xmax, ymin, ymax = self.parent.fenetre

        if self.parent.transformation is not None:
            xmin, ymin = ((xmin, ymin)*self.parent.transformation).array[0]
            xmax, ymax = ((xmax, ymax)*self.parent.transformation).array[0]
            xmin, xmax = min(xmin, xmax), max(xmin, xmax)
            ymin, ymax = min(ymin, ymax), max(ymin, ymax)
        dx = xmax - xmin
        xsup = xmax + 100*dx
        xinf = xmin - 100*dx
        dy = ymax - ymin
        ysup = ymax + 100*dy
        yinf = ymin - 100*dy
        for artiste in self.axes.artists: # si le texte est trop excentre, on ne l'affiche pas (texte._visible = False)
            if isinstance(artiste, self.__empty_text__.__class__):
                artiste._visible = xinf < artiste._x < xsup and yinf < artiste._y < ysup
        if param.old_matplotlib:
            self.axes.viewLim.intervalx().set_bounds(xmin, xmax)
            self.axes.viewLim.intervaly().set_bounds(ymin, ymax)
        else:
            self.axes.viewLim.set_points(pylab.array([[xmin, ymin], [xmax, ymax]]))
        self.parent.draw()
##        try:
##            self.parent.draw()
##        except:
##            for artiste in self.axes.artists:
##                print artiste
##            raise
    
    
    def en_cache(self):
        return [self.axes.lines[:], self.axes.patches[:], self.axes.texts[:], self.axes.artists[:]]
    
    def restaurer(self, liste):
        self.axes.lines = liste[0][:]
        self.axes.patches = liste[1][:]
        self.axes.texts = liste[2][:]
        self.axes.artists = liste[3][:]
    
    
    
    def draw_axes(self):
        if self.parent.rafraichir_axes:
            self.parent.dessine_axes()
            self.parent.rafraichir_axes = False
            self.mise_en_cache_axes = self.en_cache()
        else:
            self.restaurer(self.mise_en_cache_axes)
            
            
    def ajouter(self, liste):
        self.axes.artists += liste
    
    def supprimer(self, liste):
        self.axes.artists = [artiste for artiste in self.axes.artists if artiste not in liste]
















    





class Affichage(FigureCanvas):
    def __init__(self, parent, fixe = False):
        u"Si fixe = True, l'utilisateur ne peut pas zoomer ou recadrer la fentre d'affichage avec la souris."
        self.parent = parent
        self.figure = Figure(frameon=True, facecolor="w")
        FigureCanvas.__init__(self, parent, -1, self.figure)
        self.axes = self.figure.add_axes([0, 0, 1, 1], frameon=False)
        self.axes.set_xticks([])
        self.axes.set_yticks([])
        #if self.parent.param("transformation_affine") is not None:
        #    self.transformation = matplotlib.transforms.Affine(*self.parent.param("transformation_affine"))
        #    self.figure.set_transform(self.transformation)
        #    self.axes.set_transform(self.transformation)
        #else:
        #    self.transformation = None
        if self.parent.param("transformation") is not None:
            a, b, c, d = self.parent.param("transformation")
            # CODE OBSOLETE ( RCRIRE) : numerix n'existe plus dans matplotlib 0.98+
            self.transformation = matplotlib.numerix.Matrix([[a, b], [c, d]])
        else:
            self.transformation = None

        self.commande = self.parent.commande
        self.historique = self.parent.historique
        self.debut_zoom = None # utilis pour zoomer avec Ctrl + Clic gauche (contiendra la position initiale)
        self.debut_select = None # utilis pour slectionner simultanment plusieurs objet avec Alt + Clic gauche (pos. initiale)
        self.debut_shift = None # utilis pour translater le contenu de la fenetre (position initiale)
        self.select = None # objet couramment slctionn
        self.select_memoire = None # objet devant tre prochainement slctionn (en cas de "litige" entre 2 objets)
        self.etiquette_selectionnee = None # tiquette couramment slctionne
        self.fixe = fixe
        self.interaction = None # fonction  lancer au prochain clic de souris (au lieu des actions par dfaut)
        self.interaction_deplacement = None # fonction  lancer au prochain dplacement de souris (au lieu des actions par dfaut)
        self.editeur = MiniEditeur(self) # edite les noms et etiquettes des points, textes, etc.
        
        
        self.detecter_objets_caches = False # les objets cachs  proximit sont aussi dtects
        self.geler_affichage = False # si on met la valeur a True, self.actualiser() ne fait plus rien.
        # cela permet d'effectuer un certain nombre d'actions rapidement et de n'actualiser qu'a la fin.
        # En particulier, cela sert pour charger une figure depuis un fichier .geo
        
        self.regenerer_liste = True # utilis pour savoir si il faut rgnrer la liste des objets de la feuille.
        
        self.liste_objets_en_gras = WeakList()
        
        self.sel = [] # liste ordonne (pour la dtection) des objets de la feuille actuelle.
        # elle est remise a jour quand self.commande.compteur a chang.
        self.motion_event = None
        self.wheel_event_count = 0
        self.wheel_ctrl_event_count = 0
        self.graph = Moteur_graphique(self)
        self.parametres = [u"taille", u"gradu", u"affiche_axes", u"affiche_quadrillage", u"affiche_fleches", u"repere", u"resolution", u"origine_axes", u"utiliser_repere", u"quadrillages", u"couleur_quadrillage", u"liste_axes", u"orthonorme", u"grille_aimantee"]
        self.initialiser()
        
        self.Bind(wx.EVT_MOUSEWHEEL, self.EventOnWheel)
        self.Bind(wx.EVT_MOTION, self.EventOnMotion)
        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
        self.Bind(wx.EVT_RIGHT_UP, self.OnRightUp)
        self.Bind(wx.EVT_SIZE, self.repaint)
        self.Bind(wx.EVT_CHAR, self.OnChar)
        #self.Bind(wx.EVT_LEAVE_WINDOW, self.OnLeave)
        #self.Bind(wx.EVT_PAINT, self.OnPaint)

        self.pile_instructions = [] # instructions  excuter lorsqu'aucune autre action n'est en cours. (Idle)
        self.Bind(wx.EVT_IDLE, self.OnIdle)
    
    
    def GetTopParent(self):
        fenetre = self
        while fenetre is not None:
            fenetre_precedente = fenetre
            fenetre = fenetre.GetParent()
        return fenetre_precedente
    
    
    
    def initialiser(self):
        # actions a effectuer avant de reexecuter l'historique
        for parametre in self.parametres:
            setattr(self, parametre, self.parent.param(parametre))
        #self.largeur_point = param.largeur_point[0]
        self.rafraichir_axes = True
        
    def message(self, txt, lieu = 0):
        self.parent.parent.parent.message(txt, lieu) # cf. geometrie.py
        
        
    def message_temporaire(txt = None): # A FINIR
        self.message_temp = txt
        if txt is None:
            self.message(self.message_def)
        else:
            self.message(txt)
    
    def exporter(self, nom, dpi = None, zone = None):
        u"""Export de la feuille sous forme d'un fichier (png, eps, ...).
        dpi : rsolution souhaite (en dot par inch)
        zone = (xmin, xmax, ymin, ymax) : pour n'exporter qu'une partie de la feuille (png seulement)."""
        nom = str2(nom) # la mthode savefig ne gre pas l'unicode
        dpi = dpi or param.dpi_export 
        #self.largeur_point = param.largeur_point[1] # pour l'impression, mieux vaut des points fins
        #self.repaint()
        
        self.feuille_actuelle.objets_en_gras()
##        if self.select is not None:
##            self.select.affiche_en_gras(False)
            
        if zone is None:
            self.figure.savefig(nom, dpi = dpi)
        else:
            x0, x1, y0, y1 = zone
            pixelx0, pixely0 = self.coo2pix(x0, y0)
            pixelx1, pixely1 = self.coo2pix(x1, y1)
            l, h = self.GetSize()
            pixelx0, pixelx1 = max(0, min(pixelx0, pixelx1)), min(l, max(pixelx0, pixelx1))
            pixely0, pixely1 = max(0, min(pixely0, pixely1)), min(h, max(pixely0, pixely1))
            if pixelx1 - pixelx0 < 1 or pixely1 - pixely0 < 1:
                raise RuntimeError, "Erreur: image de dimensions nulles !"
            #buffer = self.get_renderer().tostring_argb()
            buffer = self.get_renderer().buffer_rgba(0,0)
            # Note: len(buffer) == l*h*4 (Rouge/Vert/Bleu/Alpha)
            try:
                import numpy
                matrice = numpy.fromstring(buffer, dtype=numpy.ubyte).reshape(h, l, 4)[pixely0:pixely1, pixelx0:pixelx1]
            except:
                matrice = []
                for ligne in xrange(h):
                    matrice.append([])
                    for colonne in xrange(l):
                        i = 4*(ligne*h + colonne)
                        #pixel = buffer[i:i+4]
                        pixel = [ord(buffer[i]), ord(buffer[i + 1]), ord(buffer[i + 2]), ord(buffer[i + 3])]
                        matrice[-1].append(pixel)
                matrice = pylab.array(matrice, typecode = "B")[pixely0:pixely1, pixelx0:pixelx1]
            
            #r = RendererAgg(pixelx1 - pixelx0, pixely1 - pixely0, Value(dpi))
            #r.
            #r._renderer.write_png(nom)
            im = frombyte(matrice, 0)
            im.resize(pixelx1 - pixelx0, pixely1 - pixely0)
            print (pixely1 - pixely0, pixelx1 - pixelx0)
            print matrice.shape
            im.write_png(nom)

            
        self.selection_en_gras()
##        if self.select is not None:
##            self.select.affiche_en_gras(True)
        #self.largeur_point = param.largeur_point[0] # mais a l'ecran, des points plus epais se voient mieux
        #self.repaint()
        
    @property
    def feuille_actuelle(self):
        return self.parent.actuelle

##################################
# Les fonctions suivantes assurent la conversion pixel <-> coordonnees
##################################
# en multipliant m par coeff(0), on convertit un ecart en abcisses de m pixels en coordonnees.
# en multipliant n par coeff(1), on convertit un ecart en ordonnees de n pixels en coordonnees.
# Les fonctions qui suivent permettent la conversion d'un couple de coordonnees en pixels, et reciproquement.
# Si le mode est fixe a 1, le pixel (0,0) sera le coin inferieur gauche (plus intuitif).
# Si le mode est fixe a -1, le pixel (0,0) sera le coin superieur gauche (convention, respectee par WxPython).
        
    @property
    def dimensions(self):
        return self.GetSizeTuple()

    def pas(self):
        return (self.fenetre[1] - self.fenetre[0])/(self.resolution)

    ## <DESUET> ##
    def coeff(self, i): # DESUET
        warning("Desuet. Utiliser dpix2coo et dcoo2pix")
        return (self.fenetre[1+2*i] - self.fenetre[2*i])/self.GetSizeTuple()[i]
        # Rq: une ZeroDivisionError se produit juste aprs avoir beaucoup rduit une fentre,
        # wxpython renvoit parfois (0,0) pour la taille.
        # le plus simple serait de laisser l'erreur, mais a innonde le dbugueur de messages... :-/
            
    def coeffs(self): # DESUET
        warning("Desuet. Utiliser dpix2coo et dcoo2pix")
        return self.coeff(0), self.coeff(1)
    
    def Xpix2coo(self, x, mode = None): # DESUET
        warning("Desuet. Utiliser pix2coo")
        return pylab.array(x)*self.coeff(0) + self.fenetre[0]
    
    def Ypix2coo(self, y, mode = 1): # DESUET
        warning("Desuet. Utiliser pix2coo")
        return mode*pylab.array(y)*self.coeff(1) + pylab.where(mode == 1, self.fenetre[2], self.fenetre[3])
    
    def XYpix2coo(self, couple, mode = 1): # DESUET
        warning("Desuet. Utiliser pix2coo")
        u"""conversion d'un couple de pixels en coordonnes.
        
        Si le mode est fixe a 1, le pixel (0,0) sera le coin inferieur gauche (plus intuitif).
        Si le mode est fixe a -1, le pixel (0,0) sera le coin superieur gauche (convention, respectee par WxPython)."""
        return (self.Xpix2coo(couple[0]), self.Ypix2coo(couple[1], mode))
    
    def Xcoo2pix(self, x): # DESUET
        warning("Desuet. Utiliser XYcoo2pix")
        return (pylab.array(x) - self.fenetre[0])/(self.coeff(0))
    
    def Ycoo2pix(self, y, mode = 1): # DESUET
        warning("Desuet. Utiliser XYcoo2pix")
        return mode*(pylab.array(y) - pylab.where(mode == 1, self.fenetre[2], self.fenetre[3]))/self.coeff(1)

    def XYcoo2pix(self, couple, mode = 1): # DESUET
        warning("Desuet. Utiliser coo2pix")
        u"""Conversion d'un couple de coordonnes en pixels.
        
        Si le mode est fixe a 1, le pixel (0,0) sera le coin inferieur gauche (plus intuitif).
        Si le mode est fixe a -1, le pixel (0,0) sera le coin superieur gauche (convention, respectee par WxPython)."""
        return (self.Xcoo2pix(couple[0]), self.Ycoo2pix(couple[1], mode))
    ## </DESUET> ##

    def coo2pix(self, x, y):
        u"""Convertit des coordonnes en pixel."""
        l, h = self.dimensions
        px = l*(x - self.fenetre[0])/(self.fenetre[1] - self.fenetre[0])
        py = h*(self.fenetre[3] - y)/(self.fenetre[3] - self.fenetre[2])
        return px, py
    
    def pix2coo(self, px, py):
        u"""Convertit un pixel en coordonnes."""
        l, h = self.dimensions
        x = px*(self.fenetre[1] - self.fenetre[0])/l + self.fenetre[0]
        y = py*(self.fenetre[2] - self.fenetre[3])/h + self.fenetre[3]
#        print x,  y,  -x,  -y
        return x, y
        
    def dcoo2pix(self, dx, dy):
        u"""Convertit un dplacement exprim en coordonnes en un dplacement en pixels."""
        l, h = self.dimensions
        dpx = l*dx/(self.fenetre[1] - self.fenetre[0])
        dpy = h*dy/(self.fenetre[2] - self.fenetre[3])
        return dpx, dpy

    def dpix2coo(self, dpx, dpy):
        u"""Convertit un dplacement exprim en pixels en un dplacement exprim en coordonnes."""
        l, h = self.dimensions
        dx = dpx*(self.fenetre[1] - self.fenetre[0])/l
        dy = dpy*(self.fenetre[3] - self.fenetre[2])/h
        return dx, dy

    
#    Gestion de l'affichage
#################################

    
    def repaint(self, event = None): # Utilise pour l'evt RESIZE
        if event: event.Skip() # en particulier, il faut que self.draw() soit appliqu
##        if self.orthonorme:
##            self.orthonormer(rafraichir = False)
        self.rafraichir_axes = True
        self.refresh()
    

            

                
                
        
    def ecran_blanc(self): # affiche un ecran blanc essentiellement quand il n'y a pas de feuille
        self.clear(axes = False)
        self.draw()
        #self.ClearBackground()
        
        
    def affiche(self):
        if self.feuille_actuelle == None:
            self.ecran_blanc()
            return

        self.feuille_actuelle.affiche()
        
        self.actualiser()

    def refresh(self):
        u"""Rafraichir la fentre, et redessiner les objets sensibles au rglage de la fentre.
        
        Dsuet."""
        if self.feuille_actuelle:
            self.feuille_actuelle.refresh(Droite_generique, Interpolation_generique, Demidroite, Vecteur, Segment, Arc_generique, Widget)
        warning("Utilisation a eviter desormais.")

    def saturation(self, i):
        return self.coeff(i)/self.gradu[i]
    


#    Reglage des parametres d'affichage
##########################################
# Gestion des variables d'environnement lies  l'affichage.
# A standardiser.
    
    def affichage_axes(self, afficher = None):
        u"Afficher ou non les axes."
        if afficher is not None:
            if isinstance(afficher, bool):
                self.affiche_axes = afficher
            else:
                self.affiche_axes = not self.affiche_axes
            self.rafraichir_axes = True
            self.parent.affiche()
        return self.affiche_axes

    def affichage_quadrillage(self, afficher = None):
        u"Afficher ou non le(s) quadrillage(s)."
        if afficher is not None:
            if isinstance(afficher, bool):
                self.affiche_quadrillage = afficher
            else:
                self.affiche_quadrillage = not self.affiche_quadrillage
            self.rafraichir_axes = True
            self.parent.affiche()
        return self.affiche_quadrillage

##    def affichage_orthonorme(self, afficher = None):
##        u"Afficher la figure dans un repre toujours orthonorm."
##        if afficher is not None:
##            if isinstance(afficher, bool):
##                self.orthonorme = afficher
##            else:
##                self.orthonorme = not self.orthonorme
##            if self.orthonorme:
##                self.repere_orthonormal()
##                self.rafraichir_axes = True
##                self.parent.affiche()
##        return self.orthonorme
    def affichage_orthonorme(self, afficher = None):
        u"Afficher la figure dans un repre toujours orthonorm."
        if afficher is not None:
            if isinstance(afficher, bool):
                self.orthonorme = afficher
            else:
                self.orthonorme = not self.orthonorme
            self.rafraichir_axes = True
            self.parent.affiche()
        return self.orthonorme
        

    def detecter_caches(self, detecter = None):
        u"Indique si les objets cachs sont dtects ou non."
        if detecter is not None:
            if isinstance(detecter, bool):
                self.detecter_objets_caches = detecter
            else:
                self.detecter_objets_caches = not self.detecter_objets_caches
            self.regenerer_liste = True
        return self.detecter_objets_caches

    def aimanter_grille(self, aimanter = None):
        u"Indique si les points doivent se placer sur le quadrillage."
        if aimanter is not None:
            if isinstance(aimanter, bool):
                self.grille_aimantee = detecter
            else:
                self.grille_aimantee = not self.grille_aimantee
        return self.grille_aimantee


#   Alias
######################

    def clear(self, axes = True):
#         d_.deb()
        self.graph.fast_clear(axes)
#         d_.fin("Efface")

    effacer = clear

    def plot(self, x = pylab.array([0, 1]), y = pylab.array([1, 0]), couleur = "k", **kw):
        return self.graph.fast_plot(x, y, couleur, **kw)

    def fill(self, x = pylab.array([0, 1, 1, 0]), y = pylab.array([1, 0, 0, 1]), couleur = None, **kw):
        return self.graph.fast_fill(x, y, couleur, **kw)

    def text(self, x = 0, y = 0, texte = "", **kw):
        return self.graph.fast_text(x, y, texte, **kw)
    
    def actualiser(self):
        if not self.geler_affichage:
            self.graph.fast_draw()

    def pixel_plot(self, px, py, couleur = "k", **kw):
        x, y = self.pix2coo(px, py)
        return self.plot(x, y, couleur = couleur, **kw)
    
    
#    Autres commandes de dessin
#################################
    
    def dessine_arc(self, x, y, vecteur, couleur = "k"):
        # affiche un petit demi-cercle d'orientation choisie
        # Par ex, pour le vecteur (1,0), ie. ->, l'arc est orient comme ci: (
        # Pour le vecteur (-1,0), ie. <-, l'arc est orient comme ci: ).
        # On travaille sur les pixels pour faire de la trigonometrie plus simplement
        # (le repere devient ainsi orthonorme)
##        coeff0, coeff1 = self.coeffs()
##        if coeff0 == 0 or coeff1 == 0:
##            warning("Rsolution d'affichage incorrecte (division par zro).")
##            return
        vecteur = self.dcoo2pix(*vecteur)
        if vecteur[0] == 0:
            if vecteur[1] > 0:
                angle = math.pi/2
            else:
                angle = -math.pi/2
        else:
            angle = math.atan(vecteur[1]/vecteur[0])
            if vecteur[0] < 0:
                angle += math.pi
        # donne l'angle d'incidence a l'extremite
        t = pylab.arange(angle - math.pi/2, angle + math.pi/2, 0.05)
        R = self.taille["("]
        
        x, y = self.coo2pix(x, y)
        return self.pixel_plot(x + R*(pylab.cos(t) - math.cos(angle)), y + R*(pylab.sin(t) - math.sin(angle)), couleur = couleur)
        #~ import time
        #~ tps=time.time()
        #~ print u"Temps coul :", time.time()-tps
        #~ print u"Temps coul :", time.time()-tps
    

    def dessine_point(self, x, y, couleur = "k", plein = True):
        return self.plot([x], [y], plein and couleur or "w", markersize = 2*self.taille["o"], marker = "o", markeredgecolor = couleur, markeredgewidth = 1, zorder = 2.1)
        # le zorder doit tre suprieur  celui de la courbe (2 par dfaut), et la largeur du trait doit tre celle de la courbe (1 par dfaut)
        

#    Axes et quadrillage
#################################


    def dessine_axe(self, num = 0, nbr_axes = 1):
        u"Dessine l'axe des abscisses (num = 0) ou des ordonnes (num = 1)."

        #un = nbr_axes == 1              # un seul axe
        #x = num == 0; y = not x         # axe des abscisses / des ordonnees
        #rep = self.utiliser_repere      # repere ou non
        vect = self.utiliser_repere and self.repere[1 + num] in string.ascii_lowercase # repere defini ou non par des vecteurs
        correspondance = {"b" : "bottom", "c": "center", "l": "left", "r": "right", "t": "top"}
        
        def f(a, b):
            return iff(num == 0, a, b)

        signe = f(-1, 1)
        origine = self.origine_axes[1 - num]
        
        # On cree l'axe proprement dit:       
        self.plot(f([self.fenetre[0], self.fenetre[1]], [origine, origine]), f([origine, origine], [self.fenetre[2], self.fenetre[3]]), "k")
        
        if self.affiche_fleches:
            # On cree la fleche au bout de l'axe:
            x, y = self.coo2pix(f(self.fenetre[1], origine), f(origine, self.fenetre[3]))
            dx, dy = .5*self.taille[">"]*pylab.array([f(math.sqrt(3), 1), f(1, math.sqrt(3))])
            self.pixel_plot(pylab.array((x - dx, x, x + signe*dx)), pylab.array((y + dy, y, y + signe*dy)), "k")

            
        # FORMAT (_origine_ et _graduation_) : [texte, x, y, alignement horizontal, alignement vertical]
        # _origine_ : infos sur l'origine de l'axe 
        # _graduation_ : infos sur le 2eme point de l'axe

        ox, oy = self.origine_axes
        ux, uy = self.gradu
        # TODO :  tendre  un repre non orthogonal
        kx, ky = self.dpix2coo(1, 1) 
        
        x, y = self.coo2pix(f(ox + ux, ox), f(oy, oy + uy))
        if self.gradu[num] and self.saturation(num) < param.saturation:
            if vect:
                self.pixel_plot(pylab.array((x - dx, x, x + signe*dx)), pylab.array((y + dy, y, y + signe*dy)), "k")
            else:
                d = .5*self.taille["|"]
                self.pixel_plot(pylab.array((f(x, x - d), f(x, x + d))), pylab.array((f(y - d, y), f(y + d, y))), "k")
        
        
        if nbr_axes == 1:                   # un seul axe
            if num:                         # axe des ordonnees
                if self.utiliser_repere:    # utiliser un repere (meme origine sur chaque axe...)
                    _origine_ = [self.repere[0], ox - 8*kx, oy, "r", "c"]
                    if vect:                # reperage par des vecteurs
                        _graduation_ = [u"$\\tt{\\vec{" + self.repere[2] + "}}$", ox - 8*kx, oy + .5*uy, "r", "c"]
                    else:                   # reperage par des points
                        _graduation_ = [self.repere[2], ox - 8*kx, oy + uy, "r", "c"]
                else:                       # pas un repere
                        _origine_ = [str(oy), ox - 8*kx, oy, "r", "c"]
                        _graduation_ = [str(oy + uy), ox - 8*kx, oy + uy, "r", "c"]
            else:                           # axe des abscisses
                if self.utiliser_repere:    # utiliser un repere (meme origine sur chaque axe...)
                    _origine_ = [self.repere[0], ox, oy - 8*ky, "c", "t"]
                    if vect:                # reperage par des vecteurs
                        _graduation_ = [u"$\\tt{\\vec{" + self.repere[1] + "}}$", ox + .5*ux, oy - 8*ky, "c", "t"]
                    else:                   # reperage par des points
                        _graduation_ = [self.repere[1], ox + ux, oy - 8*ky, "c", "t"]
                else:                       # pas un repere
                        _origine_ = [str(ox), ox, oy - 8*ky, "c", "t"]
                        _graduation_ = [str(ox + ux), ox + ux, oy - 8*ky, "c", "t"]
        else:                               # 2 axes
            if num:                         # axe des ordonnees
                if self.utiliser_repere:
                    _origine_ = None
                    if vect:                # reperage par des vecteurs
                        _graduation_ = ["\\vec{" + self.repere[2] + "}", ox - 8*kx, oy + .5*uy, "r", "c"]
                    else:                   # reperage par des points
                        _graduation_ = [self.repere[2], ox - 8*kx, oy + uy, "r", "c"]
                else:                       # pas un repere
                        _origine_ = [str(oy), ox - 8*kx, oy + 3*ky, "r", "b"]
                        _graduation_ = [str(oy + uy), ox - 8*kx, oy + uy, "r", "b"]
            else:                           # axe des abscisses
                if self.utiliser_repere:
                    _origine_ = [self.repere[0], ox - 8*kx, oy - 8*ky, "r", "t"]
                    if vect:                # reperage par des vecteurs
                        _graduation_ = ["\\vec{" + self.repere[1] + "}", ox + .5*ux, oy - 8*ky, "c", "t"]
                    else:                   # reperage par des points
                        _graduation_ = [self.repere[1], ox + ux, oy - 8*ky, "c", "t"]
                else:                       # pas un repere
                        _origine_ = [str(ox), ox + 3*kx, oy - 8*ky, "l", "t"]
                        _graduation_ = [str(ox + ux), ox + ux, oy - 8*ky, "l", "t"]
        
        if _origine_ and self.gradu[num]:
            self.graph.fast_text(_origine_[1], _origine_[2], "$" + _origine_[0] + "$", horizontalalignment = correspondance[_origine_[3]], verticalalignment = correspondance[_origine_[4]])
            
        if _graduation_ and self.gradu[num] and self.saturation(num) < param.saturation:
            self.graph.fast_text(_graduation_[1], _graduation_[2], "$" + _graduation_[0] + "$", horizontalalignment = correspondance[_graduation_[3]], verticalalignment = correspondance[_graduation_[4]], size = 14*param.zoom_texte)
        



        
        
        
        
    def quadriller(self, hauteur = 3, pas = (None, None), style = None, epaisseur = 1, couleur = "k"):
        u"""Cre des graduations ou un quadrillage (si hauteur = None).
        Si pas[0] = 0 (respectivement, pas[1] = 0), les graduations seront
        uniquement horizontales (respectivement verticales)."""
        
        args = []
        origine = self.origine_axes
        pas = list(pas)
        # Je n'utilise pas des listes directement, mais des tuples.
        # Ceci afin de n'avoir jamais la MEME liste.
        
        if hauteur == None: # quadrillage
            xmin, xmax, ymin, ymax = self.fenetre
            _min = ymin, xmin ; _max = ymax, xmax
            if not style:
                style = ":"
        else:               # graduations
            _min = [origine[i] - hauteur*self.coeff(i) for i in (1, 0)] # ymin, xmin
            _max = [origine[i] + hauteur*self.coeff(i) for i in (1, 0)] # ymax, xmax
            if not style:
                style = "-"
            
        #type_ligne = couleur + style
        
        for n in (0, 1):
            if pas[n] == None:  pas[n] = self.gradu[n]
            pas[n] = 1.*abs(pas[n])

            # graduations sur l'axe:
            if pas[n] and self.coeff(n)/pas[n] < param.saturation:
               
                maxi = self.fenetre[2*n + 1]
                if self.affiche_axes and (n in self.liste_axes) and hauteur: # les graduations qui chevauchent les fleches, c'est pas joli
                    maxi -= .7*self.taille[">"]*math.sqrt(3)*self.coeff(n)
                    
                valeurs = pylab.concatenate((pylab.arange(origine[n], self.fenetre[2*n], -pas[n]), pylab.arange(origine[n] + pas[n], maxi, pas[n])))
                for val in valeurs:
                    # si on affiche le quadrillage ou les graduations sur les axes, ca fait pas net:
                    
                    if val <> origine[n] or not self.affiche_axes or not (1 - n in self.liste_axes):
                        if n:   self.plot(pylab.array([_min[n], _max[n]]), pylab.array([val, val]), color = couleur, linestyle = style, linewidth = epaisseur)
                        else:   self.plot(pylab.array([val, val]), pylab.array([_min[n], _max[n]]), color = couleur, linestyle = style, linewidth = epaisseur)
                        
        
    



    
    def dessine_axes(self):
        #if self.rafraichir_axes:
            # on efface tout
        #    self.axes.lines = []
        #    self.axes.texts = []
        #    self.axes.patches = []

            if self.affiche_axes:
                for axe in self.liste_axes:
                    self.dessine_axe(axe, len(self.liste_axes))
                
            if self.affiche_quadrillage:   # quadrillage
                for quadrillage in self.quadrillages:
                    self.quadriller(None, *quadrillage)
            elif self.affiche_axes:
                self.quadriller()   # simples graduations
                
            self.axes_lignes = self.axes.lines[:]
            self.axes_textes = self.axes.texts[:]
            # self.axes_lignes doit etre une copie, distincte, de self.axes.lines.
            # Pour comprendre, essayez ceci : m = [1]; n = m; print n is m; n = m[:]; print n is m.
            #self.rafraichir_axes = False

        #else:     
        #    self.axes.lines = self.axes_lignes[:] # obtention de copies distinctes
        #    self.axes.texts = self.axes_textes[:]
        #    self.axes.patches = []





#      Gestion du zoom, etc...
########################################

        
##    def fen(self, i = None):
##        u"Affiche ou change les limites de la fentre d'affichage (commande de bas niveau)"
##        if type(i) == int:      return self.feuille_actuelle.fenetre()[i]
##        if i:
##            self.feuille_actuelle.__fenetre__ = tuple(i)
##            self.rafraichir_axes = True
##        return self.feuille_actuelle.fenetre()
##    warning("Desuet. Utiliser desormais la propriete '.fenetre'")
##
##    def set_fenetre(self, xmin, xmax, ymin, ymax):
##        u"Change les limites de la fentre d'affichage, et rafraichit l'affichage."
##        self.feuille_actuelle.fenetre(xmin, xmax, ymin, ymax)
##        self.message(u"Fentre modifie.")
##        warning("Desuet. Utiliser desormais la propriete '.fenetre'")
        
    def _get_fenetre(self):
        if self.orthonorme:
            w, h = self.GetSizeTuple()
            fenetre = self.feuille_actuelle.fenetre
            coeff0 = (fenetre[1] - fenetre[0])/w
            coeff1 = (fenetre[3] - fenetre[2])/h
            xmin, xmax, ymin, ymax = fenetre
            xcoeff = pylab.where(coeff0 > coeff1, coeff1/coeff0, 1)
            ycoeff = pylab.where(coeff0 > coeff1, 1, coeff0/coeff1)
            x, y, rx, ry = (xmin+xmax)/2., (ymin+ymax)/2., (xmax-xmin)/2., (ymax-ymin)/2.
            return x - xcoeff*rx, x + xcoeff*rx, y - ycoeff*ry, y + ycoeff*ry
        if hasattr(self, 'ratio'): #  CRIRE
            if self.ratio is not None: # ratio est le 
                xcoeff = pylab.where(self.ratio*self.coeff(0) < self.coeff(1), self.coeff(1)/(self.ratio*self.coeff(0)), 1)
                ycoeff = pylab.where((self.ratio*self.coeff(0)) < self.coeff(1), 1, (self.ratio*self.coeff(0))/self.coeff(1))
                xmin, xmax, ymin, ymax = self.fenetre
                x, y, rx, ry = (xmin+xmax)/2., (ymin+ymax)/2., (xmax-xmin)/2., (ymax-ymin)/2.
                self._fenetre(x - xcoeff*rx, x + xcoeff*rx, y - ycoeff*ry, y + ycoeff*ry)
        return self.feuille_actuelle.fenetre
        
    def _set_fenetre(self, xmin_xmax_ymin_ymax):
        self.feuille_actuelle.fenetre = xmin_xmax_ymin_ymax
        
    def _fenetre(self, xmin, xmax, ymin, ymax):
        u"Commande de bas niveau pour modifier la fentre sans rafraichir l'affichage."
        self.feuille_actuelle.__fenetre__ = xmin, xmax, ymin, ymax
        self.rafraichir_axes = True

    fenetre = property(_get_fenetre, _set_fenetre)
        
    def synchroniser_fenetre(self):
        u"""Dtecte la fentre d'affichage et l'enregistre.
        Ce peut tre utile si l'on utilise une commande de haut niveau de matplolib,
        qui calcule automatiquement la meilleure fentre d'affichage."""
        if param.old_matplotlib:
            self._fenetre(*(self.axes.viewLim.intervalx().get_bounds() + self.axes.viewLim.intervaly().get_bounds()))
        else:
            self._fenetre(*(self.axes.viewLim.intervalx + self.axes.viewLim.intervaly))
        
    def zoomer(self, coeff):
        xmin, xmax, ymin, ymax = self.fenetre
        x, y, rx, ry = (xmin+xmax)/2., (ymin+ymax)/2., (xmax-xmin)/2., (ymax-ymin)/2.
        self.fenetre = x - rx/coeff, x + rx/coeff, y - ry/coeff, y + ry/coeff
        
    def zoom_in(self, event = None):
        self.commande.executer_code_interne(u"zoomer(" + str(param.zoom_in) + u")")
    
    def zoom_out(self, event = None):
        self.commande.executer_code_interne(u"zoomer(" + str(param.zoom_out) + u")")
    
    def repere_orthonormal(self, event = None):
        self.commande.executer_code_interne(u"orthonormer()")
        
    def zoom_automatique(self, event = None):
        self.commande.executer_code_interne(u"zoom_auto()")
        
    
    def zoom_auto(self, rafraichir = True):
        a = self.fenetre
        compteur = 0
        condition = True
        while condition:
            compteur += 1
            self._zoom_auto(rafraichir) # a cause du texte dont la taille est independante de l'echelle, les calculs sont fausss.
            # Mais en ritrant le procd, l'affichage converge rapidement (A OPTIMISER ?).
            b = a
            a = self.fenetre
            erreur_x = abs(a[0] - b[0]) + abs(a[1] - b[1])
            erreur_y = abs(a[2] - b[2]) + abs(a[3] - b[3])
            #dx = a[1] - a[0] # xmax - xmin
            #dy = a[3] - a[2] # ymax - ymin
            condition = erreur_x/self.coeff(0) > 1 or erreur_y/self.coeff(1) > 1 # l'erreur doit tre infrieure  1 pixel
            if compteur > 25:
                print u"chec du zoom automatique."
                break


    def _zoom_auto(self, rafraichir = False):
        actuelle = self.feuille_actuelle
        if actuelle:
            xxyy = zip(*([obj.espace_vital() for obj in actuelle.objets(False) if obj.espace_vital() is not None] + [obj.etiquette.espace_vital() for obj in actuelle.objets(False) if obj.etiquette is not None]))
            if xxyy:
                xmin = min(xxyy[0])
                xmax = max(xxyy[1])
                ymin = min(xxyy[2])
                ymax = max(xxyy[3])
                dx = (xmax-xmin)
                dy = (ymax-ymin)
                if dx < param.tolerance: # des valeurs trop proches pour xmin et xmax risquent de faire planter l'affichage.
                    r = (self.fenetre[1] - self.fenetre[0])/2
                    xmin -= r
                    xmax += r
                else:
                    xmin -= .05*dx
                    xmax += .05*dx 
        
                if dy < param.tolerance: # idem pour ymin et ymax
                    r = abs(self.fenetre[3] - self.fenetre[2])/2
                    ymin -= r
                    ymax += r
                else:
                    ymin -= .05*dy
                    ymax += .05*dy 
                
                if rafraichir:
                    self.fenetre = xmin, xmax, ymin, ymax
                else:
                    self._fenetre(xmin, xmax, ymin, ymax)


    
    def orthonormer(self, mode = 1, rafraichir = True):
        u"""
        mode 0 : on orthonormalise le repre en restreignant la vue.
        mode 1 : on orthonormalise le repre en largissant la vue."""
        if mode:
            xcoeff = pylab.where(self.coeff(0) < self.coeff(1), self.coeff(1)/self.coeff(0), 1)
            ycoeff = pylab.where(self.coeff(0) < self.coeff(1), 1, self.coeff(0)/self.coeff(1))
        else:
            xcoeff = pylab.where(self.coeff(0) > self.coeff(1), self.coeff(1)/self.coeff(0), 1)
            ycoeff = pylab.where(self.coeff(0) > self.coeff(1), 1, self.coeff(0)/self.coeff(1))
        xmin, xmax, ymin, ymax = self.fenetre
        x, y, rx, ry = (xmin+xmax)/2., (ymin+ymax)/2., (xmax-xmin)/2., (ymax-ymin)/2.
        if rafraichir:
            self.fenetre = x - xcoeff*rx, x + xcoeff*rx, y - ycoeff*ry, y + ycoeff*ry
        else:
            self._fenetre(x - xcoeff*rx, x + xcoeff*rx, y - ycoeff*ry, y + ycoeff*ry)

    

    def repere_Oij(self, event = None):
        self.commande.executer_code_interne(u"reperage('O', 'i', 'j')")
        
    def repere_OIJ(self, event = None):
        self.commande.executer_code_interne(u"reperage('O', 'I', 'J')")

    def repere_011(self, event = None):
        self.commande.executer_code_interne(u"reperage('0', '1', '1')")

    def reperage(self, origine, x, y):
        self.repere = (origine, x, y)
        self.refresh()
        
    def graduations(self, xgradu, ygradu):
        self.gradu = xgradu, ygradu
        self.refresh()
        
    def reperage_et_graduations(self, origine, x, y, xgradu, ygradu):
        self.repere = (origine, x, y)
        self.gradu = xgradu, ygradu
        self.refresh()
        
    def modifier_quadrillage(self, *quadrillages):
        self.quadrillages = quadrillages
        self.refresh()
        
    def quadrillage_millimetre(self, event = None):
        self.commande.executer_code_interne(u"modifier_quadrillage(  ((1, 1), ':', 1, 'k'), \
                                                                        ((0.5, 0.5), '-', 0.25, 'k'), \
                                                                        ((0.1, 0.1), '-', 0.1, 'k'), \
                                                                        )")
    
    def quadrillage_millimetre_colore(self, event = None, couleur = None):
        if couleur is None:
            couleur = self.couleur_quadrillage
        self.commande.executer_code_interne(u"modifier_quadrillage(  ((1, 1), ':', 1, '%s'), \
                                                                        ((0.5, 0.5), '-', 0.25, '%s'), \
                                                                        ((0.1, 0.1), '-', 0.1, '%s'), \
                                                                        )" %(couleur, couleur, couleur))
                                    
    def quadrillage_demigraduation(self, event = None):
        self.commande.executer_code_interne(u"modifier_quadrillage(  ((1, 1), ':', 1, 'k'), \
                                                                        ((0.5, 0.5), '-', 0.25, 'k'), \
                                                                        )")
    
    def quadrillage_demigraduation_colore(self, event = None, couleur = None):
        if couleur is None:
            couleur = self.couleur_quadrillage
        self.commande.executer_code_interne(u"modifier_quadrillage(  ((1, 1), ':', 1, '%s'), \
                                                                        ((0.5, 0.5), '-', 0.25, '%s'), \
                                                                        )" %(couleur, couleur))
    
    def quadrillage_defaut(self, event = None):
        self.commande.executer_code_interne(u"modifier_quadrillage(*" + repr(self.parent.param("quadrillages", defaut = True)) + ")")

# Evenements concernant directement la feuille
################################

    def coder(self, event):
        self.commande.executer_code_interne(u"codage(True)")

    def decoder(self, event):
        self.commande.executer_code_interne(u"codage(False)")

    def codage(self, booleen):
        actuelle = self.feuille_actuelle
        if actuelle is not None:
            if booleen:
                actuelle.coder()
            else:
                actuelle.effacer_codage()

    def effacer_traces(self, event):
        self.feuille_actuelle.effacer_traces()
        
    def relier(self, point, x = True, y = True):
        if x:
            self.feuille_actuelle.relier_axe_x(point)
        if y:
            self.feuille_actuelle.relier_axe_y(point)




#    Gestion des evenements (essentiellement la souris).
###############################


    
    
    
    def coordonnees(self, event):
        u"""Renvoie les coordonnes correspondant  l'vnement, converties en coordonnes de la feuille.
        Si [Maj] est enfonce, les coordonnes sont arrondies  la graduation la plus proche.
        """
        x, y = self.pix2coo(*event.GetPositionTuple())
        if event.ShiftDown() or self.grille_aimantee:
            a, b = self.gradu
            return a*round(x/a), b*round(y/b)
        else:
            return x, y
    
    
    
    def interagir(self, fonction, aide = None, fonction_bis = None):
        u"""Permet l'interaction du canevas avec un module externe.
        
        A chaque clic de souris, la fonction indique est appele, avec
        un certains nombre de paramtres comme arguments :
        objet(s)  proximit, position en coordonnes et en pixels...
        
        Une aide est ventuellement affiche dans la barre d'tat de la fentre principale.
        
        fonction_bis est ventuellement utilise lors de chaque dplacement de la souris.
        Elle reoit en argument la position en pixels uniquement (pour ne pas alourdir le traitement).
        
        Exemple de rcupration des arguments :
        si un module appelle self.canvas.interagir(ma_fonction, u"Cliquez n'importe o.", ma_2eme_fonction),
        le module doit dfinir :
        
        def ma_fonction(self, **kw):
            pixel = kw["pixel"]
            position = kw["position"]
            objet_selectionne = kw["selection"]
            autres_objets_a_proximite = kw["autres"]
            print u"clic!"
            
        def ma_2eme_fonction(self, **kw):
            pixel = kw["pixel"]
            print u"a bouge !"
        
        """
        
        if self.feuille_actuelle is not None or not fonction:
            self.interaction = fonction
            self.interaction_deplacement = fonction_bis
            if fonction:
                self.SetCursor(wx.StockCursor(wx.CURSOR_QUESTION_ARROW))
            else:
                self.SetCursor(wx.StockCursor(wx.CURSOR_DEFAULT))
            if aide is not None:
                self.message(aide)
    
    
    def signal(self, event = None):
        if self.interaction:
            pixel = event.GetPositionTuple()
            self.detecter(pixel)
            objet = self.interaction(selection = self.select, autres = self.selections, position = self.coordonnees(event), pixel = pixel)
            self.detecter(pixel)
            
    
    def deplacable(self, obj): # indique si un objet est deplacable
        return isinstance(obj, Objet) and obj._deplacable
    
    def pointable(self, obj): # indique si un objet possede des coordonnees
        return isinstance(obj, Objet) and obj._pointable

    
    def infos(self):
        if self.select is None:
            msg = ""
        else:
            msg = self.select.nom_complet
            if self.pointable(self.select):
                msg += u" de coordonnes "+ str(tuple(self.select.coordonnees))
        self.message(msg)

    def selection_en_gras(self):
        self.feuille_actuelle.objets_en_gras(self.select, *self.liste_objets_en_gras)

    def detecter(self, position = None):
        u"""Dtecte les objets  proximit de la position indique.
        Typiquement, on utilise self.detecter(event.GetPositionTuple())."""
                
        self.debut_zoom = None
        actuelle = self.feuille_actuelle # feuille courante
            
        if actuelle and not self.geler_affichage: 
            if position is None:
                position = self.ScreenToClient(wx.GetMousePosition())
            if param.affiche_coordonnees: # affichage des coordonnees dans la barre d'etat
                self.message(str(self.pix2coo(*position)), 1)
            x, y = position
            # on place les objets 'modifiables' en premier (points libres, glisseurs, textes)
            if self.regenerer_liste:
                self.regenerer_liste = False
                #self.sel = [obj for obj in actuelle.objets(False) if obj.modifiable]
                # les autres objets sont tris selon l'attribut niveau.
                #sel2 = [obj for obj in actuelle.objets(False) if not obj.modifiable]
                #sel2.sort(key=lambda x:getattr(x,"niveau"))
                #self.sel += sel2
                self.sel = actuelle.objets(self.detecter_objets_caches)#[obj for obj in actuelle.objets(False)]
                self.sel.sort(key=lambda x:x.style("niveau"), reverse = True)              
            # liste des objets pres du pointeur de la souris :
            self.selections = []
            for obj in self.sel:
                try:
                    if obj.distance_inf(x, y, param.precision_selection):
                        self.selections.append(obj)
                except:
                    print_error()
                    self.message(u"Erreur: les coordonnes de %s sont incalculables." %obj.nom)
            proximite = len(self.selections)
            if proximite:
##                old = self.select
                self.select = self.selections[0]
                if self.select_memoire and self.select_memoire in self.selections:
                    self.select = self.select_memoire
                if self.interaction:
                    self.SetCursor(wx.StockCursor(wx.CURSOR_QUESTION_ARROW))
                else:
                    if self.deplacable(self.select):
                        self.SetCursor(wx.StockCursor(wx.CURSOR_HAND))
                    else:
                        self.SetCursor(wx.StockCursor(wx.CURSOR_DEFAULT))
##                self.select.affiche_en_gras(True)
##                if old is not self.select and old is not None:
##                    old.affiche_en_gras(False)
#                self.feuille_actuelle.objets_en_gras(self.select)
                #self.ancien_message = 
                self.infos()
            else:
                if self.interaction:
                    self.SetCursor(wx.StockCursor(wx.CURSOR_QUESTION_ARROW))
                else:
                    self.SetCursor(wx.StockCursor(wx.CURSOR_DEFAULT))
##                if self.select is not None:
##                    self.select.affiche_en_gras(False)
##                    self.select = None
##                    #self.message(self.ancien_message)
##                    self.message("")
##                    self.actualiser()
#                self.feuille_actuelle.objets_en_gras()
                self.message("")
                self.select = None
##                if self.select is not None:
##                    self.select.affiche_en_gras(False)
##                    self.select = None
##                    #self.message(self.ancien_message)
##                    self.message("")
##                    self.actualiser()
            self.selection_en_gras()


    def OnIdle(self, event):
        u"Instructions  excuter si aucun processus n'est en cours."
        if self.motion_event is not None:
            try:
                self.OnMotion(self.motion_event)
            finally:
                self.motion_event = None
        elif self.wheel_event_count != 0:
            try:
                self.OnWheel(self.wheel_event_count)
            finally:
                self.wheel_event_count = 0
        elif self.wheel_ctrl_event_count != 0:
            try:
                self.OnCtrlWheel(self.wheel_ctrl_event_count)
            finally:
                self.wheel_ctrl_event_count = 0
        elif self.pile_instructions:
            try:
                fonction, args, kwargs = self.pile_instructions[-1]
                fonction(*args, **kwargs)
            finally:
                self.pile_instructions.pop()
    
    
    def execute_on_idle(self, fonction, *args, **kwargs):
        u"Excuter une fois qu'aucun processus n'est actif."
        self.pile_instructions.append((fonction, args, kwargs))
    
    def EventOnWheel(self, event):
        if event.ControlDown():
            if event.GetWheelRotation() > 0:
                self.wheel_ctrl_event_count += 1
            else:
                self.wheel_ctrl_event_count -= 1
        else:
            if event.GetWheelRotation() > 0:
                self.wheel_event_count += 1
            else:
                self.wheel_event_count -= 1
        
            
    def OnWheel(self, wheel_event_count):     
        u"Gestion du zoom par la roulette de la souris."
        if self.feuille_actuelle == None or self.fixe: return
        if wheel_event_count > 0:
            self.zoomer(param.zoom_in**wheel_event_count)
            if not self.feuille_actuelle.historique.derniere_commande.startswith("zoomer("):
                self.feuille_actuelle.historique.archiver("zoomer(")
#            self.zoom_in()
        else:
#            self.zoom_out()
            self.zoomer(param.zoom_out**(-wheel_event_count))
            if not self.feuille_actuelle.historique.derniere_commande.startswith("zoomer("):
                self.feuille_actuelle.historique.archiver("zoomer(")


    def OnCtrlWheel(self, wheel_ctrl_event_count):     
        u"Gestion du zoom du texte par la roulette de la souris."
        if wheel_ctrl_event_count > 0:
            param.zoom_texte *= param.zoom_texte_in**wheel_ctrl_event_count
        else:
            param.zoom_texte *= param.zoom_texte_out**(-wheel_ctrl_event_count)
        self.feuille_actuelle.refresh()
    
    
    def EventOnMotion(self, event):
        self.motion_event = PseudoEvent(event)
        if event.LeftIsDown() and not self.HasCapture():
            self.CaptureMouse()


    def OnMotion(self, event):
        #if self.FindFocus() in (self.parent, self.parent.parent, self.parent.parent.parent):
        if self.GetTopParent().IsEnabled() and (event.LeftIsDown() or event.RightIsDown()):
            self.SetFocus()
        actuelle = self.feuille_actuelle
        if actuelle is None:    return
                    
        if actuelle.objet_temporaire(): # Utilis pour la prvisualisation d'objets, avant leur construction avec la souris
            actuelle.point_temporaire().coordonnees = self.coordonnees(event)
        
        if event.LeftIsDown():# or self.interaction:
            if event.ControlDown(): # selection d'un zone pour zoomer
                if event.AltDown(): # selection simultane de tous les objets d'une zone (A FAIRE)
                    self.selection_zone(event.GetPositionTuple())

                elif not self.fixe:
                    self.gestion_zoombox(event.GetPositionTuple())

            elif event.AltDown(): # deplacement de l'etiquette d'un objet
                self.debut_zoom = None
                x, y = event.GetPositionTuple()
                if self.etiquette_selectionnee is None:
                    for objet in actuelle.objets(False):
                        if objet.etiquette is not None:
                            try:
                                if objet.etiquette.distance_inf(x, y, param.precision_selection):
                                    self.etiquette_selectionnee = objet.etiquette
                                    x, y = self.etiquette_selectionnee.coordonnees
                                    x1, y1 = self.coordonnees(event)
                                    self.decalage_coordonnees = x - x1, y - y1
                                    break
                            except:
                                print_error()
                                self.message(u"Erreur: impossible de trouver l'tiquette de %s.." %objet.nom)
                if self.etiquette_selectionnee:
                    self.SetCursor(wx.StockCursor(wx.CURSOR_HAND))
                    x, y = self.coordonnees(event)
                    dx, dy = self.decalage_coordonnees
                    self.etiquette_selectionnee(x + dx, y + dy)


            elif self.select is not None and not self.interaction: # deplacement d'un objet avec la souris
                self.debut_zoom = None
                if self.deplacable(self.select):
                    x, y = self.coordonnees(event)
                    dx, dy = self.decalage_coordonnees
                    self.select(x + dx,  y + dy)
                self.infos()

        elif event.RightIsDown() and self.debut_shift and not self.fixe: # deplacement de la feuille
            self.SetCursor(wx.StockCursor(wx.CURSOR_SIZING))
            self.fin_shift = self.pix2coo(*event.GetPositionTuple())
            translation = pylab.array(self.fin_shift) - pylab.array(self.debut_shift)
            self.fenetre = self.fenetre[0] - translation[0], self.fenetre[1] - translation[0], self.fenetre[2] - translation[1], self.fenetre[3] - translation[1]
            if self.select is not None:
                self.select = None
                self.selection_en_gras()
                
        elif self.interaction_deplacement is not None:
            self.interaction_deplacement(pixel = event.GetPositionTuple())
            
        elif not event.ControlDown():   # detection des objets a proximite du pointeur
            self.detecter(event.GetPositionTuple())
        
        
        
    def gestion_zoombox(self, pixel):
#        print "zoombox"
        x, y = pixel
        xmax, ymax = self.GetSize()
        x = max(min(x, xmax), 0)
        y = max(min(y, ymax), 0)
        self.fin_zoom = self.pix2coo(x, y)
        self.debut_zoom = self.debut_zoom or self.fin_zoom
        (x0, y0), (x1, y1) = self.debut_zoom, self.fin_zoom
        if self.orthonorme:
            if ymax*abs(x0 - x1) > xmax*abs(y0 - y1):
                y1 = y0 + ymax/xmax*abs(x0 - x1)*cmp(y1, y0)
            else:
                x1 = x0 + xmax/ymax*abs(y0 - y1)*cmp(x1, x0)
            self.fin_zoom = (x1, y1)
            #if param.bouger_curseur:  # ou comment rendre fou l'utilisateur... ;)
            #    self.WarpPointer(*self.XYcoo2pix((x1, y1), -1))
        config = self.graph.en_cache()
        self.fill(pylab.array([x0,x0,x1,x1]), pylab.array([y0,y1,y1,y0]), facecolor='c', edgecolor='c',alpha = .1)
        self.plot(pylab.array([x0,x0,x1,x1,x0]), pylab.array([y0,y1,y1,y0,y0]), 'c', alpha = 1)

        self.actualiser() # pour ne pas tout rafraichir
        self.graph.restaurer(config)


    def selection_zone(self, pixel):
        x, y = pixel
        xmax, ymax = self.GetSize()
        x = max(min(x, xmax), 0)
        y = max(min(y, ymax), 0)
        self.fin_select = self.pix2coo(x, y)
        self.debut_select = self.debut_select or self.fin_select
        (x0, y0), (x1, y1) = self.debut_select, self.fin_select
        config = self.graph.en_cache()
        self.fill(pylab.array([x0,x0,x1,x1]), pylab.array([y0,y1,y1,y0]), facecolor='y', edgecolor='y',alpha = .1)
        self.plot(pylab.array([x0,x0,x1,x1,x0]), pylab.array([y0,y1,y1,y0,y0]), 'g', linestyle = ":", alpha = 1)
        
        self.actualiser() # pour ne pas tout rafraichir
        self.graph.restaurer(config)

    

    def OnLeave(self, event):
        if self.feuille_actuelle is None: return
                    
        if self.debut_zoom and not self.fixe:
            self.actualiser() # pour ne pas tout rafraichir
            
            






    def OnSelect(self, x0, x1, y0, y1):
        x0, x1 = min(x0, x1), max(x0, x1)
        y0, y1 = min(y0, y1), max(y0, y1)
        objets_dans_la_zone = []
        for objet in self.feuille_actuelle.objets(self.detecter_objets_caches):
            espace = objet.espace_vital()
            if espace is not None:
                xmin, xmax, ymin, ymax = espace
                if x0 <= xmin <= xmax <= x1 and y0 <= ymin <= ymax <= y1:
                    objets_dans_la_zone.append(objet)
        self.feuille_actuelle.objets_en_gras(*objets_dans_la_zone)
                    
        def exporte():
            actuelle = self.feuille_actuelle # feuille de travail courante
            if actuelle and actuelle.sauvegarde["export"]:
                dir, fichier = os.path.split(actuelle.sauvegarde["export"]) # on exporte sous le mme nom qu'avant par dfaut
            elif actuelle and actuelle.sauvegarde["nom"]:
                fichier = actuelle.sauvegarde["nom"] # le nom par defaut est le nom de sauvegarde
                dir = actuelle.sauvegarde["repertoire"]
            else:
                fichier = ""
                if param.rep_export is None:
                    dir = param.repertoire
                else:
                    dir = param.rep_export
                
            dlg = wx.FileDialog(
                self, message = u"Exporter la slection en png", defaultDir=dir, 
                defaultFile=fichier, wildcard = u"PNG - Portable Network Graphic (.png)|*.png", style=wx.SAVE | wx.OVERWRITE_PROMPT | wx.CHANGE_DIR
                )
            
            if dlg.ShowModal() == wx.ID_OK:
                param.rep_export = os.getcwd()
                if param.rep_open is None:
                    param.rep_open = param.rep_save
                if param.rep_save is None:
                    param.rep_save = param.rep_export
                os.chdir(param.repertoire)
##                self.geler_affichage = True
                self.selection_en_gras()
##                for objet in objets_dans_la_zone:
##                    objet.affiche_en_gras(False)
##                self.geler_affichage = False
##                self.parent.affiche()
                path = dlg.GetPath()
                self.exporter(nom = path, zone = (x0, x1, y0, y1))
                actuelle.sauvegarde["export"] = path

        if objets_dans_la_zone:
            dlg = wx.SingleChoiceDialog(self, u"Appliquer  la slection :", u"Slection", 
                        [u"Supprimer les objets", u"Masquer les objets", u"Editer les objets", u"Exporter la zone comme image"],
                        wx.CHOICEDLG_STYLE)

            if dlg.ShowModal() == wx.ID_OK:
                choix = dlg.GetSelection()
                if choix == 0:
                    for objet in objets_dans_la_zone:
                        self.commande.executer_code_interne(u"del %s" %objet.nom, mode = wxgeo.RAPIDE)
                    self.actualiser() # avec le mode rapide, il faut manuellement actualiser  la fin
                    self.commande.actualiser()
                elif choix == 1:
                    for objet in objets_dans_la_zone:
                        self.commande.executer_code_interne(u"%s.cacher()" %objet.nom, mode = wxgeo.RAPIDE)
                    self.actualiser() # avec le mode rapide, il faut manuellement actualiser  la fin
                    self.commande.actualiser()
                elif choix == 2:
                    from GUI.proprietes_objets import Proprietes
                    win = Proprietes(self, objets_dans_la_zone)
                    win.Show(True)
                elif choix == 3:
                    exporte()
                    dlg.Destroy()

            dlg.Destroy()
            
            self.geler_affichage = True
##            for objet in objets_dans_la_zone:
##                objet.affiche_en_gras(False)
            self.selection_en_gras()
            self.geler_affichage = False
        else:
            exporte()
        self.parent.affiche()

    
    def OnLeftDown(self, event):
        if self.HasCapture():
            self.ReleaseMouse()
        self.SetFocus()
        if self.deplacable(self.select):
            x, y = self.select.coordonnees
            x1, y1 = self.coordonnees(event)
            self.decalage_coordonnees = x - x1, y - y1
    
    def OnLeftUp(self, event):
        if self.HasCapture():
            self.ReleaseMouse()

        if self.feuille_actuelle is None: return
        
        if self.etiquette_selectionnee:
            x, y = self.etiquette_selectionnee.coordonnees
            self.historique.enregistrer(u"%s.etiquette(%s, %s, True)" %(self.etiquette_selectionnee.parent.nom, x, y))
            self.etiquette_selectionnee = None
            self.commande.actualiser()
            return
        
        if self.debut_zoom and not self.fixe:
            #self.ReleaseMouse()
            try:
                if event.ControlDown():
                    (x0, y0), (x1, y1) = self.debut_zoom, self.fin_zoom
                    self.commande.executer_code_interne("fenetre = " + str((x0, x1, y0, y1)))
                else:
                    self.parent.affiche()
            finally:
                self.debut_zoom = None

        elif self.debut_select and not self.fixe:
            #self.ReleaseMouse()
            try:
                if event.ControlDown() and event.AltDown():
                    #(x0, y0), (x1, y1) = self.debut_select, self.fin_select
                    (x0, y0), (x1, y1) = self.debut_select, self.fin_select
                    self.OnSelect(x0, x1, y0, y1)
                    #print u"Slection: ", (x0, y0), (x1, y1)
                    #self.commande.executer("set_fenetre(" + str((x0, x1, y0, y1))[1:-1] + ")")
                else:
                    self.parent.affiche()
            finally:
                self.debut_select = None
            
        elif self.interaction:
            self.signal(event)
            self.SetCursor(wx.StockCursor(wx.CURSOR_DEFAULT))
            
        elif self.deplacable(self.select):
            self.historique.enregistrer(self.select.nom + str(tuple(self.select.coo)))
            self.commande.actualiser()
            # pas super precis : il se peut qu'on relache le bouton sans avoir deplace le point.
            # ca fait un enregistrement inutile dans l'historique...

    
    
    def OnRightDown(self, event):
        if self.feuille_actuelle == None: return

        self.editeur.close()
        self.detecter(event.GetPositionTuple())

        if self.select is not None and not event.ControlDown():
            menu = wx.Menu()
            menu.SetTitle(self.select.nom_complet)
            selections = [obj for obj in self.selections if obj is not self.select] # autres objets a proximite
            n = len(selections)
            ids = [wx.NewId() for i in xrange(n + 7)]
            for i in range(n):
                def select(event, obj = selections[i]):
##                    select.affiche_en_gras(False)
                    self.select = self.select_memoire = obj
                    self.selection_en_gras()
                menu.Append(ids[i], u"Slectionner " + selections[i].nom_complet)
                menu.Bind(wx.EVT_MENU, select, id = ids[i])
            if n:
                menu.AppendSeparator()

            menu.Append(ids[n], u"Supprimer")
            def supprimer(event, select = self.select):
                self.commande.executer_code_interne(u"%s.supprimer()" %select.nom)
            menu.Bind(wx.EVT_MENU, supprimer, id = ids[n])

            if self.select.style("visible"):
                chaine = u"Masquer"
            else:
                chaine = u"Afficher"
            menu.Append(ids[n + 1], chaine)
            def masquer(event, select = self.select):
                self.commande.executer_code_interne(u"%s.modifier(visible = %s)" %(select.nom, not self.select.style("visible")))
            menu.Bind(wx.EVT_MENU, masquer, id = ids[n + 1])
            
            menu.Append(ids[n + 2], u"Renommer")
            def renommer(event, select = self.select):
                dlg = wx.TextEntryDialog(self, u"Note: pour personnaliser davantage le texte de l'objet,\nchoisissez \"Texte associ\" dans le menu de l'objet.", u"Renommer l'objet", select.nom_corrige)
                test = True
                while test:
                    test = dlg.ShowModal()
                    if test == wx.ID_OK:
                        try:    # on renomme, et on met l'affichage de la lgende en mode "Nom"
                            self.commande.executer_code_interne(u"%s.renommer(%s, legende = %s)" %(select.nom, repr(dlg.GetValue()), wxgeo.NOM))
                            break
                        except:
                            print_error()
                            
                    if test == wx.ID_CANCEL:
                        break
                dlg.Destroy()
            menu.Bind(wx.EVT_MENU, renommer, id = ids[n + 2])

            menu.Append(ids[n + 3], u"Texte associ")
            def etiquette(event, select = self.select):
                old_style = select.style()
                old_label = select.style(u"label")
                if old_label is None:   # le style label n'existe pas pour l'objet
                    return
                #dlg = wx.TextEntryDialog(self, "Note: le code LATEX doit etre entre $$. Ex: $\\alpha$", "Changer l'etiquette de l'objet", old_label, style = wx.TE_MULTILINE )
                #dlg.FitInside()
                dlg = wx.Dialog(self, -1, u"Changer la lgende de l'objet (texte quelconque)")
                sizer = wx.BoxSizer(wx.VERTICAL)
                sizer.Add(wx.StaticText(dlg, -1, u"Note: le code LATEX doit etre entre $$. Ex: $\\alpha$"), 0, wx.ALIGN_LEFT|wx.ALL, 5)
                dlg.text = wx.TextCtrl(dlg, -1, old_label, size=wx.Size(300,50), style = wx.TE_MULTILINE)
                sizer.Add(dlg.text, 0, wx.ALIGN_LEFT|wx.ALL, 5)
                dlg.cb = wx.CheckBox(dlg, -1, u"Interprter la formule")
                dlg.cb.SetValue(select.style(u"legende") == wxgeo.FORMULE)
                sizer.Add(dlg.cb, 0, wx.ALIGN_LEFT|wx.ALL, 5)
                line = wx.StaticLine(dlg, -1, size=(20,-1), style=wx.LI_HORIZONTAL)
                sizer.Add(line, 0, wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.RIGHT|wx.TOP, 5)
                box = wx.BoxSizer(wx.HORIZONTAL)
                btn = wx.Button(dlg, wx.ID_OK)
                #btn.SetDefault()
                box.Add(btn, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)
                btn = wx.Button(dlg, wx.ID_CANCEL, u" Annuler ")
                box.Add(btn, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)
                sizer.Add(box, 0, wx.ALIGN_RIGHT|wx.ALL, 5)
                dlg.SetSizer(sizer)
                sizer.Fit(dlg)
                dlg.CenterOnScreen()
                test = True
                while test:
                    test = dlg.ShowModal()
                    if test == wx.ID_OK:
                        try:
                            self.commande.executer_code_interne(u"%s.label(%s, %s)" %(select.nom, repr(dlg.text.GetValue()), dlg.cb.GetValue()))
                            break
                        except:
                            select.style(**old_style)
                            #~ print "Ancien style", old_style
                            print_error()
                    if test == wx.ID_CANCEL:
                        #select.label(old_label)
                        #~ select.style(**old_style)
                        break
                dlg.Destroy()
            menu.Bind(wx.EVT_MENU, etiquette, id = ids[n + 3])
            
            if self.select.label():
                chaine = u"Masquer"
            else:
                chaine = u"Afficher"
            menu.Append(ids[n + 4], chaine + u" nom/texte")
            def masquer_nom(event, select = self.select):
                if self.select.label():
                    mode = wxgeo.RIEN
                else:
                    if self.select.style(u"label"):
                        mode = wxgeo.TEXTE
                    else:
                        mode = wxgeo.NOM
                self.commande.executer_code_interne(u"%s.style(refresh = True, legende = %s)" %(select.nom, mode))
            menu.Bind(wx.EVT_MENU, masquer_nom, id = ids[n + 4])

            menu.AppendSeparator()
            
            menu.Append(ids[n + 5], u"Redfinir")
            def redefinir(event, select = self.select):
                nom = select.nom
                dlg = wx.TextEntryDialog(self, u"Exemple: transformez une droite en segment.", u"Redfinir l'objet", select.__repr__(False))
                test = True
                while test:
                    test = dlg.ShowModal()
                    if test == wx.ID_OK:
                        try:    # on redfinit l'objet
                            self.commande.executer_code_interne(u"%s.redefinir(%s)" %(nom, repr(dlg.GetValue())))
                            break
                        except:
                            print_error()
                            
                    if test == wx.ID_CANCEL:
                        break
                dlg.Destroy()
            menu.Bind(wx.EVT_MENU, redefinir, id = ids[n + 5])

            menu.AppendSeparator()

            if isinstance(self.select, Point_generique):
                ids_relier = [wx.NewId(), wx.NewId(), wx.NewId()]
                relier = wx.Menu()
                relier.Append(ids_relier[0], u"aux axes")
                relier.Append(ids_relier[1], u" l'axe des abscisses")
                relier.Append(ids_relier[2], u" l'axe des ordonnes")
                
                
                def relier0(event, self = self, select = self.select):
                    self.commande.executer_code_interne(u"relier(%s)" %select.nom, mode = wxgeo.RAPIDE)
                    self.actualiser() # avec le mode rapide, il faut manuellement actualiser  la fin
                    self.commande.actualiser()
                relier.Bind(wx.EVT_MENU, relier0, id = ids_relier[0])
                
                def relier1(event, self = self, select = self.select):
                    self.commande.executer_code_interne(u"relier(%s, y = False)" %select.nom, mode = wxgeo.RAPIDE)
                    self.actualiser() # avec le mode rapide, il faut manuellement actualiser  la fin
                    self.commande.actualiser()
                relier.Bind(wx.EVT_MENU, relier1, id = ids_relier[1])
                    
                def relier2(event, self = self, select = self.select):
                    self.commande.executer_code_interne(u"relier(%s, x = False)" %select.nom, mode = wxgeo.RAPIDE)
                    self.actualiser() # avec le mode rapide, il faut manuellement actualiser  la fin
                    self.commande.actualiser()
                relier.Bind(wx.EVT_MENU, relier2, id = ids_relier[2])

                menu.AppendMenu(wx.NewId(), u"Relier le point", relier)
                
                
                menu.AppendSeparator()

            menu.Append(ids[n + 6], u"Proprits")
            def proprietes(event, select = self.select):
                from GUI.proprietes_objets import Proprietes
                win = Proprietes(self, [select])
                win.Show(True)
            menu.Bind(wx.EVT_MENU, proprietes, id = ids[n + 6])

            self.PopupMenu(menu)
            menu.Destroy()
            if self.select is not None:
##                self.select.affiche_en_gras(False)
                self.select = None
                self.selection_en_gras()
            self.SetCursor(wx.StockCursor(wx.CURSOR_DEFAULT))
        elif not self.fixe:
            self.debut_shift = self.pix2coo(*event.GetPositionTuple())
            self.SetCursor(wx.StockCursor(wx.CURSOR_SIZING))
            if not self.HasCapture():
                self.CaptureMouse()


    
    
    def OnRightUp(self, event):
        if self.HasCapture():
            self.ReleaseMouse()

        if self.feuille_actuelle == None or self.fixe: return

        self.SetCursor(wx.StockCursor(wx.CURSOR_DEFAULT))

#         (x0, y0), (x1, y1) = self.debut_zoom, self.fin_zoom
#         self.commande.executer("set_fenetre(" + str((x0, x1, y0, y1))[1:-1] + ")")
        if self.debut_shift:
            self.debut_shift= None
            self.historique.enregistrer(u"fenetre = " + str(self.fenetre))
            #self.feuille_actuelle.sauvegarde["modifie"] = True
            self.commande.actualiser()
            #self.parent.rafraichir_titre()
#         self.parent.affiche()


    def editer(self, mode = 0):
        if self.select is not None:
            self.editeur.init(self.select, mode)
        else:
            self.editeur.close()

    
    def OnChar(self, event):
        key = event.GetKeyCode()
        debug(u"key: ", key)
        if key == wx.WXK_DELETE and self.select:
            if event.ShiftDown():
                self.commande.executer_code_interne(u"%s.cacher()" %self.select.nom)
            else:
                self.commande.executer_code_interne(u"%s.supprimer()" %self.select.nom)
        if (key == wx.WXK_RETURN or key == wx.WXK_NUMPAD_ENTER) and self.editeur.objet is not self.select: 
            self.editer(event.ShiftDown())
        elif self.editeur:
            self.editeur.key(key)
        
        if key == wx.WXK_ESCAPE and self.interaction:
            print "ESCAPE !"
            self.interaction(special = "ESC")
        
        if key < 256:debug(chr(key))
        event.Skip()
    
    
    
#    def OnPaint(self, event):
#        self.rafraichir_axes = True
#        self.parent.affiche()

