# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                   Panel                     #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode


# importation des librairies classiques
from LIB import *
from API import *
from sauvegarde import *
from parametres import *


#-----------------------
#       Panels
#-----------------------



#class Canvas_vide:
#    """Canvas bidon, qui sert pour l'initialisation des classes menu."""
#    def __getattr__(self, nom):
#        return None
#
#    def __nonzero__(self):
#        return False
#
#    def __repr__(self):
#        return "<Canvas Vide>"
#
#    def __str__(self):
#        return "<Canvas Vide>"





class Panel_simple(wx.Panel):
    u"""Remplace la classe la classe wx.Panel pour les diffrents modules.
    Pour les modules ayant besoin des fonctions graphiques evolues de WxGomtrie, mieux vaut utiliser
    la classe Panel_API_graphique, plus evolue que celle-ci."""

    def __init__(self, parent, couleur = None):
        wx.Panel.__init__(self, parent, -1)

        self.SetBackgroundColour(couleur is not None or wx.NamedColor(u"WHITE"))
        self.canvas = self.historique = self.commande = None
        self.parent = parent
        self.nom = self.__class__.__name__.lower()

        self.actuelle = None # pas de feuille

        self.menu = None

        #if self.menu == None:
        #    import MOD
        #    self.menu = getattr(MOD, self.nom + "MenuBar")
        #    self.menu.module = self
        #    del MOD


    def message(self, texte = ""):
        self.parent.parent.barre.SetStatusText(texte, 0)

    def changer_titre(self, texte = None):
        if texte is None:
            texte = (self.actuelle is not None) and self.actuelle.nom_complet() or ""
        self.parent.parent.titre(texte)


    def ouvrir(self, fichier):
        fgeo, message = ouvrir_fichierGEO(fichier)
        self.message(message)
        self._ouvrir(fgeo)
        if self.actuelle:   # il y a une feuille
            self.actuelle.sauvegarde["modifie"] = False
            rep, fich = os.path.split(fichier)
            self.actuelle.sauvegarde["repertoire"] = rep
            self.actuelle.sauvegarde["nom"] = removeend(fich, ".geo") # nom sans l'extension
            self.rafraichir_titre()




    def sauvegarder(self, fichier = None):
        if fichier is None:
            if self.actuelle and self.actuelle.sauvegarde["nom"]:
                fichier = os.path.join(self.actuelle.sauvegarde["repertoire"], self.actuelle.sauvegarde["nom"])
            else:
                fichier = "sauvegarde"
        if fichier.endswith(".geoz"):
            fichier = removeend(fichier.strip(), ".geo", ".geoz") + ".geoz"     # le nom par defaut sera ainsi [nom_de_la_feuille].geo"
        else:
            fichier = removeend(fichier.strip(), ".geo", ".geoz") + ".geo"     # le nom par defaut sera ainsi [nom_de_la_feuille].geo"
        f = None
        try:
            fgeo = FichierGEO(module = self.nom)
            self._sauvegarder(fgeo)
            fgeo.ecrire(fichier, zip = fichier.endswith(".geoz"))
            self.message(u"Sauvegarde effectue.")
        except:
            if f:   f.close()
            self.message(u"Echec de la sauvegarde.")
            if param.debug: raise

        if self.actuelle:   # il y a une feuille
            self.actuelle.sauvegarde["modifie"] = False
            rep, fich = os.path.split(fichier)
            self.actuelle.sauvegarde["repertoire"] = rep
            self.actuelle.sauvegarde["nom"] = removeend(fich, ".geo") # nom sans l'extension
            self.rafraichir_titre()


    def _ouvrir(self, fgeo): # fonction a reecrire pour chaque module
        pass


    def _sauvegarder(self, fgeo): # fonction a reecrire pour chaque module
        pass

    def param(self, parametre, valeur = 'X7#H4>u87-Kt/y;tr7"', defaut = False):
        u"""Recherche la valeur d'un paramtre d'abord dans les paramtres du module (paramtres locaux), puis dans ceux de wxgomtrie.
        
        Si defaut vaut True, la valeur par dfaut du paramtre est renvoye."""
        if valeur != 'X7#H4>u87-Kt/y;tr7"':
            setattr(self._param_, parametre, valeur)
        if self._param_ is not None and hasattr(self._param_, parametre):
            if defaut:
                if self._param_._parametres_par_defaut.has_key(parametre):
                    return self._param_._parametres_par_defaut[parametre]
                elif param._parametres_par_defaut.has_key(parametre):
                    return param._parametres_par_defaut[parametre]
            return getattr(self._param_, parametre)
        
        elif hasattr(param, parametre):
            if defaut and param._parametres_par_defaut.has_key(parametre):
                return param._parametres_par_defaut[parametre]
            return getattr(param, parametre)
        else:
            debug(u"Module %s: Paramtre %s introuvable." %(self.__titre__, parametre))


    def sauver_preferences(self, lieu = None):
        if self._param_ is not None:
            try:
                if lieu is None:
                    lieu = path2(param.emplacements["preferences"] + "/" + self.nom + "/parametres.xml")
                fgeo = sauvegarder_module(self._param_, self.nom)
                fgeo.ecrire(lieu)
            except:
                self.message(u"Impossible de sauvegarder les prfrences.")
                print_error()





class Panel_API_graphique(Panel_simple):
    u"""Pour les modules ayant besoin de TOUTE l'API de WxGomtrie (canvas, historique, gestion des commandes, etc...)
    et pas seulement des bibliothques, mieux vaut utiliser cette classe.
    Cela concerne essentiellement les modules qui ont besoin de tracer des objets gomtriques."""

    def __init__(self, parent):
        Panel_simple.__init__(self, parent)

        # contruire toujours dans cet ordre :
        self.historique = Historique(self)
        self.commande = Commande(self)
        self.canvas = Affichage(self)
        self.feuilles = {} # dictionnaire contenant un numero unique comme clef, et une feuille comme valeur associe.
        # on ne peut pas utiliser une liste, car le numro ne doit pas varier quand des feuilles sont supprimes.
        # par ailleurs, id() renvoit de grands nombres, difficiles  retenir...
        # En effet, l'utilisateur vera la feuille recense comme 'Feuille n'.
        self.doc_ouverts = RSSMenu(self.parent, u"Documents ouverts", [], self.charger, u"Documents ouverts.")
        self.nouveau()
        self.canvas.initialiser()


    def nouveau(self):
        self.commande.executer_code_interne(u"creer_feuille()")

    def charger(self, event):
        identifiant = event.nom_menu.split("-")[0] # quelquechose du style 'Feuille n', avec n > 0
        self.commande.executer_code_interne(u"charger_feuille('%s')" %identifiant)

    def fermer(self):
        self.commande.executer_code_interne(u"fermer_feuille()")


    def creer_feuille(self, nom = ""):
        num = self.feuilles and max(self.feuilles) + 1 or 1

        nom = nom or (u"Feuille %s" %num)

        #if self.f(nom):    # il y a une feuille de ce nom ouverte
        #    self.fermer_feuille(nom)

        self.actuelle = self.feuilles[num] = Feuille(self, titre = nom)
        self.update(self.actuelle)
        self.canvas.fenetre = self.param("fenetre") # sert en particulier  orthonormaliser le repre au besoin



    def charger_feuille(self, nom_ou_num):
        if isinstance(nom_ou_num, basestring):
            nom_ou_num = int(nom_ou_num[7:])
        self.update(self.feuilles[nom_ou_num])




    def fermer_feuille(self, nom_ou_num = None): # ferme une feuille
        # AJOUTER UNE PROPOSITION DE SAUVEGARDE
        if self.actuelle: # si il y a une feuille (!)
            if nom_ou_num is None:
                nom_ou_num = self.actuelle.id()
            else:
                if isinstance(nom_ou_num, basestring):
                    nom_ou_num = int(nom_ou_num[7:])
            self.feuilles.pop(nom_ou_num)
            if self.feuilles:
                self.charger_feuille(self.feuilles.keys()[-1])
            else:
                self.update(None)


    def fermer_feuilles(self):
        u"Ferme toute les feuilles."
        i = 0
        while self.actuelle and i < 100000:
            i += 1
            #try:
            self.fermer_feuille()
            #except:
            #    print "Fermeture incorrecte de la feuille %s !" %self.actuelle.nom()
            #    self.feuilles.pop(self.actuelle.nom())
            #    if param.debug:
            #        raise

    def rafraichir_titre(self):
        u"Actualise le titre de la fentre, et la liste des feuilles ouvertes dans le menu."
        self.changer_titre()
        liste_feuilles = [f.nom for f in self.feuilles.values()]
        liste_feuilles.sort()
        self.doc_ouverts.update(liste_feuilles)

    def update(self, feuille):
        u"Fait les actualisations ncessaires quand la feuille courante change."
        self.actuelle = feuille
        self.rafraichir_titre()
        self.canvas.rafraichir_axes = True
        self.affiche()


    def num_feuille(self, feuille):
        for num, f in self.feuilles.items():
            if f is feuille:
                return num
        return 0

    def nom_feuille(self, feuille):
        num = self.num_feuille(feuille)
        if num:
            return "Feuille " + str(num)
        return ""


    def feuille(self, nom_ou_num):
        if isinstance(nom_ou_num, basestring):
            nom_ou_num = int(nom_ou_num[7:])
        return self.feuilles.get(nom_ou_num, None)


    def f(self, nom) : return self.feuille(nom) # alias de feuille()


    def _sauvegarder(self, fgeo):
        fgeo.contenu["Figure"] = [self.actuelle.sauvegarder()]

        fgeo.contenu["Affichage"] = [{}]
        for parametre in self.canvas.parametres:
            fgeo.contenu["Affichage"][0][parametre] = [repr(getattr(self.canvas, parametre))]

        fgeo.contenu["Meta"] = [{}]
        self.actuelle.infos["modification"] = time.strftime("%d/%m/%Y - %H:%M:%S",time.localtime())
        for nom, info in self.actuelle.infos.items():
            fgeo.contenu["Meta"][0][nom] = [info]
        #fgeo.contenu["Meta"][0]["modification"] = [time.strftime("%d/%m/%Y - %H:%M:%S",time.localtime())]




    def _ouvrir(self, fgeo):
        if fgeo.contenu.has_key("Affichage"):
            if fgeo.contenu["Affichage"]:
                parametres = fgeo.contenu["Affichage"][0]
                for parametre in parametres.keys():
                    setattr(self.canvas, parametre, securite.eval_safe(parametres[parametre][0]))

        if fgeo.contenu.has_key("Figure"):
            for figure in fgeo.contenu["Figure"]:                
                self.commande.executer_code_interne(u"creer_feuille()", mode = wxgeo.RAPIDE) # pour que ca apparaisse dans l'historique
                try:
                    for commande in figure.split("\n"):
                        self.commande.executer(commande, abreviations = False, mode = wxgeo.RAPIDE)
                finally:
                    self.rafraichir_feuille()
                    self.commande.actualiser()

        if fgeo.contenu.has_key("Meta"): # obligatoirement APRES la creation du document, donc aprs "Figure"
            infos = fgeo.contenu["Meta"][0]
            for key, value in infos.items():
                self.actuelle.infos[key] = value[0]


        for macro in fgeo.contenu.get("Macro", []):
            code = macro["code"][0]
            autostart = (macro["autostart"][0].strip().capitalize() == "True")
            mode_avance = (macro["mode_avance"][0].strip().capitalize() == "True")
            print "mode avance", mode_avance
            nom = macro["nom"][0]
            self.actuelle.macros[nom] = {"code": code, "autostart": autostart, "mode_avance": mode_avance}
            if autostart:
                self.executer_macro(nom = nom, **self.actuelle.macros[nom])
        
        
    def executer_macro(self, **kw):
        code = kw.get("code", "")
        if kw.get("mode_avance", False):
            code = ":" + code
        else:    
            # On va maintenant rajouter des lignes "pause()" au dbut de chaque boucle "for" ou "while".
            # l'instruction pause() permet d'interrompre la boucle au besoin.
            lignes = code.splitlines()
            rajouter_pause = False
            for i in xrange(len(lignes)):
                if rajouter_pause:
                    lignes[i] = (len(lignes[i]) - len(lignes[i].lstrip()))*" " + "pause()\n" + lignes[i] # on insere "pause()" avant la ligne l, en respectant son indentation.
                    rajouter_pause = False
                if (lignes[i].startswith("for") or lignes[i].startswith("while")) and lignes[i].endswith(":"):
                    rajouter_pause = True
            code = "\n".join(lignes)
        
        if param.multi_threading:
            thread.start_new_thread(self.commande.executer, (code + "\nwx.Yield()",), {"abreviations":False})
        else:
            self.commande.executer(code, abreviations = False)
        
        
            
    def exporter(self, path):
        self.canvas.exporter(path)
        self.actuelle.sauvegarde["export"] = path

    def rafraichir_feuille(self): # rafraichit la feuille actuelle
        if self.actuelle:
            self.actuelle.refresh()


    def affiche(self, event = None):
        try:
            self.canvas.effacer()
            if self.actuelle: # si il y a une feuille pour dessiner
                self._affiche()
            self.canvas.affiche()
        except ZeroDivisionError:  # se produit apres avoir reduit la fenetre, en dessous d'une certaine taille.
            print_error(u"Warning: Fentrage incorrect.")
            # il semble que WxPython mette du temps a determiner la nouvelle taille de la fenetre, et GetSize() renvoit alors temporairement des valeurs nulles
        except:
            print_error()
            self.message(u"Erreur d'affichage !")


    def _affiche(self): # fonction a reecrire pour chaque module
        pass



    def sauver_preferences(self, lieu = None):
        if self._param_ is not None:
            for parametre in self.canvas.parametres: # Permet de sauvegarder les paramtres du moteur d'affichage pour chaque module de manire indpendante.
                setattr(self._param_, parametre, getattr(self.canvas, parametre))
            Panel_simple.sauver_preferences(self, lieu)

