# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                   Objets                    #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode


from points import *






##########################################################################################

# Intersections

def Intersection(objet1, objet2, **styles):
    def case(type1, type2):
        return (isinstance(objet1, type1) and isinstance(objet2, type2))\
                    or (isinstance(objet2, type1) and isinstance(objet1, type2))
    if case(Ligne_generique, Ligne_generique):
        return Intersection_droites(objet1, objet2)
    elif case(Ligne_generique, (Cercle_generique, Arc_generique)):
        return Intersection_droite_cercle(objet1, objet2)
    elif case(Ligne_generique, (Cercle_generique, Arc_generique)):
        return Intersection_cercles(objet1, objet2)
    else:
        raise TypeError, "Intersection d'objets non supportes."
            




class Intersection_generique(Point_generique):
    u"""Une intersection gnrique.

    La classe mre des diffrents types d'intersections"""

    objet1 = __objet1 = Argument("Objet")
    objet2 = __objet2 = Argument("Objet")

    def __init__(self, objet1, objet2, **styles):
        self.__objet1 = objet1 = Ref(objet1)
        self.__objet2 = objet2 = Ref(objet2)
        Point_generique.__init__(self, **styles)
 

    def intersections(self):    # NON OPTIMISE (calcul systmatique)
        if self.existe:
            return self._get_intersections()
        else:
            return None # si l'objet n'existe pas, renvoie None








class Intersection_droites(Intersection_generique): # ATTENTION, il y a des modifications  faire avant de surclasser !
    u"""une intersection de droites.

    L'intersection de 2 droites, segments, demi-droites..."""

    droite1 = __droite1 = Argument("Ligne_generique")
    droite2 = __droite2 = Argument("Ligne_generique")

    def __init__(self, droite1, droite2, **styles):
        self.__droite1 = droite1 = Ref(droite1)
        self.__droite2 = droite2 = Ref(droite2)
        Intersection_generique.__init__(self, objet1 = droite1, objet2 = droite2)


    def _conditions_existence(self):
        a, b, c = self.__droite1.equation
        d, e, f = self.__droite2.equation
        #self.__coeffs = a, b, c, d, e, f
        self.__determinant = a*e - b*d
        if abs(self.__determinant) <= param.tolerance:
            return False
        x = self.__x = (f*b - c*e)/self.__determinant
        y = self.__y = (d*c - a*f)/self.__determinant
        M = (x, y)
        return (isinstance(self.droite1, ALL.Droite_generique) or M in self.droite1) and (isinstance(self.droite2, ALL.Droite_generique) or M in self.droite2)


    def _get_coordonnees(self):
        return (self.__x, self.__y)






class Intersection_droite_cercle(Intersection_generique): # ATTENTION, il y a des modifications  faire avant de surclasser !
    u"""Une intersection d'une droite et d'un cercle.

    Un des deux points M et N d'intersection d'une droite et d'un cercle.
    Supposons qu'on le note M, et que la droite soit (AB).
    L'argument premier_point (optionnel) sert  diffrencier les deux points d'intersection.
    Il sert  indiquer si M se trouve "prs" de A (1er point de (AB)) ou de B (2me point).
    (Plus prcisment, si premier_point == True, MA + NB est minimal).
    """

    droite = __droite = Argument("Ligne_generique")
    cercle = __cercle = Argument("Cercle_generique, Arc_generique")
    premier_point = __premier_point = Argument("bool", defaut = True)

    def __init__(self, droite, cercle, premier_point = None, **styles):
        # l'intersection de la droite et du cercle (si elle existe) est deux points (ventuellement confondus).
        # lorsque l'utilisateur cr deux fois deux suite un point d'intersection, ce ne doit pas tre le mme.
 
        self.__droite = droite = Ref(droite)
        self.__cercle = cercle = Ref(cercle)
        self.__premier_point = premier_point = Ref(premier_point)
        Intersection_generique.__init__(self, objet1 = droite, objet2 = cercle)

    def _set_feuille(self):
        for objet in self.__feuille__.objets(type = Intersection_droite_cercle):
            if objet.droite is droite and objet.cercle is cercle:
                # une intersection existe dj, on va construire l'autre
                self.__premier_point = not objet._Intersection_droite_cercle__premier_point
                return


    def _get_coordonnees(self):
        return self.__sol1
        # on a dj calcul les coordonnes en cherchant les conditions d'existence

    def _get_intersections(self):
        u"""Retourne les coordonnes des deux points d'intersection.
        (Dans l'ordre: celles du point, puis celles du 2me point d'intersection).
        """
        return self.__sol1, self.__sol2


    def _conditions_existence(self):
        # Trouver les conditions d'existence revient pratiquement  chercher les coordonnes.
        # Tout le calcul de coordonnes se fait donc ici, pour simplifier.

        # Cas de l'intersection d'une tangente  un cercle avec "son" cercle :
        if isinstance(self.droite, ALL.Tangente) and self.droite.cercle is self.cercle:
            self.__sol1 = self.__sol2 = self.droite.point_tangence.coordonnees
            return True



        a, b, c = self.droite.equation
        d, e, f = self.cercle.equation

        if b: # La droite n'est pas verticale
            m = -a/b; n = -c/b # equation de la droite : y = mx + n

            #p = Polynome([m**2 + 1, 2*n*m + d + e*m, n**2 + e*n + f])
            sols = racines(m**2 + 1, 2*n*m + d + e*m, n**2 + e*n + f)

            if len(sols) == 2: # 2 points d'intersection.
                x1, x2 = sols
                M = (x1, m*x1+n); N = (x2, m*x2+n)
            elif len(sols) == 1: # 1 seul point
                x = sols[0]
                M = self.__sol1 = self.__sol2 = (x, m*x+n)
                return (isinstance(self.droite, ALL.Droite_generique) or M in self.droite) and (isinstance(self.cercle, ALL.Cercle_generique) or M in self.cercle)
            else:
                return False # intersection vide.

        else: # La droite est verticale
            x = -c/a
            #p = Polynome([1, e, x**2 + d*x + f])
            sols = racines(1, e, x**2 + d*x + f)

            if len(sols) == 2: # 2 points d'intersection.
                y1, y2 = sols
                M = (x, y1); N = (x, y2)
            elif len(sols) == 1:
                y = sols[0]
                M = self.__sol1 = self.__sol2 = (x, y)
                return (isinstance(self.droite, ALL.Droite_generique) or M in self.droite) and (isinstance(self.cercle, ALL.Cercle_generique) or M in self.cercle)
            else:
                return False # intersection vide.

        # La partie la plus complexe consiste a distinguer les deux points d'intersection a l'aide du "point proche".
        # Pour comprendre ce qui s'y passe, il est conseille de faire un dessin !

        if self.__premier_point:
            A = self.__droite.point1
            B = self.__droite.point2
        else:
            A = self.__droite.point2
            B = self.__droite.point1
        u = self._vec(A, B)
        if produit_scalaire(u, self._vec(A, M)) < produit_scalaire(u, self._vec(A, N)):
            self.__sol1, self.__sol2 = M, N
            return (isinstance(self.droite, ALL.Droite_generique) or M in self.droite) and (isinstance(self.cercle, ALL.Cercle_generique) or M in self.cercle)
        self.__sol1, self.__sol2 = N, M
        return (isinstance(self.droite, ALL.Droite_generique) or N in self.droite) and (isinstance(self.cercle, ALL.Cercle_generique) or N in self.cercle)


















class Intersection_cercles(Intersection_generique): # ATTENTION, il y a des modifications  faire avant de surclasser !
    u"""Une intersection de cercles.

    Un des deux points M et N d'intersection des deux cercles.
    L'argument angle_positif permet de distinguer les deux points.
    Si on note A et B les centres respectifs des 2 cercles, l'angle orient (AB>, AM>) doit tre de signe constant.
    """

    cercle1 = __cercle1 = Argument("Cercle_generique, Arc_generique")
    cercle2 = __cercle2 = Argument("Cercle_generique, Arc_generique")
    angle_positif = __angle_positif = Argument("bool", defaut = True)

    def __init__(self, cercle1, cercle2, angle_positif = None, **styles):
        self.__cercle1 = cercle1 = Ref(cercle1)
        self.__cercle2 = cercle2 = Ref(cercle2)
        self.__angle_positif = angle_positif = Ref(angle_positif)
        Intersection_generique.__init__(self, objet1 = cercle1, objet2 = cercle2)


    def _set_feuille_(self):
        # l'intersection de deux cercles (si elle existe) est deux points (eventuellement confondus).
        # lorsque l'utilisateur cree deux fois de suite un point d'intersection, ce ne doit pas etre le meme.
        for objet in self.__feuille__.objets(type = Intersection_cercles):
            if objet._Intersection_cercles__cercle1 is cercle1 and objet._Intersection_cercles__cercle2 is cercle2:
                self.__angle_positif = not objet._Intersection_cercles__angle_positif # pour ne pas crer le mme point.


    def _conditions_existence(self):
        # Trouver les conditions d'existence revient pratiquement  chercher les coordonnes.
        # Tout le calcul de coordonnes se fait donc ici, pour simplifier.

        a, b, c = self.cercle1.equation
        d, e, f = self.cercle2.equation
        #print a, b, c, d, e, f
        if (a, b, c) == (d, e, f):
            return None

        # (a-d)x + (b-e)y + (c-f) = 0 : on soustrait les 2 equations de cercle -> equation d'une droite.
        # On se ramne donc au cas de l'intersection d'un cercle et d'une droite
        a, b, c = a - d, b - e, c - f

        if b: # La droite n'est pas verticale
            m = -a/b; n = -c/b # equation de la droite : y = mx + n

            #p = Polynome([m**2 + 1, 2*n*m + d + e*m, n**2 + e*n + f])
            sols = racines(m**2 + 1, 2*n*m + d + e*m, n**2 + e*n + f)

            if len(sols) == 2: # 2 points d'intersection.
                x1, x2 = sols
                self.__sol1 = (x1, m*x1+n); self.__sol2 = (x2, m*x2+n)
            elif len(sols) == 1: # 1 seul point
                x = sols[0]
                self.__sol1 = self.__sol2 = (x, m*x+n)
            else:
                return False # intersection vide.

        else: # La droite est verticale
            x = -c/a
            #p = Polynome([1, e, x**2 + d*x + f])
            sols = racines(1, e, x**2 + d*x + f)

            if len(sols) == 2: # 2 points d'intersection.
                y1, y2 = sols
                self.__sol1 = (x, y1); self.__sol2 = (x, y2)
            elif len(sols) == 1:
                y = sols[0]
                self.__sol1 = self.__sol2 = (x, y)
            else:
                return False # intersection vide.

        # L encore, la partie la plus complexe consiste  distinguer les deux points d'intersection,  l'aide cette fois du signe de l'angle.
        # Pour comprendre ce qui s'y passe, il est conseill de faire un dessin !
        A = self.cercle1.centre.coordonnees
        B = self.cercle2.centre.coordonnees
        M = self.__sol1
        a = angle_vectoriel(self._vec(A, B), self._vec(A, M))

        if (a<0 and self.__angle_positif) or (a>0 and not self.__angle_positif):
            self.__sol1, self.__sol2 = self.__sol2, self.__sol1
        M = self.__sol1

        return (isinstance(self.cercle1, ALL.Cercle_generique) or M in self.cercle1) and (isinstance(self.cercle2, ALL.Cercle_generique) or M in self.cercle2)


    def _get_coordonnees(self):
        return self.__sol1
        # on a dj calcul les coordonnes en cherchant les conditions d'existence


    def _get_intersections(self):
        u"""Retourne les coordonnes des deux points d'intersection.
        (Dans l'ordre: celles du point, puis celles du 2me point d'intersection).
        """
        return self.__sol1, self.__sol2








