# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                   Objets                    #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode


##from points import *
from lignes import *




##########################################################################################

# Polygones








class Cote(Segment):
    u"""Un cot.

    Un cot d'un polygone, reliant le point numero 'n' au point numero 'n + 1'.
    Note: n commence  0.
    L'objet est cr automatiquement lors de la cration du polygone.
    De plus, si l'objet est supprim, le polygone est automatiquement supprim."""

    _prefixe_nom = "c"
    
    polygone = __polygone = ArgumentNonModifiable("Polygone_generique")
    n = __n = ArgumentNonModifiable("int")

    def __init__(self, polygone, n, **styles):
        self.__polygone = polygone
        self.__n = n
        Segment.__init__(self, polygone._Polygone_generique__sommets[n], polygone._Polygone_generique__sommets[(n + 1)%len(polygone._Polygone_generique__sommets)], **polygone.style())
#        self._enregistrer = False # l'objet ne doit pas tre sauvegard

#    def supprimer(self):
#        self.__polygone.supprimer()

    def _modifier_hierarchie(self, valeur = None):
        # Voir commentaires pour Sommet._modifier_hierarchie
        N = len(self.__polygone._Polygone_generique__points)
        Objet._modifier_hierarchie(self, self.__polygone._hierarchie + (self.__n + N + 2)/(2*N + 2))


    def _update(self, objet):
        u"""Pseudo mise  jour: seul un objet identique est accept.
        
        Cela sert pour les objets crs automatiquement, qui peuvent tre enregistrs deux fois dans la feuille."""
        if isinstance(objet, Cote) and self.__polygone is objet._Cote__polygone and self.__n == objet._Cote__n :
            self.style(**objet.style())
        else:
            raise RuntimeError




class Sommet(Point_generique):
    u"""Un sommet.

    Le nime sommet d'un polygone.
    Note: n commence  0.
    L'objet est cr automatiquement lors de la cration du polygone.
    De plus, si l'objet est supprim, le polygone est automatiquement supprim."""

    _style_defaut = {"legende" : wxgeo.RIEN}
    _prefixe_nom = "S"
    
    polygone = __polygone = ArgumentNonModifiable("Polygone_generique")
    n = __n = ArgumentNonModifiable("int")

    def __init__(self, polygone, n, **styles):
        self.__polygone = polygone
        self.__n = n
        Point_generique.__init__(self, **styles)
#        self.__point = polygone.points[n]
#        self._enregistrer = False

#    def supprimer(self):
#        self.__polygone.supprimer()

    def _get_coordonnees(self):
        return self.__polygone._Polygone_generique__points[self.__n].coordonnees

#    def _conditions_existence(self):
#        return [self.__polygone._Polygone__points[n].existe]

    def _modifier_hierarchie(self, valeur = None):
        # Pour les sauvegardes par exemple, il est prfrable que les sommets, puis les cots,
        # apparaissent juste aprs la construction du polygone ; ils doivent occuper des places conscutives dans la hirarchie.
        # Par exemple, si le polygone a 4 sommets, et si sa place dans la hierarchie est 18, ses trois sommets 
        # auront  comme valeur hierarchique, dans l'ordre, 18.1, 18.2, 18.3 et 18.4,
        # et ses cots auront pour valeur hirarchique 18.6, 18.7, 18.8, 18.9.
        N = len(self.__polygone._Polygone_generique__points)
        Objet._modifier_hierarchie(self, self.__polygone._hierarchie + (self.__n + 1)/(2*N + 2))

    def _update(self, objet):
        u"""Pseudo mise  jour: seul un objet identique est accept.
        
        Cela sert pour les objets crs automatiquement, qui peuvent tre enregistrs deux fois dans la feuille."""
        if isinstance(objet, Sommet) and self.__polygone is objet._Sommet__polygone and self.__n == objet._Sommet__n :
            self.style(**objet.style())
        else:
            raise RuntimeError




class Polygone_generique(Objet):
    u"""Un polygone gnrique.

    Classe mre de tous les polygones."""
   
    _style_defaut = param.polygones
    _prefixe_nom = "p"
    
    points = __points = Arguments("Point_generique")

    def __init__(self, *points, **styles):
        n = len(points)
        self.__points = points = tuple(Ref(obj) for obj in points)
        self.__centre = ALL.Barycentre(*(ALL.Point_pondere(point, 1) for point in points))
        Objet.__init__(self, **styles)
        self.etiquette = ALL.Label_polygone(self)
        self.__sommets = tuple(ALL.Sommet(self, i) for i in xrange(n))
        self.__cotes = tuple(ALL.Cote(self, i) for i in xrange(n))

    def _set_feuille(self):
        # On rfrence automatiquement tous les cts et sommets du polygone dans la feuille.
        # (En particulier, en mode graphique, cela permet de faire apparaitre tous les sommets du polygone lorsque celui-ci est cr)
        n = len(self.__points)
        points_feuille = self.__feuille__.objets(ALL.Point_generique)
        noms = self._style.get("_noms_", {"sommets": n*("", ), "cotes": n*("", )})
        for i in xrange(n):
            # On exclue les sommets qui seraient dj dans la feuille :
            if not is_in(self.__points[i], points_feuille): 
                nom = noms["sommets"][i]
                self.__feuille__.objets[nom] = self.__sommets[i]
            nom = noms["cotes"][i]
            self.__feuille__.objets[nom] = self.__cotes[i]


    def __repr__(self, *args, **kwargs):
        self.style(_noms_ = { "sommets" : tuple(sommet.nom for sommet in self.__sommets), 
                                        "cotes": tuple(cote.nom for cote in self.__cotes)})
        return Objet.__repr__(self, *args, **kwargs)

    @property
    def centre(self):
        return self.__centre
    
    centre_gravite = centre
    
    @property
    def sommets(self):
        return self.__sommets
        
    @property
    def cotes(self):
        return self.__cotes
    



    def _affiche(self):
        if not self.__representation__:
            self.__representation__ = [self.__canvas__.fill(), self.__canvas__.plot(zorder = 1)]

        fill = self.__representation__[0]
        plot = self.__representation__[1]
        niveau = self.style("niveau")
        points = self.__points + (self.__points[0],)
        xy = [pt.coordonnees for pt in points]
        x, y = zip(*xy)
        fill.xy = xy
        plot.set_data(x, y)
        fill._alpha = self.style("alpha")
        hachures = self.style("hachures")
        if hachures is not None:
            fill._hatch = hachures
        fill._edgecolor = fill._facecolor = plot._color = self.style("couleur")
        fill.zorder = niveau - 0.01
        plot._linestyle = self.style("style")
        fill._linestyle = ALL.FILL_STYLES.get(self.style("style"), "solid")
        plot._linewidth = fill._linewidth = self.style("epaisseur")
        plot.zorder = niveau




    def _distance_inf(self, x, y, d):
        xy = [self._pixel(pt) for pt in self.__points]
        return point_dans_polygone((x,y), xy)

    def __contains__(self, M):
        for cote in self.cotes:
            if M in cote:
                return True
        return point_dans_polygone(tuple(M), [pt.coordonnees for pt in self.__points])

    @property
    def aire(self):
        u"""D'aprs David Chandler, Area of a general polygon.
        http://www.davidchandler.com/AreaOfAGeneralPolygon.pdf"""

        if self.existe:
            points = self.__points
            xy = [pt.coordonnees for pt in (points + (points[0],))]
            s1 = s2 = 0
            for i in xrange(len(points)):
                s1 += xy[i][0]*xy[i+1][1]
                s2 += xy[i][1]*xy[i+1][0]
            return .5*abs(s1-s2)
        return None

    @property
    def perimetre(self):
        return sum(cote.longueur for cote in self.__cotes)

    def _espace_vital(self):
        points = self.__points
        x1 = min(pt.abscisse for pt in points)
        x2 = max(pt.abscisse for pt in points)
        y1 = min(pt.ordonnee for pt in points)
        y2 = max(pt.ordonnee for pt in points)
        return (x1, x2, y1, y2)





class Polygone(Polygone_generique):
    u"""Un polygone.

    Un polygone dfini par ses sommets."""
    
    
    def __new__(cls, *points, **styles):
        if isinstance(points[0], (list, tuple)):
            points = tuple(points[0])
        n = len(points)
        if n == 3 :
            newclass = Triangle
        elif n == 4:
            newclass = Quadrilatere
        elif n == 5 :
            newclass = Pentagone
        elif n == 6 :
            newclass = Hexagone
        elif n == 7 :
            newclass = Heptagone
        elif n == 8 :
            newclass = Octogone
        else:
            return super(Polygone, cls).__new__(cls, *points, **styles)
        objet = newclass.__new__(newclass, *points, **styles)
        objet.__init__(*points, **styles)
        return objet


    points = __points = Arguments("Point_generique")

    def __init__(self, *points, **styles):
        if isinstance(points[0], (list, tuple)):
            points = tuple(points[0])
        self.__points = points = tuple(Ref(obj) for obj in points)
        Polygone_generique.__init__(self, *points, **styles)












class Triangle(Polygone_generique):
    u"""Un triangle."""

    point1 = __point1 = Argument("Point_generique")
    point2 = __point2 = Argument("Point_generique")
    point3 = __point3 = Argument("Point_generique")

    def __init__(self, point1, point2, point3, **styles):
        self.__point1 = point1 = Ref(point1)
        self.__point2 = point2 = Ref(point2)
        self.__point3 = point3 = Ref(point3)
        Polygone_generique.__init__(self, point1, point2, point3, **styles)
        d1 = Perpendiculaire(Droite(point1, point2), point3)
        d2 = Perpendiculaire(Droite(point2, point3), point1)
        self.orthocentre = ALL.Intersection_droites(d1, d2)
        self.centre_cercle_circonscrit = Point_equidistant(point1, point2, point3)
        d1 = Bissectrice(point1, point2, point3)
        d2 = Bissectrice(point3, point1, point2)
        self.centre_cercle_inscrit = ALL.Intersection_droites(d1, d2)








class Quadrilatere(Polygone_generique):
    u"""Un quadrilatre."""

    point1 = __point1 = Argument("Point_generique")
    point2 = __point2 = Argument("Point_generique")
    point3 = __point3 = Argument("Point_generique")
    point4 = __point4 = Argument("Point_generique")

    def __init__(self, point1, point2, point3, point4, **styles):
        self.__point1 = point1 = Ref(point1)
        self.__point2 = point2 = Ref(point2)
        self.__point3 = point3 = Ref(point3)
        self.__point4 = point4 = Ref(point4)
        Polygone_generique.__init__(self, point1, point2, point3, point4, **styles)




class Pentagone(Polygone_generique):
    u"""Un pentagone."""

    point1 = __point1 = Argument("Point_generique")
    point2 = __point2 = Argument("Point_generique")
    point3 = __point3 = Argument("Point_generique")
    point4 = __point4 = Argument("Point_generique")
    point5 = __point5 = Argument("Point_generique")

    def __init__(self, point1, point2, point3, point4, point5, **styles):
        self.__point1 = point1 = Ref(point1)
        self.__point2 = point2 = Ref(point2)
        self.__point3 = point3 = Ref(point3)
        self.__point4 = point4 = Ref(point4)
        self.__point5 = point5 = Ref(point5)
        Polygone_generique.__init__(self, point1, point2, point3, point4, point5, **styles)




class Hexagone(Polygone_generique):
    u"""Un hexagone."""

    point1 = __point1 = Argument("Point_generique")
    point2 = __point2 = Argument("Point_generique")
    point3 = __point3 = Argument("Point_generique")
    point4 = __point4 = Argument("Point_generique")
    point5 = __point5 = Argument("Point_generique")
    point6 = __point6 = Argument("Point_generique")

    def __init__(self, point1, point2, point3, point4, point5, point6, **styles):
        self.__point1 = point1 = Ref(point1)
        self.__point2 = point2 = Ref(point2)
        self.__point3 = point3 = Ref(point3)
        self.__point4 = point4 = Ref(point4)
        self.__point5 = point5 = Ref(point5)
        self.__point6 = point6 = Ref(point6)
        Polygone_generique.__init__(self, point1, point2, point3, point4, point5, point6, **styles)




class Heptagone(Polygone_generique):
    u"""Un heptagone."""

    point1 = __point1 = Argument("Point_generique")
    point2 = __point2 = Argument("Point_generique")
    point3 = __point3 = Argument("Point_generique")
    point4 = __point4 = Argument("Point_generique")
    point5 = __point5 = Argument("Point_generique")
    point6 = __point6 = Argument("Point_generique")
    point7 = __point7 = Argument("Point_generique")

    def __init__(self, point1, point2, point3, point4, point5, point6, point7, **styles):
        self.__point1 = point1 = Ref(point1)
        self.__point2 = point2 = Ref(point2)
        self.__point3 = point3 = Ref(point3)
        self.__point4 = point4 = Ref(point4)
        self.__point5 = point5 = Ref(point5)
        self.__point6 = point6 = Ref(point6)
        self.__point7 = point7 = Ref(point7)
        Polygone_generique.__init__(self, point1, point2, point3, point4, point5, point6, point7, **styles)




class Octogone(Polygone_generique):
    u"""Un octogone."""

    point1 = __point1 = Argument("Point_generique")
    point2 = __point2 = Argument("Point_generique")
    point3 = __point3 = Argument("Point_generique")
    point4 = __point4 = Argument("Point_generique")
    point5 = __point5 = Argument("Point_generique")
    point6 = __point6 = Argument("Point_generique")
    point7 = __point7 = Argument("Point_generique")
    point8 = __point8 = Argument("Point_generique")

    def __init__(self, point1, point2, point3, point4, point5, point6, point7, point8, **styles):
        self.__point1 = point1 = Ref(point1)
        self.__point2 = point2 = Ref(point2)
        self.__point3 = point3 = Ref(point3)
        self.__point4 = point4 = Ref(point4)
        self.__point5 = point5 = Ref(point5)
        self.__point6 = point6 = Ref(point6)
        self.__point7 = point7 = Ref(point7)
        self.__point8 = point8 = Ref(point8)
        Polygone_generique.__init__(self, point1, point2, point3, point4, point5, point6, point7, point8, **styles)




class Parallelogramme(Quadrilatere):
    u"""Un paralllogramme."""

    point1 = __point1 = Argument("Point_generique")
    point2 = __point2 = Argument("Point_generique")
    point3 = __point3 = Argument("Point_generique")

    def __init__(self, point1, point2, point3, **styles):
        self.__point1 = point1 = Ref(point1)
        self.__point2 = point2 = Ref(point2)
        self.__point3 = point3 = Ref(point3)
        Quadrilatere.__init__(self, point1, point2, point3, ALL.Barycentre(ALL.Point_pondere(point1, 1), ALL.Point_pondere(point2, -1), ALL.Point_pondere(point3, 1)), **styles)





class Rectangle(Parallelogramme):
    u"""Un rectangle."""

    #  dfinir





class Losange(Parallelogramme):
    u"""Un losange."""

    #  dfinir




##class Polygone_regulier_centre_generique(Polygone_generique):
##    u"""Un polygone rgulier gnrique.
##
##    Classe mre de tout polygone rgulier dfini par son centre, un sommet, et le nombre de cts."""
##
##    def __new__(cls, *args, **styles):
##        if cls is Polygone_regulier_centre:
##            if len(args) == 3:
##                n = args[2]
##            else:
##                n = styles.get("n", None)
##            if n == 3:
##                return Objet.__new__(Triangle_equilateral_centre)
##            elif n == 4:
##                return Objet.__new__(Carre_centre)
##        return Objet.__new__(cls)
##
##    def __init__(self, centre, sommet, n, **styles):
##        self.__args = GestionnaireArguments()
##        self.__centre = centre = Argument(centre)
##        self.__sommet = sommet = Argument(sommet)
##        self.__n = n = Argument(int(n))
##        self._initialiser(centre = Point_generique, sommet = Point_generique, n = int)
##        Polygone.__init__(self, sommet, *(Rotation(centre, 2*pi*i/n)(sommet) for i in xrange(1, n)), **styles)
##        self.centre = NotImplemented # supprime l'avertissement qui est affich lorsqu'on "crase" une mthode ou un attribut d'une classe parente.




class Polygone_regulier_centre(Polygone_generique):
    u"""Un polygone rgulier.

    Un polygone rgulier dfini par son centre, un sommet, et le nombre de cts."""

    def __new__(cls, centre, sommet, n, **styles):
        if n == 3:
            newclass = Triangle_equilateral_centre
        elif n == 4:
            newclass = Carre_centre
        else:
            return super(Polygone_regulier_centre, cls).__new__(cls, centre, sommet, n, **styles)
        objet = newclass.__new__(newclass, centre, sommet, **styles)
        objet.__init__(centre, sommet, **styles)
        return objet

    centre = __centre = Argument("Point_generique")
    sommet = __sommet = Argument("Point_generique")
    n = __n = ArgumentNonModifiable("int")

    def __init__(self, centre, sommet, n, **styles):
        self.__centre = centre = Ref(centre)
        self.__sommet = sommet = Ref(sommet)
        self.__n = n # inutile de faire suivre la rfrence, car n n'est pas modifiable
        Polygone_generique.__init__(self, sommet, *tuple(ALL.Rotation(centre, 2*math.pi*i/n)(sommet) for i in xrange(1, n)), **styles)







class Triangle_equilateral_centre(Triangle):
    u"""Un triangle quilatral.

    Un triangle quilatral dfini par son centre et un sommet."""

    centre = __centre = Argument("Point_generique")
    sommet = __sommet = Argument("Point_generique")

    def __init__(self, centre, sommet, **styles):
        self.__centre = centre = Ref(centre)
        self.__sommet = sommet = Ref(sommet)
        Triangle.__init__(self, sommet, ALL.Rotation(centre, 2*math.pi/3)(sommet), ALL.Rotation(centre, 4*math.pi/3)(sommet), **styles)






class Carre_centre(Quadrilatere):
    u"""Un carr.

    Un carr dfini par son centre et un sommet."""

    centre = __centre = Argument("Point_generique")
    sommet = __sommet = Argument("Point_generique")

    def __init__(self, centre, sommet, **styles):
        self.__centre = centre = Ref(centre)
        self.__sommet = sommet = Ref(sommet)
        Quadrilatere.__init__(self, sommet, ALL.Rotation(centre, math.pi/2)(sommet), ALL.Rotation(centre, math.pi)(sommet), ALL.Rotation(centre, 3*math.pi/2)(sommet), **styles)









class Polygone_regulier(Polygone_generique):
    u"""Un polygone rgulier.

    Un polygone rgulier dfini par deux points conscutif (sens direct)."""

    def __new__(cls, point1, point2, n,  **styles):
        if n == 3:
            newclass = Triangle_equilateral
        elif n == 4:
            newclass = Carre
        else:
            return super(Polygone_regulier, cls).__new__(cls, point1, point2, n,  **styles)
        objet = newclass.__new__(newclass, point1, point2, **styles)
        objet.__init__(point1, point2, **styles)
        return objet

    point1 = __point1 = Argument("Point_generique")
    point2 = __point2 = Argument("Point_generique")
    n = __n = ArgumentNonModifiable("int")

    def __init__(self, point1, point2, n, **styles):
        self.__point1 = point1 = Ref(point1)
        self.__point2 = point2 = Ref(point2)
        self.__n = n # inutile de faire suivre la rfrence, car n n'est pas modifiable
        points = [point1, point2]
        for i in xrange(n - 2):
            # on ajoute l'image d'un sommet par une rotation de centre le sommet suivant
            points.append(ALL.Rotation(points[-1], -(n - 2)*math.pi/n)(points[-2])) 
        Polygone_generique.__init__(self, *points, **styles)






class Triangle_equilateral(Triangle):
    u"""Un triangle quilatral.

    Un triangle quilatral dfini par deux points conscutif (sens direct)."""

    point1 = __point1 = Argument("Point_generique")
    point2 = __point2 = Argument("Point_generique")

    def __init__(self, point1, point2, **styles):
        self.__point1 = point1 = Ref(point1)
        self.__point2 = point2 = Ref(point2)
        Triangle.__init__(self, point1, point2, ALL.Rotation(point1, math.pi/3)(point2), **styles)






class Carre(Quadrilatere):
    u"""Un carr.

    Un carr dfini par deux points conscutif (sens direct)."""

    point1 = __point1 = Argument("Point_generique")
    point2 = __point2 = Argument("Point_generique")

    def __init__(self, point1, point2, **styles):
        self.__point1 = point1 = Ref(point1)
        self.__point2 = point2 = Ref(point2)
        point3 = ALL.Rotation(point2, -math.pi/2)(point1)
        Quadrilatere.__init__(self, point1, point2, point3, ALL.Rotation(point1, math.pi/2)(point2), **styles)




##class PrevisualisationPolygone(Polygone):
##    u"""Une forme de polygone utilise pour la prvisualisation.
##    
##    Usage interne."""
##    
##    points = __points = None # On ne veut pas passer par un objet 'Arguments' ici (le but tant d'tre rapide).
##    
##    def __init__(self, *points):
##        self.points = points
##        self.__representation__ = [self.__canvas__.fill(), self.__canvas__.plot(zorder = 1)]
##        
##    def style(self, nom):
##        return param.polygones.get(nom, None)
##
##    def refresh(self):
##        u"Version (trs) raccourcie optimise pour la prvisualisation."
##        self._affiche()
##        self.__canvas__.actualiser()
##        
##    def _affiche(self):
##        fill = self.__representation__[0]
##        plot = self.__representation__[1]
##        niveau = self.style("niveau")
##        xy = [pt.coordonnees for pt in self.points]
##        x, y = zip(*xy)
##        xy.append(self.points[0].coordonnees)
##        fill.xy = xy
##        plot._x = x; plot._y = y
##        fill._alpha = self.style("alpha")
##        fill._facecolor = plot._color = self.style("couleur")
##        fill.zorder = niveau - 0.01
##        plot._linestyle = self.style("style")
##        plot._linewidth = self.style("epaisseur")
##        fill._linewidth = 0
##        plot.zorder = niveau


class PrevisualisationPolygone(Polygone_generique):
    u"""Une forme de polygone utilise uniquement pour la prvisualisation.
    
    Usage interne."""
    
    def _get_points(self):
        return self.__points
    
    def _set_points(self, points):
        for pt in self.__points:
            pt.vassaux.remove(self)
        for pt in points:
            pt.vassaux.append(self)
        # NOTE: self.__points doit tre de type tuple et surtout pas liste
        # (une ventuelle modification de la liste ne gererait pas correctement les vassaux)
        self.__points = points
    
    # Exceptionnellement, on ne passe par un objet 'Arguments' ici (le but tant d'tre le plus rapide possible).
    points = property(_get_points, _set_points)
    
    # De mme, inutile de passer par un descripteur de type 'DescripteurFeuille' pour l'attribut '__feuille__'
    __feuille__ = None 
    
    def __init__(self, *points):
        Objet.__init__(self)
#        self.vassaux = ()
        self.__points = ()
#        self.etiquette = None
        self.points = points
        
    def style(self, nom):
        return param.polygones.get(nom, None)

    def refresh(self):
        u"Version (trs) raccourcie optimise pour la prvisualisation."
        self._affiche()
        self.__canvas__.actualiser()
        
##    def _affiche(self):
##        if not hasattr(self,  "__fill__"):
##            self.__fill__ = self.__canvas__.fill()
##        fill = self.__fill__
##        xy = [pt.coordonnees for pt in self.__points]
##        xy.append(self.__points[0].coordonnees)
##        fill.xy = xy
##        fill._alpha = self.style("alpha")
##        fill._facecolor = self.style("couleur")
##        fill.zorder = self.style("niveau")
##        fill._linewidth = self.style("epaisseur")

    def _affiche(self):
        if not self.__representation__:
            self.__representation__ = [self.__canvas__.fill(), self.__canvas__.plot(zorder = 1)]

        fill = self.__representation__[0]
        plot = self.__representation__[1]
        niveau = self.style("niveau")
        points = self.__points
        xy = [pt.coordonnees for pt in points]
        x, y = zip(*xy)
        xy.append(self.__points[0].coordonnees)
        fill.xy = xy
        plot.set_data(x, y)
        fill._alpha = self.style("alpha")
        fill._edgecolor = fill._facecolor = plot._color = self.style("couleur")
        fill.zorder = niveau - 0.01
        plot._linestyle = self.style("style")
        fill._linestyle = ALL.FILL_STYLES.get(self.style("style"), "solid")
        plot._linewidth = fill._linewidth = self.style("epaisseur")
        plot.zorder = niveau

    def __repr__(self):
        return "PrevisualisationPolygone(%s)" %repr(self.__points)
