# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

from customtest import *



class TestGeolibAngles(CustomTest):

    def test_Secteur_angulaire(self):
        u = ALL.Vecteur_libre(5.458, -2.546)
        v = ALL.Vecteur_libre(-5.75, 12.6)
        P = ALL.Point(2.54, -5.68)
        a = ALL.Secteur_angulaire(P, u, v)
        self.assert_(isinstance(a.etiquette, ALL.Label_angle))
        self.assertAlmostEqual(a.val, 2.43538435941)
        self.assertAlmostEqual(a.degre, 139.537245287)
 
    def test_Angle_oriente(self):
        A = ALL.Point(250.54, 612.78)
        B = ALL.Point(115.54, 168.24)
        C = ALL.Point(412.78, -254.23)
        a = ALL.Angle_oriente(A, B, C)
        self.assertAlmostEqual(a.rad, -2.2336365048)
        
    def test_Angle(self):
        A = ALL.Point(250.54, 612.78)
        B = ALL.Point(115.54, 168.24)
        C = ALL.Point(412.78, -254.23)
        a = ALL.Angle(A, B, C)
        self.assertAlmostEqual(a.rad, +2.2336365048)

    def test_Angle_libre(self):
        x = random()
        a = ALL.Angle_libre(x)
        self.assertAlmostEqual(a.deg, x*180/math.pi)
        self.assertAlmostEqual(a.grad, x*200/math.pi)
        self.assertAlmostEqual(a.rad, x)
        self.assertAlmostEqual(a.val, x)
        self.assertAlmostEqual(math.sin(a.val), math.sin(x))
        self.assertAlmostEqual(math.cos(a.val), math.cos(x))
        self.assertAlmostEqual(math.tan(a.val), math.tan(x))
        y = x*180/math.pi
        a = ALL.Angle_libre(y, u"")
        self.assertAlmostEqual(a.deg, y)
        self.assertAlmostEqual(a.grad, x*200/math.pi)
        self.assertAlmostEqual(a.rad, x)
        self.assertAlmostEqual(a.val, a.deg)
        sin = ALL.universal_functions.sin
        cos = ALL.universal_functions.cos
        tan = ALL.universal_functions.tan
        self.assertAlmostEqual(sin(a), math.sin(x))
        self.assertAlmostEqual(cos(a), math.cos(x))
        self.assertAlmostEqual(tan(a), math.tan(x))
        a.unite = "g"
        self.assertAlmostEqual(a.val, a.grad)
        

    def test_Angle_vectoriel(self):
        u = ALL.Vecteur_libre(5.458, -2.546)
        v = ALL.Vecteur_libre(-5.75, 12.6)
        a = ALL.Angle_vectoriel(u, v)
        self.assertAlmostEqual(a.val, 2.43538435941)
        


if __name__ == '__main__':
    unittest.main()
