# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

from customtest import *
from feuille import *


class TestGeolibFeuille(CustomTest):
    
    def test_abreviations(self):
        f = Feuille(titre = u"Feuille de travail n1")
        o = f.objets
        
        self.assert_(o.has_key("Point"))
        self.assert_(o.has_key("Variable"))
        self.assert_(o.has_key("Texte"))
        
        o.t = ["salut"]
        self.assert_(isinstance(o.t, ALL.Texte))
        self.assert_(o.t.texte == "salut")
        o.s = [u"H, a marche !"]
        self.assert_(isinstance(o.s, ALL.Texte))
        self.assert_(o.s.texte == u"H, a marche !")

        o.A = (1, 2)
        o.k = 7
        self.assert_(isinstance(o.A, ALL.Point))
        self.assert_(isinstance(o.k, ALL.Variable))
        o.h = 'A.x'
        o.A.x = 15
        self.assert_(o.h == 15)
        o.h = "A.x-10"
        self.assert_(o.h == 5)
        self.assert_(o.h - 3 == 2)
        o.h = math.pi/3
        self.assertAlmostEqual(math.cos(o.h), 1/2)
        o.B = (-1, 3)
        o.u = o.A>o.B
        self.assert_(isinstance(o.u, ALL.Vecteur))
        self.assert_(o.u.coordonnees == (o.B.x - o.A.x, o.B.y - o.A.y))
        o.C = 2-3j
        self.assert_(isinstance(o.C, ALL.Point))
        self.assert_(o.C.z == 2-3j)
        o.C.x = "A.x"
        self.assert_(isinstance(o.C.x.val, (float, int, long)))
        self.assert_(o.C.x == o.A.x)
        o.A.coordonnees = -11, 3
        self.assert_(o.C.coordonnees[0] == -11)
        o.B.x = "A.x + 1"
        self.assert_(isinstance(o.B.x.val, (float, int, long)))
        self.assert_(o.B.x == o.A.x + 1)
        o.A.coordonnees = 30, -5
        self.assert_(o.B.coordonnees[0] == 31)
        o.A(-3.6, 0.4)
        self.assert_(o.C.coordonnees[0] ==-3.6)
        
 
        
    def test_acces_objets(self):
        f = Feuille()
        o = f.objets
        o.A = (1, 2)
        self.assert_(o["A"] is o.A)
        o.A_ = (7, -1.5)
        self.assert_(o["A`"] is o.A_)
        exec("b=Vecteur_libre()", o)
        self.assert_("b" in o.keys())
        exec("del b", o)
        self.assert_("b" not in o.keys())
        
    def test_sauvegarde(self):
        f = Feuille(titre = "Ma feuille")
        o = f.objets
        o.A = (1, 2)
        o.B = (-1, 3)
        o.k = 7
        o.s = ALL.Segment(o.A, o.B)
        f.sauvegarder()
       
    def test_rattachement_objets(self):    
        A=ALL.Point()
        self.assert_(A.x == 0 and A.y == 0)
        self.assert_(isinstance(ALL.Point.__feuille__, ALL.DescripteurFeuille))
        self.assert_(A.__feuille__ is None)
        f = Feuille()
        f.objets.A = A
        self.assert_(A.__feuille__ is f)
        self.assert_(A.x != 0 and A.y != 0)
        xmin, xmax, ymin, ymax = f.fenetre
        self.assert_(xmin <= A.x <= xmax and ymin <= A.y <= ymax)
        
    def test_variables_composees_1(self):
        f = Feuille()
        A = f.objets.A = ALL.Point()
        B = f.objets.B = ALL.Point()
        A(-5, 13)
        B.x = A.x
        self.assert_(B.x == B.coordonnees[0] == A.x == -5)
        A(1, 9)
        self.assert_(B.x == B.coordonnees[0] == -5 and A.x == 1)
        B.x="A.x"
        A(17, 5)
        self.assert_(B.coordonnees[0] == B.x == 17)

    def test_variables_composees_2(self):
        f = Feuille()
        f.objets.M1 = ALL.Point(-1.27482678984, 1.69976905312, legende=2)
        f.objets.M2 = ALL.Point(2.42032332564, 1.25635103926, legende=2)
        f.objets.s1 = ALL.Segment(f.objets.M1,f.objets.M2)
        f.objets.M1(-2.77136258661, 2.91916859122)
        f.objets.M1(4.74826789838, -1.07159353349)
        f.objets.M5 = ALL.Point(-5.11778290993, 2.30946882217, legende=2)
        f.objets.M6 = ALL.Point(-1.86605080831, 3.25173210162, legende=2)
        f.objets.s4 = ALL.Segment(f.objets.M5,f.objets.M6)
        f.objets.M5(-5.59815242494, 2.34642032333)
        f.objets.M1(-2.42032332564, -1.60739030023)
        f.objets.M6(-1.86605080831, 3.25173210162)
        f.objets.M6.renommer('B', legende = 1)
        f.objets.M6 = ALL.Point(2.91916859122, 3.5103926097, legende=2)
        f.objets.M6.modifier(**{'legende': 3, 'label': u'B.x'})
        f.objets.B(-1.18244803695, 1.25635103926)
        f.objets.M6.supprimer()
        f.objets.B(-2.21709006928, 2.64203233256)
        f.objets.M6 = ALL.Point(-6.6143187067, 0.443418013857, legende=2)
        f.objets.M6.renommer('C', legende = 1)
        f.objets.C.x=f.objets.B.x
        f.objets.B(-3.17782909931, 3.36258660508)
        f.objets.C.x="B.x"
        f.objets.B(-4.74826789838, 3.47344110855)
        f.objets.B(-1.99538106236, 3.63972286374)
        self.assert_(f.objets.C.coordonnees[0] == f.objets.C.x == f.objets.B.coordonnees[0] == f.objets.B.x)

    def test_polygones_et_representants_de_vecteurs(self):
        f = Feuille()
        f.objets.A = A = rand_pt()
        f.objets.B = B = rand_pt()
        f.objets.C = C = rand_pt()
        f.objets.p = ALL.Parallelogramme(A, B, C)
        f.objets.S1.renommer("D")
        s = repr(f.objets.p)
        del f.objets.p
        self.assert_("D" not in f.objets)
        exec("p=" + s, f.objets)
        self.assert_("D" in f.objets)
        self.assert_(f.objets.D is f.objets.p.sommets[3])
        
    def test_relier_point_axes(self):
        f = Feuille()
        f.objets.M1 = Point(-1.27482678984, 1.69976905312, legende=2)
        f.objets.M2 = Point(2.42032332564, 1.25635103926, legende=2)
        f.objets.s1 = Segment(f.objets.M1, f.objets.M2)
        f.relier_axe_x(f.objets.M1)
        f.relier_axe_y(f.objets.M1)
        
    def test_noms_latex(self):
        f = Feuille()
        f.objets.A = Point()
        self.assert_(f.objets.A.nom == "A")
        self.assert_(f.objets.A.nom_latex == "$A$")
        f.objets.B1 = Point()
        self.assert_(f.objets.B1.nom == "B1")
        self.assert_(f.objets.B1.nom_latex == "$B_{1}$")
        f.objets.C17 = Point()
        self.assert_(f.objets.C17.nom == "C17")
        self.assert_(f.objets.C17.nom_latex == "$C_{17}$")
        f.objets.objet5 = Point()
        self.assert_(f.objets.objet5.nom == "objet5")
        self.assert_(f.objets.objet5.nom_latex == "$objet_{5}$")
        f.objets.Delta = Point()
        self.assert_(f.objets.Delta.nom == "Delta")
        self.assert_(f.objets.Delta.nom_latex == "$\\Delta$")
        f.objets.delta = Point()
        self.assert_(f.objets.delta.nom == "delta")
        self.assert_(f.objets.delta.nom_latex == "$\\delta$")
        f.objets.phi15 = Point()
        self.assert_(f.objets.phi15.nom == "phi15")
        self.assert_(f.objets.phi15.nom_latex == "$\\phi_{15}$")
        f.objets.A__ = Point()
        self.assert_(f.objets.A__.nom == "A__")
        self.assert_(f.objets.A__.nom_latex == "$A''$")
        f.objets.A_B_ = Point()
        self.assert_(f.objets.A_B_.nom == "A_B_")
        self.assert_(f.objets.A_B_.nom_latex == "$A'B'$")
        f.objets.A_71 = Point()
        self.assert_(f.objets.A_71.nom == "A_71")
        self.assert_(f.objets.A_71.nom_latex == "$A'_{71}$")
        f.objets.A17B22 = Point()
        self.assert_(f.objets.A17B22.nom == "A17B22")
        self.assert_(f.objets.A17B22.nom_latex == "$A_{17}B_{22}$")
        
        
if __name__ == '__main__':
    unittest.main()
