# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

from customtest import *



class TestGeolibPoints(CustomTest):
    
    def test_Point(self):
        A = ALL.Point(1, 2)
        self.assert_(isinstance(A.etiquette, ALL.Label_point))
        self.assertEqual(A.x, 1)
        self.assertEqual(A.y, 2)
        self.assertEqual(type(A.coordonnees),  tuple)
        A.x = 5
        A.y = 7
        self.assertEqual(A.coordonnees, (5, 7))
        self.assert_(A.style("legende") == ALL.param.NOM)
        # Test du typage dynamique
        d = ALL.Droite(rand_pt(), rand_pt())
        B = ALL.Point(d)
        self.assert_(isinstance(B, ALL.Glisseur_droite))
        c = ALL.Cercle(A, 3)
        C = ALL.Point(c)
        self.assert_(isinstance(C, ALL.Glisseur_cercle))
        d = ALL.Demidroite(rand_pt(), rand_pt())
        B = ALL.Point(d)
        self.assert_(isinstance(B, ALL.Glisseur_demidroite))
        s = ALL.Segment(rand_pt(), rand_pt())
        B = ALL.Point(s)
        self.assert_(isinstance(B, ALL.Glisseur_segment))
        a = ALL.Arc_points(ALL.Point(), ALL.Point(1, 1),  ALL.Point(1, 2))
        B = ALL.Point(a)
        self.assert_(isinstance(B, ALL.Glisseur_arc_cercle))
        
 
        
    def test_Milieu(self):
        A=ALL.Point(1,2)
        B=ALL.Point(2,4)
        I=ALL.Milieu(A,B)
        self.assert_(I.x == (A.x+B.x)/2 and I.y == (A.y+B.y)/2)
        self.assertEqual(type(I.coordonnees),  tuple)

    def test_Barycentre(self):
        M=ALL.Point(3, 4)
        A=ALL.Point(1,2)
        B=ALL.Point(2,4)
        I=ALL.Milieu(A,B)
        K=ALL.Barycentre(M, A, B, (I, -4))
        self.assertEqual(K.x,  0)
        self.assertEqual(K.y,  2)
        self.assert_(not ALL.Barycentre(M, A, (B, -2)).existe)
        self.assertEqual(type(K.coordonnees),  tuple)
        
    def test_Point_final(self):
        A = ALL.Point(random(),random())
        B = ALL.Point(random(),random())
        C = ALL.Point(random(),random())
        u = ALL.Vecteur_libre(random(),random())
        v = ALL.Vecteur(ALL.Point(random(),  random()),  ALL.Point(random(),  random()))
        w = ALL.Representant(ALL.Vecteur(ALL.Point(random(),  random()),  ALL.Point(random(),  random())),  A)
        F = ALL.Point_final(C,  (u, v, w), (-2, 1.5, 4))
        self.assertEqual(type(F.coordonnees),  tuple)
        self.assertAlmostEqual(F.x,  C.x -2*u.x+1.5*v.x+4*w.x)
        self.assertAlmostEqual(F.y,  C.y -2*u.y+1.5*v.y+4*w.y)
        
    def test_Point_translation(self):
        u = ALL.Vecteur_libre(1.56, 2.78)
        v = ALL.Vecteur(ALL.Point(1.1457, 2.7895), ALL.Point(2.458, -8.25))
        A = ALL.Point(5.256, -7.231)
        tu = ALL.Translation(u)
        tv = ALL.Translation(v)
        Au = ALL.Point_translation(A, tu)
        Av = ALL.Point_translation(A, tv)
        self.assertAlmostEqual(Au.coordonnees,  (6.816, -4.451))
        self.assertAlmostEqual(Av.coordonnees, (6.5683, -18.2705))
        
    def test_Point_rotation(self):
        A = ALL.Point(1, -4)
        r = ALL.Rotation(ALL.Point(-2, 7.56), 4.25)
        B = ALL.Point_rotation(A, r)
        self.assertAlmostEqual(r(A).coordonnees, B.coordonnees)
        self.assertAlmostEqual(B.coordonnees,  (-13.684339450863803, 10.031803308717693))

    def test_Point_homothetie(self):
        A = ALL.Point(1, -2)
        h = ALL.Homothetie(A, -3)
        M = ALL.Point(2, 7)
        self.assertAlmostEqual(ALL.Point_homothetie(M, h).coordonnees, (-2, -29))
#        self.assert_(ALL.Point_homothetie(A, h) is A)   -> mthode __new__ ?

    def test_Point_reflexion(self):
        d = ALL.Droite_equation(1, 6, -2)
        r = ALL.Reflexion(d)
        M = ALL.Point(random()+randint(50)-randint(50), random()+randint(50)-randint(50))
        m = ALL.Mediatrice(M,  ALL.Point_reflexion(M, r))
        self.assertAlmostEqual(d.equation_reduite,  m.equation_reduite)
        
    def test_Projete_droite(self):
        d = ALL.Droite_equation(1.25, -7.12, 2.15)
        A = ALL.Point(1.52, 2.14)
        M = ALL.Projete_droite(A, d)
        self.assert_(M in d)
        self.assert_(ALL.Droite(A, M)._perpendiculaire(d))
        
    def test_Projete_cercle(self):
        c = ALL.Cercle_equation(1, -2, -20)
        self.assert_(c.existe)
        A = ALL.Point(randint(50) - randint(50) + random(), randint(50) - randint(50) + random())
        P = ALL.Projete_cercle(A, c)
        self.assert_(P.existe)
        self.assert_(P in c)
        self.assert_(P in ALL.Demidroite(c.centre, A))
        A.coordonnees = c.centre.coordonnees
        self.assert_(not P.existe)

    def test_Projete_arc_cercle(self):
        O = ALL.Point(23.15, -12.75)
        A = ALL.Point(-12.5, 7.14)
        B = ALL.Point(7.15, 8.64)
        a = ALL.Arc_cercle(O, A, B)
        M = rand_pt()
        P = ALL.Projete_arc_cercle(M, a)
        self.assert_(P in a)
        M.coordonnees = a.centre.coordonnees
        self.assert_(not P.existe)
        M.coordonnees = -17.826266675199999, 11.760911186
        self.assert_(type(P.coordonnees) is tuple)
        self.assertAlmostEqual(A.coordonnees, P.coordonnees)
        self.assertEqual(A, P)

    def test_Projete_segment(self):
        s = ALL.Segment(rand_pt(), rand_pt())
        M = rand_pt()
        self.assert_(ALL.Projete_segment(M, s) in s)
        A = ALL.Point(0, 1)
        B = ALL.Point(2, 1)
        s = ALL.Segment(A, B)
        M = ALL.Point(1, 7.15)
        P = ALL.Projete_segment(M, s)
        self.assertAlmostEqual(ALL.Milieu(A, B).coordonnees, P.coordonnees)
        M.x = .5
        self.assertAlmostEqual(ALL.Barycentre((A, 3), (B, 1)).coordonnees, P.coordonnees)
        M.x = -1
        self.assertAlmostEqual(A.coordonnees, P.coordonnees)
        M.x = 3
        self.assertAlmostEqual(B.coordonnees, P.coordonnees)
       

    def test_Projete_demidroite(self):
        d = ALL.Demidroite(rand_pt(), rand_pt())
        M = rand_pt()
        self.assert_(ALL.Projete_demidroite(M, d) in d)
        A = ALL.Point(0, 1)
        B = ALL.Point(2, 1)
        s = ALL.Demidroite(A, B)
        M = ALL.Point(1, 7.15)
        P = ALL.Projete_demidroite(M, s)
        self.assertAlmostEqual(ALL.Milieu(A, B).coordonnees, P.coordonnees)
        M.x = .5
        self.assertAlmostEqual(ALL.Barycentre((A, 3), (B, 1)).coordonnees, P.coordonnees)
        M.x = -1
        self.assertAlmostEqual(A.coordonnees, P.coordonnees)
        M.x = 3
        self.assertAlmostEqual(ALL.Barycentre((A, -1), (B, 3)).coordonnees, P.coordonnees)
        

    def test_Centre_gravite(self):
        M = rand_pt()
        A = rand_pt()
        B = rand_pt()
        C = rand_pt()
        I = ALL.Milieu(B, C)
        J = ALL.Milieu(A, C)
        K = ALL.Milieu(A, B)
        G = ALL.Centre_gravite(ALL.Triangle(A, B, C))
        self.assertAlmostEqual(ALL.Segment(A, G).longueur, 2*ALL.Segment(I, G).longueur)
        self.assertAlmostEqual(ALL.Segment(B, G).longueur, 2*ALL.Segment(J, G).longueur)
        self.assertAlmostEqual(ALL.Segment(C, G).longueur, 2*ALL.Segment(K, G).longueur)

    def test_Orthocentre(self):
        A, B, C = rand_pt(), rand_pt(), rand_pt()
        p = ALL.Polygone(A, B, C)
        H = ALL.Orthocentre(p)
        self.assert_(ALL.Droite(A, H).perpendiculaire(ALL.Droite(B, C)))
        self.assert_(ALL.Droite(B, H).perpendiculaire(ALL.Droite(A, C)))
        self.assert_(ALL.Droite(C, H).perpendiculaire(ALL.Droite(B, A)))

    def test_Centre_cercle_circonscrit(self):
        A, B, C = rand_pt(), rand_pt(), rand_pt()
        p = ALL.Polygone(A, B, C)
        O= ALL.Centre_cercle_circonscrit(p)
        self.assert_(O == ALL.Point_equidistant(A, B, C))

    def test_Centre_cercle_inscrit(self):
        A, B, C = rand_pt(), rand_pt(), rand_pt()
        p = ALL.Polygone(A, B, C)
        I = ALL.Centre_cercle_inscrit(p)
        P = ALL.Projete_segment(I, ALL.Segment(B, C))
        Q = ALL.Projete_segment(I, ALL.Segment(A, C))
        R = ALL.Projete_segment(I, ALL.Segment(A, B))
        c = ALL.Cercle_rayon(I, P)
        self.assert_(P in c and Q in c and R in c)

    def test_Centre(self):
        c = rand_cercle()
        O = ALL.Centre(c)
        M = ALL.Glisseur_cercle(c)
        self.assertAlmostEqual(ALL.Segment(O, M).longueur, c.rayon)

    def test_Point_equidistant(self):
        A = rand_pt()
        B = rand_pt()
        C = rand_pt()
        P = ALL.Point_equidistant(A, B, C)
        self.assertAlmostEqual(ALL.Segment(A, P).longueur, ALL.Segment(B, P).longueur)
        self.assertAlmostEqual(ALL.Segment(A, P).longueur, ALL.Segment(C, P).longueur)
        self.assert_(P in ALL.Mediatrice(A, B))

    def test_Glisseur_droite(self):
        A = rand_pt()
        B = rand_pt()
        d = ALL.Droite(A, B)
        M = ALL.Glisseur_droite(d)
        self.assert_(M in d)
        M.k = 0
        self.assertEqual(M.k, 0)
        self.assertAlmostEqual(M.coordonnees, A.coordonnees)
        P = ALL.Point(*M.coordonnees)
        M.k = 1
        self.assertEqual(M.k, 1)
        self.assertAlmostEqual(M.coordonnees, B.coordonnees)
        M.k = 2
        self.assertEqual(M.k, 2)
        M.k = -1
        self.assertEqual(M.k, -1)
        Q = ALL.Point(*M.coordonnees)
        self.assertAlmostEqual(ALL.Droite(P, Q).equation_reduite, d.equation_reduite)
        M.k = 1.7
        M(*M.coordonnees)
        self.assertAlmostEqual(M.k, 1.7)
        

    def test_Glisseur_segment(self):
        A = rand_pt()
        B = rand_pt()
        s = ALL.Segment(A, B)
        M = ALL.Glisseur_segment(s)
        self.assert_(M in s)
        M.k = 0
        self.assertEqual(M.k, 0)
        self.assertAlmostEqual(M.coordonnees, A.coordonnees)
        P = ALL.Point(*M.coordonnees)
        M.k = 1
        self.assertEqual(M.k, 1)
        self.assertAlmostEqual(M.coordonnees, B.coordonnees)
        M.k = 2
        self.assertEqual(M.k, 1) # 0<=k<=1 pour un segment
        self.assert_(M in s)
        M.k = -1
        self.assertEqual(M.k, 0) # 0<=k<=1 pour un segment
        self.assert_(M in s)
        M.k = 1
        Q = ALL.Point(*M.coordonnees)
        self.assertAlmostEqual(ALL.Droite(P, Q).equation_reduite, s.equation_reduite)
        M.k = 1.7
        M(*M.coordonnees)
        self.assertAlmostEqual(M.k, 1)

    def test_Glisseur_demidroite(self):
        A = rand_pt()
        B = rand_pt()
        d = ALL.Demidroite(A, B)
        M = ALL.Glisseur_demidroite(d)
        self.assert_(M in d)
        M.k = 0
        self.assertEqual(M.k, 0)
        self.assertAlmostEqual(M.coordonnees, A.coordonnees)
        P = ALL.Point(*M.coordonnees)
        M.k = 1
        self.assertEqual(M.k, 1)
        self.assertAlmostEqual(M.coordonnees, B.coordonnees)
        M.k = 2
        self.assertEqual(M.k, 2)
        M.k = -1
        self.assertEqual(M.k, 0) # k>=0 pour une demi-droite
        M.k = 1
        Q = ALL.Point(*M.coordonnees)
        self.assertAlmostEqual(ALL.Droite(P, Q).equation_reduite, d.equation_reduite)
        M.k = 1.7
        M(*M.coordonnees)
        self.assertAlmostEqual(M.k, 1.7)
 
    def test_Glisseur_cercle(self):
        c = rand_cercle()
        M = ALL.Glisseur_cercle(c)
        self.assert_(M in c)
        O = c.centre
        M.coordonnees = O.coordonnees
        # il faudrait complter un peu
        
    def test_Glisseur_arc_cercle(self):
        A = rand_pt()
        B = rand_pt()
        C = rand_pt()
        a = ALL.Arc_cercle(A, B, C)
        M = ALL.Glisseur_arc_cercle(a)
        self.assert_(M in a)
        O = a.centre
        M.coordonnees = O.coordonnees       
        # il faudrait complter un peu


        
        
if __name__ == '__main__':
    unittest.main()
