# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                  Variable                   #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode

from objet import *





class Formule(object):
    
    def _get_feuille(self):
        return self.__feuille
        
    def _set_feuille(self, value):
        self.__feuille = value
        liste = self._contenu[:]
        for i in xrange(1, len(liste), 2):
            self._contenu[i].__feuille__ = value
        
    __feuille__ = property(_get_feuille, _set_feuille)
    
    def __init__(self, parent, chaine = ""):
        print "Initialisation formule:", chaine, type(chaine)
        self._parent = weakref.ref(parent) # self._parent est une fonction qui renvoit parent si il existe encore. Cela permet de ne pas le maintenir en vie artificiellement (pas de rfrence circulaire).
        #~ self._cache_repr = chaine
        #~ self._cache_str = "<?>".join()
        if "{" not in chaine:
            liste = ["", "{" + chaine + "}", ""]
        else:
            liste = re.split("([{][^}]+[}])", chaine)
        for i in xrange(1, len(liste), 2):
            cache = liste[i][1:-1] # "{A.x}" -> "A.x"
            var = liste[i] = Variable(cache)
            var._cache = cache
            # on va maintenant redfinir la mthode affiche de toutes les variables de la formule : au lieu d'tre inactive, la mthode affiche va actualiser l'affichage de l'objet contenant la formule.
            def affiche(self, actualiser = False, formule = self):
                formule.parent.affiche()
            var.affiche = new.instancemethod(affiche, var, var.__class__)
        self._contenu = liste
        
        # il faut faire un systme de cache pour chaque variable :
        # - si la variable est calculable, on renvoie la valeur de la variable (et on met  jour son cache)
        # - sinon, on renvoie le cache s'il s'agit de repr, et <?> s'il s'agit de str.
        
        self.__feuille__ = self.parent.__feuille__

        
    @property
    def parent(self):
        return self._parent()
        
    def supprimer(self):
        for i in xrange(1, len(self._contenu), 2):
            self._contenu[i].supprimer()
    
    
    def __repr__(self):
        liste = self._contenu[:]
        for i in xrange(1, len(liste), 2):
            if liste[i].val is not None:
                liste[i]._cache = str(liste[i])
            liste[i] = "{" + liste[i]._cache + "}"
        return repr(unicode2("".join(liste)))
        #~ return unicode2("".join(liste))
        
        

        
    def __unicode__(self):
        liste = self._contenu[:]
        for i in xrange(1, len(liste), 2):
            if liste[i].val is None:
                s = "<?>"
            else:
                s = str(nchiffres(float(liste[i]), param.chiffres_significatifs))
            #~ if s == "None":
                #~ s = "<?>"
            liste[i] = s
        return unicode2("".join(liste))
        
        
    def __str__(self):
        return unicode(self).encode(param.encodage)
    
    
    



class Variable(Objet_numerique):
    u"""Une variable.
    
    Une variable numrique ; l'argument peut tre un nombre, ou une expression sous forme de chaine de caractres.
    Exemple: Variable(17.5), Variable('AB.longeur+1').
    Dans ce dernier cas, il est ncessaire qu'une feuille de travail soit dfinie.
    
    Note : ne pas dfinir directement l'attribut __valeur !"""
    
    _prefixe_nom = "k"
    
    __re = re.compile("((?<=[^.A-Za-z0-9_])[A-Za-z_][A-Za-z0-9_]*|^[A-Za-z_][A-Za-z0-9_]*)")
    
    def _set_valeur(self, value):
        if isinstance(value, Variable):
            if value.__feuille__ is not None:
                value.__feuille__ = self.__feuille__
            if value._Variable__fonction is None:
                return value.valeur
            return value.val
        elif isinstance(value, basestring):
            value = value.replace(" ","")
        return value
        
        
    __valeur = Argument("basestring, int, float, long", None,  _set_valeur, defaut = 0) 
    
    def valeur(self, value = None):
        if value is None:
            if self.__liste: # variable contenant une expression compile
                # On regnre l'expression  partir de l'expression compile.
                # C'est important, car certains objets de la feuille peuvent avoir chang de nom entre temps.
                valeur = ""
                for elt in self.__liste:
                    if isinstance(elt, Objet):
                        valeur += elt.nom
                    else:
                        valeur += elt
                return valeur
            else:
                return self.__valeur
        else:
            self._set_val(value)
    
    valeur = property(valeur, valeur)
    
    
    def __init__(self, valeur = 0, **styles):            
        Objet.__init__(self,  **styles)
        self.__liste = []
        self.__fonction = None
        self.__valeur = valeur = Ref(valeur)
    
    def _test_dependance_circulaire(self, valeur):
        u"""Provoque une erreur si l'objet se retrouve dpendre de lui-mme avec la nouvelle valeur.
        
        Retourne une liste compose alternativement d'instructions et d'objets de la feuille,
        et un ensemble constitu des objets de la feuille mis en jeu dans le code.
        (...  documenter ...)"""
        if isinstance(valeur, basestring) and self.__feuille__ is not None:
            liste = re.split(self.__re, valeur)
            ensemble = set()
            for i in xrange(1, len(liste), 2):
                obj = self.__feuille__.objets[liste[i]]
                if isinstance(obj, Objet):
                    liste[i] = obj
                    ensemble.add(obj)
                    if self is obj or is_in(self, obj._tous_les_ancetres()):
                        print self, 
                        raise RuntimeError, "Definition circulaire dans %s : l'objet %s se retrouve dependre de lui-meme." %(self, obj)
            return liste, ensemble
        return None, None
    
    
    def _compile(self,  liste, ensemble):
        u"""Compile l'expression stocke dans la variable ; les arguments sont les valeurs retournes par '_test_dependance_circulaire'.
        
        La compilation doit toujours avoir lieu  la fin de la procdure de redfinition de la variable,
        car elle ne doit tre excute que si la redfinition de la variable va effectivement avoir lieu,
        c'est--dire si tout le processus prcdent s'est excut sans erreur."""
        if self._type == "compose" and self.__feuille__ is not None:
##            re.findall(self.__re,  self.valeur)
            self.__liste = liste
            self.__fonction = eval("lambda:" + self.__valeur, self.__feuille__.objets)
            # on supprime la variable de la liste des vassaux pour les objets dont elle ne dpendra plus desormais:
            for objet in self._ancetres: 
                objet.vassaux.remove(self)
            self._ancetres = ensemble
            self._modifier_hierarchie()
            for objet in self._ancetres:   # l'objet est vassal de chacun des objets dont il depend
                objet.vassaux.append(self)
        else:
            for objet in self._ancetres: 
                objet.vassaux.remove(self)
            self._ancetres = set()
            self._modifier_hierarchie()
            self.__liste = []
            self.__fonction = None
    
    def _get_val(self):
        return self.__val_cache # cf. self._conditions_existence
   
    def _set_val(self, valeur = None): 
##        if isinstance(valeur, list): # Rserv  un usage interne (via self.__valeur)
##            raise TypeError,  "l'argument d'une variable doit-etre un nombre ou une chaine de caracteres."
        args = self._test_dependance_circulaire(valeur)
        self.__valeur = valeur
        self._compile(*args)


    def _set_feuille(self):
        self._compile(*self._test_dependance_circulaire(self.__valeur))
        self._heritiers_a_recalculer(self._heritiers())

    @property
    def _type(self):
        return isinstance(self.__valeur, basestring) and "compose" or "simple"


 
    def _recenser_les_ancetres(self):
#        warning("'_recenser_les_ancetres' n'a aucun effet pour une variable.")
        self._modifier_hierarchie()

                
            
    def _conditions_existence(self): # conditions specifiques pour que l'objet existe, a definir pour chaque objet
        if self._type == "compose":
            try:
                self.__val_cache = float(self.__fonction()) # Conversion en float pour l'affichage des points en particulier
            except:
                print_error(u"Impossible de dterminer la valeur de la variable " + self.nom + repr(self))
#                self.__val_cache = None
                return False
        else:
            self.__val_cache = self.valeur
        return True

    
        
    
    def __str__(self):
        return str(self.valeur)
    
    
        

    def varier(self, debut = 0, fin = 1, pas = 0.02, periode = 0.03):
        if self.__feuille__ is not None:
            self.__feuille__.start()
            for i in pylab.concatenate((pylab.arange(debut, fin, pas), (fin,))):
                t = time.clock()
                self.val = i
                while time.clock() < t + periode:
                    wx.YieldIfNeeded()
                    if self.__feuille__.__stop__:
                        break
                wx.YieldIfNeeded()
                if self.__feuille__.__stop__:
                    break



    def affiche(self, actualiser = False):
        if actualiser and self.__feuille__ is not None:
            canvas = self.__canvas__
            if canvas:
                canvas.actualiser()
            
        
    def add(self, y):
        u"Addition lie (le rsultat est une variable qui reste toujours gale  la somme des 2 valeurs)."
        if self._soustype() == "simple":
            if isinstance(y, Variable) and y._soustype() == "simple":
                return Variable(self()+y())
            elif isinstance(y, (float, int, long, complex)):
                return Variable(self()+y)
        return Variable("(%s)+(%s)" %(self, y))
    
    def mult(self, y):
        u"Multiplication lie (le rsultat est une variable qui reste toujours gale au produit des 2 valeurs)."
        if self._soustype() == "simple":
            if isinstance(y, Variable) and y._soustype() == "simple":
                return Variable(self()*y())
            elif isinstance(y, (float, int, long, complex)):
                return Variable(self()*y)
        return Variable("(%s)*(%s)" %(self, y))
        
        
        

     
    @staticmethod
    def _convertir(objet):
        u"Convertit un objet en variable."
##        if isinstance(objet, ALL.Variable):   #  Inutile (?)
##            return objet.copy()
        return ALL.Variable(objet)


    def _update(self, objet):
        if not isinstance(objet, Variable):
            objet = self._convertir(objet)
        if isinstance(objet, Variable):
            if objet.__feuille__ is not None:
                objet.__feuille__ = self.__feuille__
            if objet._Variable__fonction is None:
                self.valeur = objet.valeur
            else:
                self.val = objet.val
        else:
            raise TypeError, "l'objet n'est pas une variable."



##class Variable(Objet_numerique):
##    u"""Une variable.
##    
##    Une variable numrique ; l'argument peut tre un nombre, ou une expression sous forme de chaine de caractres.
##    Exemple: Variable(17.5), Variable('AB.longeur()+1').
##    Dans ce dernier cas, il est ncessaire qu'une feuille de travail soit dfinie."""
##    
##
##    def __init__(self, valeur = 0, **styles):            
##        Objet.__init__(self,  **styles)
##        self.__arguments__ = ('valeur', ) #  RECRIRE EN MME TEMPS QUE TOUTE LA CLASSE VARIABLE POUR SE CONFORMER AU MME MODLE QUE LES AUTRES OBJETS
##        self._set_val(valeur)
##    
##    def _recenser_les_ancetres(self): #  RECRIRE EN MME TEMPS QUE TOUTE LA CLASSE VARIABLE POUR SE CONFORMER AU MME MODLE QUE LES AUTRES OBJETS
##        u"""Met  jour l'ensemble des anctres, c--d. l'ensemble des objets dont dpend l'objet.
##        
##        L'ensemble est mis en cache dans self._ancetres.
##        """
##        if self.__arguments__:
##            self._ancetres.clear()
##            val = self.__valeur
##            if isinstance(val, (list, tuple)):
##                for item in val:
##                    if isinstance(item, Objet):
##                        self._ancetres.add(item)
##            elif isinstance(val, Objet):
##                self._ancetres.add(val)
##            self._modifier_hierarchie(max(-1, -1, *(objet._hierarchie for objet in self._ancetres)) + 1)
##
##   
##    def _soustype(self):
##        return isinstance(self.__valeur, list) and "compose" or "simple"
##
##    def _set_val(self, valeur = None):
##        u"""val() propose une interface standard pour afficher une valeur
##        (ou la modifier, si c'est une valeur libre), indpendamment de son origine.
##        
##        Pour comprendre ce qui se passe, il faut savoir que le contenu de la variable est stock dans self.__valeur.
##        S'il s'agit d'un nombre, il est stock tel quel, mais s'il s'agit d'une expression,
##        le code qui suit va remplacer toutes les noms d'objets par les objets eux-mmes.
##        Ainsi, si un objet est renomm, le contenu reste pertinent.
##        Exemple :
##        "u.norme() + v.norme()" devient ["", u, ".norme() + ", v, ".norme()"]
##        Le premier terme de la liste est toujours une chane (ventuellement vide), et le dernier aussi.
##        Le nombre d'lments de la liste est donc impair, avec une alternance de chanes et d'objets.
##        
##        La connaissance de ce fonctionnement interne est rarement requise, mais peut-tre utile
##        pour court-circuiter le fonctionnement normal de Variable par exemple.
##        """
##                
##        if valeur is not None:
####            if isinstance(valeur, Argument):
####                valeur = valeur.__objet__
##            if isinstance(valeur, Variable): # si u est une variable, Variable(u) renvoie une copie de u
##                valeur = valeur.__valeur
##
##            # on supprime la variable de la liste des vassaux pour les objets dont elle ne dpendra plus desormais:
##            for objet in self._ancetres: 
##                objet.vassaux.remove(self)
##            
##
##            
##            if isinstance(valeur, basestring): # on le convertit en liste
##                liste = regsub("[^.A-Za-z0-9_][A-Za-z_][A-Za-z0-9_]*", " " + valeur.replace(" ",""), lambda x:x[0]+'$'+x[1:]+'$').split("$")
##                for i in xrange(1, len(liste), 2):
##                    if liste[i]:
##                        liste[i] = self.__feuille__.objets[liste[i]]
##                valeur = liste
##
##            self.__valeur = valeur
##            self._recenser_les_ancetres()
##
##            if isinstance(valeur, (list, tuple)):
##            
##                # La premire tape, c'est d'liminer les dpendances circulaires.
##                # Par exemple, u = Variable(3); A = Point("u", "2*u"); u("A.x")
##                # A deviendrait alors  la fois hritier de u (ie. on a besoin de u pour calculer A), et anctre de u (ie. on a besoin de A pour calculer u !)
##                heritiers = self._heritiers() + [self]
##                for ancetre in valeur:
##                    if isinstance(ancetre, Objet):
##                        for heritier in heritiers:
##                            if ancetre is heritier:
##                                liste = valeur[:]
##                                for i in xrange(1, len(liste), 2):
##                                    liste[i] = [key for key, value in self.__feuille__.objets.items() if value is liste[i]][0]
##                                self.erreur(u"Dfinition circulaire dans %s : l'objet %s se retrouve dpendre de lui-mme." %("".join(liste).strip(), ancetre))
##                                raise RuntimeError, "Definition circulaire dans %s : l'objet %s se retrouve dependre de lui-meme." %("".join(liste).strip(), ancetre)
##                    
##                self.__valeur = list(valeur)
##                
##                for objet in self._ancetres:   # l'objet est vassal de chacun des objets dont il depend
##                    objet.vassaux.append(self)
##                    
##            elif isinstance(valeur, (int, float, long)):
##                self.__valeur = valeur
##            
##            else:
##                self.__valeur = float(valeur)
##
##                
##            
##    def _get_val(self):
##        if self._soustype() == "compose":
##            liste = self.__valeur[:]
##            for i in xrange(1, len(liste), 2):
##                try:
##                    liste[i] = [key for key, value in self.__feuille__.objets.items() if value is liste[i]][0]
##                except:
##                    print u"Objet introuvable sur la feuille (id %s) : %s" %(id(self.__feuille__), liste[i])
##                    raise
##            val = eval("".join(liste), self.__feuille__.objets)
##            if isinstance(val, Variable): # Variable("u+1") devient un alias pratique de Variable("u()+1")
##                val = val.val             # (mais d'un autre cote, ca entretient la confusion entre u et u()...)
##            return val
##        else:
##            return self.__valeur
##
##    
##    # donne une representation affichable de l'objet qui sert a generer la variable
##    def __repr__(self, styles = True):
##        u"""Attention, pour les variables, l'argument styles n'a pas la mme signification.
##        Il indique si l'affichage doit tre complet ou non.
##        
##        >>> Variable(7.2).__repr__(True)
##        "Variable(7.2)"
##        >>> Variable(7.2).__repr__(False)
##        "7.2"
##        """
##        if isinstance(self.__valeur, (int, float, long)):
##            if styles:
##                return "Variable(%s)" %self.val
##            else:
##                return repr(self.val)
##        else:
##            liste = self.__valeur[:]
##            for i in xrange(1, len(liste), 2):
##                liste[i] = [key for key, value in self.__feuille__.objets.items() if value is liste[i]][0]
##            return "Variable('%s')" % "".join(liste).strip()
##            
##        
##    
##    def __str__(self):
##        if isinstance(self.__valeur, (int, float, long)):
##            return str(self.val)
##        else:
##            liste = self.__valeur[:]
##            for i in xrange(1, len(liste), 2):
##                liste[i] = [key for key, value in self.__feuille__.objets.items() if value is liste[i]][0]
##            return "".join(liste).strip() # renvoit la chaine qui sert a initialiser la variable
##    
##    
##        
##
##    def varier(self, debut = 0, fin = 1, pas = 0.02, periode = 0.03):
##        if self.__feuille__ is not None:
##            self.__feuille__.start()
##            for i in pylab.concatenate((pylab.arange(debut, fin, pas), (fin,))):
##                t = time.clock()
##                self.val(i)
##                while time.clock() < t + periode:
##                    wx.YieldIfNeeded()
##                    if self.__feuille__.__stop__:
##                        break
##                wx.YieldIfNeeded()
##                if self.__feuille__.__stop__:
##                    break
##
##
##
##    def affiche(self, actualiser = False):
##        if actualiser and self.__feuille__ is not None:
##            canvas = self.__canvas__
##            if canvas:
##                canvas.actualiser()
##            
##        
##    def add(self, y):
##        u"Addition lie (le rsultat est une variable qui reste toujours gale  la somme des 2 valeurs)."
##        if self._soustype() == "simple":
##            if isinstance(y, Variable) and y._soustype() == "simple":
##                return Variable(self()+y())
##            elif isinstance(y, (float, int, long, complex)):
##                return Variable(self()+y)
##        return Variable("(%s)+(%s)" %(self, y))
##    
##    def mult(self, y):
##        u"Multiplication lie (le rsultat est une variable qui reste toujours gale au produit des 2 valeurs)."
##        if self._soustype() == "simple":
##            if isinstance(y, Variable) and y._soustype() == "simple":
##                return Variable(self()*y())
##            elif isinstance(y, (float, int, long, complex)):
##                return Variable(self()*y)
##        return Variable("(%s)*(%s)" %(self, y))
##        
##        
##        
##
##    def _conditions_existence(self): # conditions specifiques pour que l'objet existe, a definir pour chaque objet
##        try:
##            self._get_val()
##            return True
##        except:
##            print_error()
##            return False
##    
##    @staticmethod
##    def _convertir(objet):
##        u"Convertit un objet en variable."
####        if isinstance(objet, ALL.Variable):   #  Inutile (?)
####            return objet.copy()
##        return ALL.Variable(objet)
##
##
##    def _update(self, objet):
##        if not isinstance(objet, Variable):
##            objet = self._convertir(objet)
##        if isinstance(objet, Variable):
##            self.val = objet.val
##        else:
##            raise TypeError, "l'objet n'est pas une variable."
