#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#           Mathlib 2 (sympy powered)         #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode

## Cette librairie contient les fonctions de haut niveau non inclues dans sympy.

import sympy
import custom_objects
import math

def deg(x):
    if isinstance(x, (list, tuple)):
        x = pylab.array(x)
    return x*180/math.pi

def rad(x):
    if isinstance(x, (list, tuple)):
        x = pylab.array(x)
    return x*math.pi/180


def cbrt(x):
    u"Racine cubique de x."
    return cmp(x, 0)*math.exp(math.log(abs(x))/3)

def root(x, n):
    u"""Racine nime de x.
    
    N'est dfinie pour x ngatif que si n est pair."""
    if n%2:
        return cmp(x, 0)*math.exp(math.log(abs(x))/n)
    return math.exp(math.log(x)/n)

def prod(facteurs):
    return reduce(lambda x,y:x*y, facteurs, 1)


def gcd(a, b):
    u"pgcd de a et de b"
    # algorithme d'Euclide
    a, b = max(abs(a),abs(b)), min(abs(a),abs(b))
    while b:
        a, b = b, a%b
    return a

def lcm(a, b):
    u"ppcm de a et de b"
    return a*b//gcd(a,b)

def pgcd(*termes):
    u"Le plus grand dnominateur commun  un nombre quelconque d'entiers."
    return reduce(lambda x,y:gcd(x,y), termes)

    
def ppcm(*termes):
    u"Le plus petit multiple commun  un nombre quelconque d'entiers."
    return reduce(lambda x,y:lcm(x,y), termes)

#~ #TODO: Rcrire facteurs en utilisant une classe "Produit"
#~ def facteurs(n):
    #~ u"Dcompose n en produit de facteurs premiers."
    #~ n = int(n)
    #~ p = 1 # produit des facteurs
    #~ if n < 0:
        #~ n = abs(n)
        #~ p.append(-1)
    
    #~ while n%2 == 0:
        #~ n = n//2; p *= sympy.var('2')
    
    #~ i = 3
    #~ m = math.sqrt(n)
    #~ while i <= m:
        #~ while n%i == 0:
            #~ n = n//i; m = m//math.sqrt(i)
            #~ p *=sympy.var(str(i))
        #~ i += 2
        
    #~ if n != 1:  p *=sympy.var(str(n))
    
    #~ return p    






def n_premiers(n = 100, maximum = 50000): # securite face aux erreurs de frappe...!
    u"Donne la liste des n premiers nombres premiers."
    liste = [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97]
    m = liste[-1] + 2
    if n > maximum:
        raise OverflowError
    while len(liste) < n:
        r = math.sqrt(m)
        for k in liste:
            if k > r: # pas de diviseurs inferieurs  sa racine, donc le nombre est premier
                liste.append(m)
                break
            if not m%k:
                break
        m += 2
    return liste
    
    

    


def frac(x, maximum = 100000):
    u"Convertit un nombre dcimal en fraction." # (par nombre dcimal, il faut entendre float)
    for i in xrange(2, maximum):
        if (int(x*i)-i*x) == 0:
            return int(x*i), i
    for i in xrange(1, maximum):
        if (int(x*10**i)-10**i*x) == 0:
            return int(float(str(float(x*10**i)))), 10**i # le passage par str permet de supprimer les chiffres non significatifs
            
            
def bin(n):
    u"Conversion en binaire."
    s = ""
    while n:
        s = str(n%2) + s
        n //= 2
    return s
    
    
    
    
    
    
    
def custom_latex(expr, inline=True):
    return custom_objects.CustomLatexPrinter(inline).doprint(expr)



    
def derivee(f):
    return custom_objects.Fonction(f.args[0], sympy.diff(f.args[-1],f.args[0]))

    
    

    
    
