#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#   Mathlib 2 (sympy powered) #
##--------------------------------------#######
#WxGeometrie
#Dynamic geometry, graph plotter, and more for french mathematic teachers.
#Copyright (C) 2005-2007  Nicolas Pourcelot
#
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode

## Objets complmentaires  ceux de sympy

import sympy
#~ import end_user_functions
from sympy.printing.latex import LatexPrinter
from internal_objects import ObjetMathematique
import custom_functions

class Fonction(ObjetMathematique):
    def __init__(self, variables, expression):
        if not isinstance(variables, (list, tuple)):
            variables = (variables,)
        self.variables = variables
        self.expression = expression
    
    @classmethod
    def _substituer(cls, expression, dico):
        if hasattr(expression, "__iter__"):
            return expression.__class__(cls._substituer(elt, dico) for elt in expression)
        return expression.subs(dico)
    
    def __call__(self, *args, **kw):
        if kw:
            if args:
                raise ArgumentError, "les arguments sont entres de deux facons differentes."
            return self._substituer(self.expression, [(sympy.Symbol(key), value) for key, value in kw.iteritems()])
        if len(args) > len(self.variables):
            raise ArgumentError, "il y a plus d'arguments que de variables."
        return self._substituer(self.expression, zip(self.variables[:len(args)], args))
    
    def _variables(self):
        return tuple(str(arg) for arg in self.variables)
            
    def _expression(self, latex = False):
        if latex:
            expression = custom_functions.custom_latex(self.expression)
        else:
            expression = str(self.expression)
        return expression
    
    def __str__(self):
        #~ args = self.variables
        #~ expression = str(self.expression)
        #~ chaine = ""
        #~ for arg in args:
            #~ arg = str(arg)
            #~ expression = expression.replace(arg, arg.lstrip("_"))
            #~ chaine += ", " + arg[1:]
        #~ chaine += " -> " + expression
        return ", ".join(self._variables()) + " -> " + self._expression()
        
     
    for op in ("add", "mul", "div", "rdiv", "pow", "rpow"):    
        op = "__" + op + "__"
        def __op__(self, y, op = op):
            if isinstance(y, Fonction):
                if self.variables == y.variables or not y.variables:
                    return Fonction(self.variables, getattr(self.expression, op)(y.expression),)
                elif not self.variables:
                    return Fonction(y.variables, getattr(self.expression, op)(y.expression),)
                else:
                    raise ValueError, "les deux fonctions n'ont pas les memes variables."
            else:
                return Fonction(self.variables, getattr(self.expression, op)(y),)
        exec("%s=__op__" %op)
    del __op__
    
    def __ne__(self):
        return Fonction(self.variables, -self.expression)
        
    def __abs__(self):
        return Fonction(self.variables, abs(self.expression))

    def __eq__(self, y):
        if isinstance(y, Fonction):
            return self.expression == y.expression
        else:
            return self.expression == y
            
    def __gt__(self, y):
        if isinstance(y, Fonction):
            return self.expression > y.expression
        else:
            return self.expression > y


class ProduitEntiers(int):
    u"""Usage interne : destin  tre utilis avec sympy.factorint.
    
    Ajouter un attribut 'couples' APRS sa cration."""
    
    __slots__ = ["couples"]
    
        #~ def __init__(self, *couples):
            #~ self.couples = couples
            #~ n = 1
            #~ for entier, exposant in couples:
                #~ n *= entier**exposant
            #~ sympy.Integer.__init__(self, n)
    
    def __str__(self):
        def formater(exposant):
            if exposant == 1:
                return ""
            return "**" + str(exposant)
        return "*".join((str(entier) + formater(exposant)) for entier, exposant in self.couples)
        

class CustomLatexPrinter(LatexPrinter):
    def _print_exp(self, expr, exp=None):
        tex = r"\mathrm{e}^{%s}" % self._print(expr.args[0])
        return self._do_exponent(tex, exp)
        
    def _print_Exp1(self, expr):
        return r"\mathrm{e}"
        
    def _print_ImaginaryUnit(self, expr):
        return r"\mathrm{i}"
        
    def _print_Fonction(self, expr):
        return ", ".join(expr._variables()) + "\\mapsto" + expr._expression(latex = True)[1:-1]
        
    def _print_Function(self, expr, exp=None):
        func = expr.func.__name__

        if hasattr(self, '_print_' + func):
            return getattr(self, '_print_' + func)(expr, exp)
            
        else:
            if exp is not None:
                name = r"\mathrm{%s}^{%s}" % (func, exp)
            else:
                name = r"\mathrm{%s}" % func
            if len(expr.args) == 1 and isinstance(expr.args[0], sympy.Symbol):
                return name + "(" + str(self._print(expr.args[0])) +")"
            else:
                args = [ str(self._print(arg)) for arg in expr.args ]
                return name + r"\left(%s\right)" % ",".join(args)

    def _print_ProduitEntiers(self, expr):
        def formater(exposant):
            if exposant == 1:
                return ""
            return "^" + str(exposant)
        return "\\times ".join((str(entier) + formater(exposant)) for entier, exposant in expr.couples)
            
    def _print_Real(self, expr):
        s = str(expr)
        if "e" in s:
            nombre,  exposant = s.split("e")
            return nombre + "\\times 10^{" + exposant.lstrip("+") + "}"            
        else:
            return s
