#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#           Mathlib 2 (sympy powered)         #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode

## Cette librairie est une interface pour sympy :
## Elle modifie si ncessaire le comportement de fonctions sympy.

import sympy

import custom_objects 
import internal_functions

def expand(expression, variable = None):
    expression = sympy.expand(expression)
    if isinstance(expression, sympy.Basic) and expression.is_fraction():
        if variable is None and len(expression.atoms(sympy.Symbol)) == 1:
            variable = expression.atoms(sympy.Symbol).pop()
        return sympy.apart(expression, variable)
    return expression
    
    
def evalf(expression):
    return getattr(expression, "evalf")()
    
    
def factor(expression, variable = None, corps = None):
    if isinstance(expression, (int, long, sympy.Integer)):
        couples = sympy.factorint(expression)
        n = 1
        for entier, exposant in couples:
            n *= entier**exposant
        pdt = custom_objects.ProduitEntiers(n)
        pdt.couples = couples
        return pdt
        
        
    elif isinstance(expression, sympy.Basic) and expression.is_polynomial():
        if variable is None and len(expression.atoms(sympy.Symbol)) == 1:
            variable = expression.atoms(sympy.Symbol).pop()
        if variable is None:
            return sympy.factor(expression)
        else:
            return internal_functions.poly_factor(expression, variable, corps)
    resultat = sympy.together(expression)
    if resultat.is_fraction():
        num, den = resultat.as_numer_denom()
        if den != 1:
            return factor(num)/factor(den)
    return resultat
        
        
def cfactor(expression, variable = None):
    return factor(expression, variable, corps = "C")
        
        
def series(expression, variable, point, ordre):
    return getattr(expression, "series")(variable, point, ordre)
    
    
def diff(expression, variable = None, n = 1):
    if variable is None and hasattr(expression, "atoms") and len(expression.atoms(sympy.Symbol)) == 1:
        variable = expression.atoms(sympy.Symbol).pop()
    return sympy.diff(expression, variable, n)
    
    
def divisors(n):
    return sympy.divisors(int(n))
    
    
def limit(expression, *args):
    args =list(args)
    if args[-1] == "-" or args[-1] == "+":
        dir = args.pop()
    else:
        dir = None
    if len(args) == 1 and len(expression.atoms(sympy.Symbol)) == 1:
        args = [expression.atoms(sympy.Symbol).pop()] + args
    if dir is None:
        limite_gauche = sympy.limit(expression, args[0], args[1], "-")
        limite_droite = sympy.limit(expression, args[0], args[1], "+")
        if limite_gauche == limite_droite:
            return limite_gauche
        return limite_gauche, limite_droite
    else:
        return  sympy.limit(expression, args[0], args[1], dir)
    
