#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                Calculatrice                 #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode

import re
import decimal

from API import *
from GUI import *
#~ from mathlib.fonctions_utilisateur import __fonctions__
#~ import sympy
import mathlib
import sympy
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from mathlib.custom_objects import Fonction
from mathlib.custom_functions import custom_latex
from mathlib.end_user_functions import __classement__


class CalculatriceMenuBar(MenuBar):
    def __init__(self, panel):
        MenuBar.__init__(self, panel)
        self.ajouter(u"Fichier", ["quitter"])
        self.ajouter(u"Affichage", ["onglet"])
        liste = []
        for rubrique in __classement__:
            if rubrique != "Symboles":
                self.ajouter(rubrique, *([titre, doc, "", self.generer_fonction(nom)] for titre, nom, doc in __classement__[rubrique]))
            else:
                self.ajouter(rubrique, *([titre, doc, "", self.generer_fonction(nom, False)] for titre, nom, doc in __classement__[rubrique])) # pas de parenthese apres un symbole
        self.ajouter(u"Outils",
                        [u"Mmoriser le rsultat", u"Copie le resultat du calcul dans le presse-papier, afin de pouvoir l'utiliser ailleurs.", "Ctrl+M", self.panel.vers_presse_papier],
                        [u"Rinitialiser", u"Rinitialiser la calculatrice", "Ctrl+I", self.panel.initialiser],
                        [u"debug"],
                        [u"options"])
        self.ajouter("?")



    def generer_fonction(self, nom, parenthese = True):
        def f(event = None, panel = self.panel, nom = nom, parenthese = parenthese):
            deb, fin = panel.entree.GetSelection()
            if parenthese:
                panel.entree.SetInsertionPoint(fin)
                panel.entree.WriteText(")")
                panel.entree.SetInsertionPoint(deb)
                panel.entree.WriteText(nom + "(")
                panel.entree.SetFocus()
                if deb == fin:
                    final = fin + len(nom) + 1
                else:
                    final = fin + len(nom) + 2
            else:
                panel.entree.WriteText(nom)
                final = fin + len(nom)
            panel.entree.SetFocus()
            panel.entree.SetInsertionPoint(final)
            panel.entree.SetSelection(final, final)
        return f







#~ class GlobalDict(dict):
    #~ def __getitem__(self, name):
        #~ if self.has_key(name):
            #~ return dict.__getitem__(self, name)
        #~ return sympy.Symbol(name)
    
class LocalDict(dict):
    globals = {}
    def __getitem__(self, name):
        #~ print "Nom de clef: ", name
        #~ print "local: ", self.has_key(name)
        #~ print "global: ", self.globals.has_key(name)
        if self.has_key(name) or self.globals.has_key(name): # doit renvoyer une KeyError si la cl est dans le dictionnaire global, pour que Python y aille chercher ensuite la valeur associe  la cl
            return dict.__getitem__(self, name)
        return sympy.Symbol(name)

    def __setitem__(self, name, value):
        # Pour viter que l'utilisateur redfinisse pi, i, e, etc. par mgarde.
        if self.globals.has_key(name):
            raise KeyError, "%s est un nom reserve" %name
        if isinstance(value, str):
            # exec/eval encodent les chanes cres en utf8.
            value = value.decode("utf8").encode(param.encodage)
        dict.__setitem__(self, name, value)





class Decimal(decimal.Decimal):
    def __new__(cls, nbr):
        if isinstance(nbr, float):
            nbr = str(nbr)
        return decimal.Decimal.__new__(cls, nbr)


class Interprete(object):
    def __init__(self, parent):
        self.parent = parent
        # Dictionnaire local (qui contiendra toutes les variables dfinies par l'utilisateur).
        # Dictionnaire global (qui contient les fonctions, variables et constantes prdfinies).
        self.globals = vars(mathlib.end_user_functions).copy()
        #~ print "mathlib2 vars", self.globals
        self.globals.update({
                "__builtins__": None,
                "__fonction__": Fonction,
                "__sympify__": sympy.sympify,
                "ans": self.parent.ans,
                "__vars__": self.vars,
                "__decimal__": Decimal,
                "__type__":type, 
                            })
        # pour viter que les procdures de rcriture des formules ne touchent au mots clefs,
        # on les rfrence comme fonctions (elles seront inaccessibles, mais ce n'est pas grave).
        # ainsi, "c and(a or b)" ne deviendra pas "c and*(a or b)" !
        self.globals.update({}.fromkeys(securite.__keywords_autorises__, lambda:None))

        self.locals = LocalDict()
        self.locals.globals = self.globals
        self.param = self.parent.param
        

    def initialiser(self):
        self.locals.clear()

    def evaluer(self, calcul = "", mode_exact = True):
        # calcul = re.sub("[_]+", "_", calcul.strip()) # par mesure de scurit, les "__" sont interdits.
        # Cela permet ventuellement d'interdire l'accs  des fonctions.
        # Warning: inefficace. Cf. "_import _builtins_import _
        
        # Ferme automatiquement les parentheses.
        parentheses = "({[", ")}]"
        for i in range(3):
            correction = parentheses[1][i]*(calcul.count(parentheses[0][i])-calcul.count(parentheses[1][i]))
            calcul = calcul + correction
            if correction: self.parent.warning += u" Attention, il manque des parenthses \"" + parentheses[1][i] + "\"."

        if calcul[0] in "><!=^*/%+":
            calcul = "_" + calcul

        if ">>" in calcul:
            liste = calcul.split(">>")
            calcul = calcul[0]
            for s in liste[1:]:
                calcul = s + "(" + calcul + ")"

        # Cas d'une fonction.
        # Exemple: 'f(x,y)=x+y+3' sera traduit en 'f=__fonction__((x,y), x+y+3)'
        if re.match("[^=()]+[(][^=()]+[)][ ]*=[^=]", calcul):
            var, val = calcul.split("=", 1)
            var = var.strip()
            i = var.find("(")
            nom = var[:i]
            variables = var[i:]
            calcul = nom + "=__fonction__(" + variables + "," + val + ")"

        self.executer(self.traduire(calcul))
        
        if not mode_exact and hasattr(self.locals["_"], "evalf"):
            return self.formater(self.locals["_"].evalf())
        return self.formater(self.locals["_"])


    def _ecriture_scientifique(self, valeur):
        mantisse = int(math.floor(math.log10(abs(valeur))))
        latex = resultat = str(round(valeur/10.**mantisse, self.param("ecriture_scientifique_decimales")))
        if mantisse:
            resultat += "*10^"+str(mantisse)
            latex += "\\times 10^{%s}" %mantisse
        return  resultat, "$" + latex + "$"
 
    def _formater(self, valeur):
        resultat = unicode2(valeur)        
        # log -> ln
        resultat = regsub("(?<![A-Za-z0-9_])log[(]", resultat, "ln(")
        # E -> e
        resultat = regsub("(?<![A-Za-z0-9_.])E(?![A-Za-z0-9_])", resultat, "e")
        # I -> i
        resultat = regsub("(?<![A-Za-z0-9_])I(?![A-Za-z0-9_])", resultat, "i")
        # 452.25e+54 -> 452.25*10^54
        def puiss10(s):
            return s[:-2] + "*10^" + (s[-1] == "-" and "-" or "")
        resultat = regsub("(?<![A-Za-z0-9._])[0-9.]+e[-+]", resultat, puiss10)
        # ** -> ^
        resultat = resultat.replace("**", "^")
##        #~ resultat = resultat.replace("-", " - ").replace("+", " + ").replace("e - ", "E-").replace("e + ", "E+")
        # -x-25 -> -x - 25
        resultat = regsub("(?<=[A-Za-z0-9._])[-+](?=[A-Za-z0-9._])", resultat, lambda s: " " + s + " ")
##        .replace("+", " + ") = resultat.replace("-", " - ").replace("+", " + ").replace("- >", "->")
        return resultat
   
    def formater(self, valeur):
        if not self.param("calcul_exact") and self.param("ecriture_scientifique") and isinstance(valeur, sympy.Number):
            resultat, latex = self._ecriture_scientifique(valeur)
        else:
            resultat = self._formater(valeur)
            if valeur is None:
                latex = ""
            else:
                try:
                    latex = custom_latex(valeur)
                except Exception:
                    print_error()
                    latex = "<Affichage impossible>"
            
        if self.param("changer_separateurs"):
            resultat = resultat.replace(",", self.param("separateurs_personnels")[1]).replace(".", self.param("separateurs_personnels")[0])
            latex = latex.replace(",", self.param("separateurs_personnels")[1]).replace(".", self.param("separateurs_personnels")[0])
            
        return resultat, latex


    def traduire(self, formule):
        variables = self.globals.copy()
        variables.update(self.locals)
        #callable_types = (  types.BuiltinFunctionType,
                            #types.BuiltinMethodType, 
                            #types.FunctionType, 
                            #types.UfuncType, 
                            #types.MethodType, 
                            #types.TypeType, 
                            #types.ClassType)
        #fonctions = [key for key, val in variables.items() if isinstance(val, callable_types)]
        fonctions = [key for key, val in variables.items() if hasattr(val, "__call__")]
        #print fonctions
        # La fonction traduire_formule de la librairie mathlib.formatage permet d'effectuer un certain nombre de conversions.
        formule = traduire_formule(formule, fonctions, 
                        OOo = self.parent.param("formatage_OOo"), 
                        LaTeX = self.parent.param("formatage_LaTeX"), 
                        changer_separateurs = self.parent.param("changer_separateurs"), 
                        separateurs_personnels = self.parent.param("separateurs_personnels"), 
                        )
                        
        # On transforme les nombres en objets 'sympy.Number', et les dcimaux en objets 'decimal.Decimal'
        def transformer(chaine):
            if "." in chaine:
                return "__sympify__(__decimal__(" + chaine + "))"
            else:
                return "__sympify__(" + chaine + ")"
        return regsub("(?<![0-9A-Za-z_.])(([.][0-9]+)|([0-9]+[.]?[0-9]*))", formule, transformer)


    def executer(self, instruction):
        # dans certains cas, il ne faut pas affecter le rsultat  la variable "_" (cela provoquerait une erreur de syntaxe)
        # (Mots cls devant se trouver en dbut de ligne : dans ce cas, on ne modifie pas la ligne)
##        if True in [instruction.startswith(exception + " ") for exception in securite.__keywords_debut_ligne__]:
        def re_keywords(liste):
            global securite
            return "("+"|".join(securite.__keywords_debut_ligne__)+")[ :(;'\"]"

        if re.match(re_keywords(securite.__keywords_debut_ligne__), instruction):
            self.locals["_"] = None
        else:
            instruction = "_=" + instruction
        
##        keywords_interdits = [kw + " " for kw in securite.__keywords_interdits__] + [kw + "(" for kw in securite.__keywords_interdits__]
##        instruction = recursive_mreplace(instruction, keywords_interdits)
        regsub(re_keywords(securite.__keywords_interdits__), instruction, " ")

        #print self.globals.keys()
        exec(instruction, self.globals, self.locals)
        
    
    def vars(self):
        dictionnaire = self.globals.copy()
        dictionnaire.update(self.locals)
        return dictionnaire


















class Calculatrice(Panel_simple):
    __titre__ = u"Calculatrice" # Donner un titre a chaque module
    
    def __init__(self, parent):
        Panel_simple.__init__(self,parent)


        self.entrees = wx.BoxSizer(wx.HORIZONTAL)
        self.entree = wx.TextCtrl(self, size = (500, -1), style = wx.TE_PROCESS_ENTER)
        self.entrees.Add(self.entree, 1, wx.ALL|wx.GROW, 5)
        self.valider = wx.Button(self, wx.ID_OK)
        self.entrees.Add(self.valider, 0, wx.ALL, 5)
                
        self.corps = wx.BoxSizer(wx.HORIZONTAL)
        self.gauche = wx.BoxSizer(wx.VERTICAL)
        self.resultats = wx.TextCtrl(self, size = (400,310), style = wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_RICH)
        self.gauche.Add(self.resultats, 0, wx.ALL, 5)
        
        self.figure = Figure(figsize=(5,1.3),frameon=True, facecolor="w")
        self.visualisation = FigureCanvas(self, -1, self.figure)
        self.axes = self.figure.add_axes([0, 0, 1, 1], frameon=False)
        self.axes.axison = False
        self.pp_texte = self.axes.text(0.5, 0.5, "", horizontalalignment='center', verticalalignment='center', transform = self.axes.transAxes, size="large")
        self.gauche.Add(self.visualisation, 0, wx.ALL|wx.ALIGN_CENTER, 5)
        self.corps.Add(self.gauche, 0, wx.ALL, 5)
        
        
        ### Pave numerique de la calculatrice ###
        # On construit le pav de la calculatrice.
        # Chaque bouton du pav doit provoquer l'insertion de la commande correspondante.
        
        self.pave  = wx.BoxSizer(wx.VERTICAL)
        self.corps.Add(self.pave, 0, wx.ALL, 5)
        boutons = ["2nde", "ans", "ouv", "ferm", "egal", "7", "8", "9", "div", "x", "4", "5", "6", "mul", "y", "1", "2", "3", "minus", "z", "0", "pt", "pow", "plus", "t", "rac", "sin", "cos", "tan", "exp", "i", "pi", "e", "abs", "mod"]
        inserer = ["", "ans()", "(", ")", "=", "7", "8", "9",  "/", "x", "4", "5", "6", "*", "y", "1", "2", "3", "-", "z", "0", ".", "^", "+", "t", "sqrt(", ("sin(", "asin(", "sinus / arcsinus"), ("cos(", "acos(", "cosinus / arccosinus"), ("tan(", "atan(", "tangente / arctangente"), ("exp(", "ln(", "exponentielle / logarithme neperien"), ("i", "cbrt(", "i / racine cubique"), ("pi", "sinh(", "pi / sinus hyperbolique"), ("e", "cosh", "e / cosinus hyperbolique"), ("abs(", "tanh", "valeur absolue / tangente hyperbolique"), (" mod ", "log10(", "modulo / logarithme decimal")]
        
        self.seconde = False # indique si la touche 2nde est activee.
        
        def action(event = None):
            self.seconde = not self.seconde
            if self.seconde: self.message(u"Touche [2nde] active.")
            else: self.message("")
            
        self.actions = [action]
        
        for i in range(len(boutons)):
            # On aligne les boutons de la calculatrice par rangees de 5.
            if i%5 == 0:
                self.rangee = wx.BoxSizer(wx.HORIZONTAL)
                self.pave.Add(self.rangee, 0, wx.ALL,0)
            
            # Ensuite, on construit une liste de fonctions, parallelement  la liste des boutons.
            if i > 0:
                def action(event = None, entree = self.entree, j = i):
                    if type(inserer[j]) == tuple:
                        entree.WriteText(inserer[j][self.seconde])
                    else:
                        entree.WriteText(inserer[j])
                    n = entree.GetInsertionPoint()
                    entree.SetFocus()
                    entree.SetInsertionPoint(n)
                    self.seconde = False
                    self.message("")
                self.actions.append(action)
                
            bmp = png('btn_' + boutons[i])
            bouton = wx.BitmapButton(self, -1, bmp, style=wx.NO_BORDER)
            bouton.SetBackgroundColour(self.GetBackgroundColour())
            espace = 3
            if param.plateforme == "Linux":           
                espace = 0
            self.rangee.Add(bouton, 0, wx.ALL, espace)
            # A chaque bouton, on associe une fonction de la liste.
            bouton.Bind(wx.EVT_BUTTON, self.actions[i])
            if type(inserer[i]) == tuple: bouton.SetToolTipString(inserer[i][2])

        self.pave.Add(wx.BoxSizer(wx.HORIZONTAL))

        
        ### Liste des options ###
        # En dessous du pav apparait la liste des diffrents modes de fonctionnement de la calculatrice.

        # Calcul exact
        ligne = wx.BoxSizer(wx.HORIZONTAL)
        self.cb_calcul_exact = wx.CheckBox(self)
        self.cb_calcul_exact.SetValue(not self.param("calcul_exact"))
        ligne.Add(self.cb_calcul_exact, flag = wx.ALIGN_CENTER_VERTICAL)
        ligne.Add(wx.StaticText(self, -1, u" Valeur approche."), flag = wx.ALIGN_CENTER_VERTICAL)
        self.pave.Add(ligne)

        self.cb_calcul_exact.Bind(wx.EVT_CHECKBOX, self.EvtCalculExact)

        # Notation scientifique
        ligne = wx.BoxSizer(wx.HORIZONTAL)
        self.cb_notation_sci = wx.CheckBox(self)
        self.cb_notation_sci.SetValue(self.param("ecriture_scientifique"))
        ligne.Add(self.cb_notation_sci, flag = wx.ALIGN_CENTER_VERTICAL)
        self.st_notation_sci = wx.StaticText(self, -1, u" criture scientifique (arrondie  ")
        ligne.Add(self.st_notation_sci, flag = wx.ALIGN_CENTER_VERTICAL)
        self.sc_decimales = wx.SpinCtrl(self, -1, size = (45, -1), min = 0, max = 11)
        self.sc_decimales.SetValue(self.param("ecriture_scientifique_decimales"))
        ligne.Add(self.sc_decimales, flag = wx.ALIGN_CENTER_VERTICAL)
        self.st_decimales = wx.StaticText(self, -1, u" dcimales).")
        ligne.Add(self.st_decimales, flag = wx.ALIGN_CENTER_VERTICAL)
        self.pave.Add(ligne)
        
        self.EvtCalculExact()
        
        self.cb_notation_sci.Bind(wx.EVT_CHECKBOX, self.EvtNotationScientifique)
        self.sc_decimales.Bind(wx.EVT_SPINCTRL, self.EvtNotationScientifique)
        
        # Autres options
        self.options = [(u"Virgule comme sparateur dcimal.", u"changer_separateurs"), 
                    (u"Copie du rsultat dans le presse-papier.", u"copie_automatique"),
                    (u"Syntaxe OpenOffice.org", u"formatage_OOo"),
                    (u"Syntaxe LaTeX", u"formatage_LaTeX"),
                        ]
        self.options_box = []
        for i in range(len(self.options)):
            ligne = wx.BoxSizer(wx.HORIZONTAL)
            self.options_box.append(wx.CheckBox(self))
            ligne.Add(self.options_box[i], flag = wx.ALIGN_CENTER_VERTICAL)
            self.options_box[i].SetValue(self.param(self.options[i][1]))
            def action(event, chaine = self.options[i][1], entree = self.entree, self = self):
                self.param(chaine, not self.param(chaine))
                entree.SetFocus()
            self.options_box[i].Bind(wx.EVT_CHECKBOX, action)
            ligne.Add(wx.StaticText(self, -1, " " + self.options[i][0]), flag = wx.ALIGN_CENTER_VERTICAL)
            self.pave.Add(ligne)
            
            
        self.option1 = wx.BoxSizer(wx.HORIZONTAL)
        self.pave.Add(self.option1)
        #wx.CheckBox(self)
        
        self.option2 = wx.BoxSizer(wx.HORIZONTAL)
        self.pave.Add(self.option2)
 
        
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.entrees, 0, wx.ALL, 5)
        self.sizer.Add(self.corps, 0, wx.ALL, 5)
        self.SetSizer(self.sizer)
        self.Fit()
        
        # historique des calculs
        self.entree.Bind(wx.EVT_KEY_UP, self.EvtChar)
        self.entree.Bind(wx.EVT_RIGHT_DOWN, self.EvtMenu)
        #self.Bind(wx.EVT_CHAR, self.EvtChar, self.entree)
        self.valider.Bind(wx.EVT_BUTTON, self.affichage_resultat)
        self.visualisation.Bind(wx.EVT_RIGHT_DOWN, self.EvtMenuVisualisation)
        
        self.interprete = Interprete(self)
        
        self.initialiser()
        
        
    # <EN TRAVAUX>
    
    def _sauvegarder(self, fgeo):
        fgeo.contenu["Calculatrice"] = [{}]
        fgeo.contenu["Calculatrice"][0]["Calcul"] = self.historique
        fgeo.contenu["Calculatrice"][0]["Resultats"] = [repr(self.derniers_resultats)]
        fgeo.contenu["Calculatrice"][0]["Affichage"] = [self.resultats.GetValue()]
        fgeo.contenu["Calculatrice"][0]["Dernier_resultat"] = [self.dernier_resultat]
        fgeo.contenu["Calculatrice"][0]["Position"] = [repr(self.position)]
        fgeo.contenu["Calculatrice"][0]["Options"] = [{}]
        for i in range(len(self.options)):
            fgeo.contenu["Calculatrice"][0]["Options"][0][self.options[i][1]] = [str(self.options_box[i].GetValue())]


        
    def _ouvrir(self, fgeo):
        if fgeo.contenu.has_key("Calculatrice"):
            calc = fgeo.contenu["Calculatrice"][0]
            self.initialiser()
            
            self.historique = calc["Calculs"]
            self.derniers_resultats = securite.eval_safe(calc["Resultats"][0])
            self.position = int(calc["Position"][0])
            self.resultats.SetValue(calc["Affichage"][0] + "\n")
            
            liste = calc["Options"][0].items()
            options = [option for aide, option in self.options]
            for key, value in liste:
                value = securite.eval_safe(value[0])
                self.param(key, value)
                if key in options:
                    self.options_box[options.index(key)].SetValue(value)
            # il faudrait encore sauvegarder les variables, mais la encore, 2 problemes :
            # - pb de securite pour evaluer les variables
            # - pb pour obtenir le code source d'une fonction.
            # Pour remedier a cela, il faut envisager de :
            # - creer un module d'interpretation securisee.
            # - rajouter a chaque fonction un attribut __code__, ou creer une nouvelle classe.
    
    # </EN TRAVAUX>


    def modifier_pp_texte(self, chaine):
        u"""Modifier le rsultat affich en LaTeX (pretty print)."""
        if self.param("latex"):
            chaine = "$" + chaine + "$"
        else:
            chaine = chaine.replace("\\mapsto", "\\rightarrow")

        self.pp_texte.set_text(chaine)
        self.visualisation.draw()

    def vers_presse_papier(self, event = None, texte = None):
        if texte is None:
            texte = self.dernier_resultat
        self.clipBoard=wx.TheClipboard
        if self.clipBoard.Open():
            self.clipBoard.AddData(wx.TextDataObject(texte))
        self.clipBoard.Close()
    
    def copier_latex(self, event = None):
        self.vers_presse_papier(texte = self.pp_texte.get_text().strip("$"))
        
    def initialiser(self, event = None):
        self.dernier_resultat = "" # dernier resultat, sous forme de chaine formatee pour l'affichage
        self.historique = []
        self.position = -1
        self.derniers_resultats = [] # liste des resultats, sous forme de valeurs numeriques
        self.entree.SetFocus()
        self.interprete.initialiser()
        self.resultats.Clear()
    
    def affichage_resultat(self, event = None):
        entree = self.entree.GetValue()
        if not entree:
            if self.historique:
                entree = self.historique[-1]
            else:
                return
        try:
            self.warning = ""
            resultat, latex = self.interprete.evaluer(entree, self.param("calcul_exact"))
            #LaTeX
            debug("Expression LaTeX: "+ latex)
            try:
                self.modifier_pp_texte(latex)
            except Exception:
                print_error()
                self.modifier_pp_texte("<Affichage impossible>")
            #Presse-papier
            self.dernier_resultat = resultat
            if self.param("copie_automatique"): 
                self.vers_presse_papier()
            # TextCtrl
            longueur = len(self.resultats.GetValue())
            self.resultats.AppendText(u" Calcul n\xb0" + str(len(self.derniers_resultats)) + " :   " 
                                                        + unicode2(entree) + u"\n R\xe9sultat :" 
                                                        + " "*(4+len(str(len(self.derniers_resultats)))) 
                                                        + resultat + "\n__________________\n\n")
            self.message(u"Calcul effectu\xe9." + self.warning)
            self.historique.append(entree)
            self.derniers_resultats.append(resultat)
            self.position = len(self.historique) - 1
            self.entree.Clear()
            self.resultats.SetInsertionPoint(len(self.resultats.GetValue()))
            self.resultats.SetFocus()
            self.resultats.ScrollLines(1)
            self.entree.SetFocus()
        except Exception, erreur:
            self.message(u"Calcul impossible.")
            self.entree.SetFocus()
            if param.debug:
                raise


    def EvtMenu(self, event):
        if not event.ControlDown():
            event.Skip()
            return
        def generer_fonction(nom, parenthese = True, self = self):
            def f(event = None, panel = self, nom = nom, parenthese = parenthese):
                deb, fin = panel.entree.GetSelection()
                if parenthese:
                    panel.entree.SetInsertionPoint(fin)
                    panel.entree.WriteText(")")
                    panel.entree.SetInsertionPoint(deb)
                    panel.entree.WriteText(nom + "(")
                    panel.entree.SetFocus()
                    if deb == fin:
                        final = fin + len(nom) + 1
                    else:
                        final = fin + len(nom) + 2
                else:
                    panel.entree.WriteText(nom)
                    final = fin + len(nom)
                panel.entree.SetFocus()
                panel.entree.SetInsertionPoint(final)
                panel.entree.SetSelection(final, final)
            return f
        menu = wx.Menu()
        menu.SetTitle(u"Fonctions mathmatiques")
        debut = True
        for rubrique in __classement__:
            if not debut:
                menu.AppendSeparator()
            debut = False
            for titre, nom, doc in __classement__[rubrique]:
                i = wx.NewId()
                menu.Append(i, titre, doc)
                if rubrique != "Symboles":
                    menu.Bind(wx.EVT_MENU, generer_fonction(nom), id =i)
                else:
                    menu.Bind(wx.EVT_MENU, generer_fonction(nom, False), id =i) # pas de parenthese apres un symbole
        self.PopupMenu(menu)
        menu.Destroy()
#        self.entree.SetFocus()


    def EvtMenuVisualisation(self, event):
        menu = wx.Menu()
        i = wx.NewId()
        menu.Append(i, "Copier LaTeX",  "Copier le code LaTeX dans le presse-papier.")
        menu.Bind(wx.EVT_MENU, self.copier_latex, id=i)
        self.PopupMenu(menu)
        menu.Destroy()
        
    
    def EvtChar(self, event):    # gere la ligne d'entree des calculs.
        code = event.GetKeyCode()
        
        #print code
        if code == wx.WXK_RETURN or code == wx.WXK_NUMPAD_ENTER:
            self.affichage_resultat()

        elif code == wx.WXK_UP:   # on remonte dans l'historique des entrees
            if self.position >= 0:
                self.entree.SetValue(self.historique[self.position])
                self.position = self.position - 1

        elif code == wx.WXK_DOWN:   # on redescend dans l'historique vers les entrees recentes
            if self.position < len(self.historique) - 1:
                if self.position == (len(self.historique) - 2):
                    self.entree.Clear()
                else:
                    self.entree.SetValue(self.historique[self.position + 2])
                self.position = self.position + 1
        else:
            #return
            event.Skip()
            
            
        
    def EvtCalculExact(self, event = None):
        valeur = self.cb_calcul_exact.GetValue()
        self.param("calcul_exact", not valeur)
        if valeur:
            self.cb_notation_sci.Enable()
            self.st_notation_sci.Enable()
            self.sc_decimales.Enable()
            self.st_decimales.Enable()
        else:
            self.cb_notation_sci.Disable()
            self.st_notation_sci.Disable()
            self.sc_decimales.Disable()
            self.st_decimales.Disable()


    def EvtNotationScientifique(self, event):
        self.param("ecriture_scientifique", self.cb_notation_sci.GetValue())
        self.param("ecriture_scientifique_decimales", self.sc_decimales.GetValue())
    

    def ans(self, n = -1):
        n = int(n)
        if self.derniers_resultats: return self.derniers_resultats[n]
        self.warning += u" Ans(): aucun calcul antrieur." 
        return 0
    
    
##    def formater(self, valeur):
##        resultat = unicode2(valeur)
##        
##        # log -> ln
##        resultat = regsub("(?<![A-Za-z0-9_])log[(]", resultat, "ln(")
##        # E -> e
##        resultat = regsub("(?<![A-Za-z0-9_.])E(?!A-Za-z0-9_)", resultat, "e")
##        # I -> i
##        resultat = regsub("(?<![A-Za-z0-9_])I(?!A-Za-z0-9_)", resultat, "i")
##        # 452.25e+54 -> 452.25*10^54
##        resultat = regsub("(?<![A-Za-z0-9._])[0-9.]+e[-+]", resultat, lambda s:s[:-2]+"*10^")
##        # ** -> ^
##        resultat = resultat.replace("**", "^")
##        
##        #~ resultat = resultat.replace("-", " - ").replace("+", " + ").replace("e - ", "E-").replace("e + ", "E+")
##        resultat = resultat.replace("-", " - ").replace("+", " + ").replace("- >", "->")
##        
##        latex = ""
##        
##        # Gestion de l'ecriture scientifique
##        if not self.param("calcul_exact"):
##            if self.param("ecriture_scientifique") and isinstance(valeur, sympy.Number):
##                resultat, latex = self.ecriture_scientifique(valeur)
##            else:
##                latex = "$" + resultat + "$"
##            latex = latex.replace("*", "\\times ")
##            latex = regsub("[^][(]?[-]?[0-9]+[)]?", latex, lambda s: "^{" + s[1:].strip("()") + "}")
##
##        if self.param("changer_separateurs"):
##            resultat = resultat.replace(",", self.param("separateurs_personnels")[1]).replace(".", self.param("separateurs_personnels")[0])
##            latex = latex.replace(",", self.param("separateurs_personnels")[1]).replace(".", self.param("separateurs_personnels")[0])
##            
##        try:
##            if latex:
##                self.modifier_pp_texte(latex)
##        except:
##            print_error()
##
##        return resultat
