# -*- coding: iso-8859-1 -*-

from math import *

#------------------------------------------------------------------------------
class shape(object):
  """
  Classe de base.
  """

#------------------------------------------------------------------------------
class circle(shape):
  """
  circle(r), circle(d=...), circle(r=...)

  Implmentation d'un cercle.
  Un cercle peut tre dfinit soit par son rayon, soit par son diamtre.
  Si les deux paramtres sont dfinis, le rayon est utilis.

  Attributs disponibles :
  rayon : radius, r
  diamtre : diameter, d
  surface : area, a
  primtre : circumference, c, perimeter, r
  """

  def __init__(self, r=None, d=None):
    if r==None:
      r = abs(d/2.0)
    self.radius = abs(r)
    self.r = self.radius
    self.diameter = self.d = 2*self.r
    self.area = self.a = pi * self.r**2
    self.circumference = self.c = self.perimeter = self.p = 2 * pi * self.r

  def __name__(self):
    return 'circle'

#------------------------------------------------------------------------------
class sphere(shape):
  """
  sphere(r), sphere(d=...), sphere(r=...)

  Implmentation d'une sphre.
  Une sphre peut tre dfinie soit par son rayon, soit par son diamtre.
  Si les deux paramtres sont dfinis, le rayon est utilis.

  Attributs disponibles :
  rayon : radius, r
  diamtre : diameter, d
  surface : surface, s
  volume : volume, v
  """

  def __init__(self, r=None, d=None):
    if r==None:
      r = abs(d/2.0)
    self.radius = abs(r)
    self.r = self.radius
    self.diameter = self.d = 2*self.r
    self.surface = self.s = 4 * pi * self.r**2
    self.volume = self.v = 4.0/3 * pi * self.r**3

  def __name__(self):
    return 'sphere'

#------------------------------------------------------------------------------
class cylinder(shape):
  """
  cylinder(r, h)

  Implmentation d'un cylindre (droit).
  Un cylindre est dfinit par sa hauteur et le rayon de sa base.

  Attributs disponibles :
  rayon : radius, r
  hauteur : height, h
  base (surface) : basearea, base, b
  surface : surface, s
  volume : volume, v
  primtre : circumference, c, perimeter, r
  """

  def __init__(self, r, h):
    self.r = self.radius = abs(r)
    self.h = self.height = abs(h)
    self.b = self.basearea = self.base = circle(r=self.r).a
    self.volume = self.v = self.b * self.h
    self.surface = self.s = circle(r=self.r).c * self.h
    self.circumference = self.c = self.perimeter = self.p = 2 * pi * self.r

  def __name__(self):
    return 'cylinder'

#------------------------------------------------------------------------------
class rectangle(shape):
  """
  rectangle(x, y)

  Implmentation d'un rectangle.
  Un rectangle est dfinit par ses deux cots.

  Attributs disponibles :
  longeur : x
  largeur : y
  surface : area, a
  primtre : perimeter, p
  diagonale : diag, d
  moment quadratique par rapport  Y : Iyy, iyy
  moment quadratique par rapport  Z : Izz, izz
  """

  def __init__(self, x, y):
    self.x, self.y = abs(x), abs(y)
    self.area = self.a = x*y
    self.perimeter = self.p = 2*(x + y)
    self.Iyy = self.iyy = 1.0/12 * x * y**3
    self.Izz = self.izz = 1.0/12 * y * x**3
    self.diag = self.d = sqrt(x**2 + y**2)

  def __name__(self):
    return 'rectangle'

#------------------------------------------------------------------------------
class square(rectangle):
  """
  square(x)

  Implmentation d'un carr.
  Un carr est dfinit par son cot.
  Cette classe surcharge la classe rectangle.

  Attributs disponibles :
  longeur : x, y
  surface : area, a
  primtre : perimeter, p
  diagonale : diag, d
  : Iyy, iyy, secmon
  : Izz, izz
  """

  def __init__(self, x):
    rectangle.__init__(self, x, x)

  def __name__(self):
    return 'square'

#------------------------------------------------------------------------------
class triangle(shape):
  """
  triangle(x, h), triangle(a, b, c)

  Implmentation d'un triangle quelconque.
  Un triangle est dfinit par sa base et sa hauteur ou par ses trois cots.

  Attributs disponibles :
  longeur (base) : x
  hauteur : h
  surface : surface, s
  rayon du cercle circonscrit : rc
  rauon du cercle inscrit : ri
  """

  def __init__(self, x, h, c=None):
    if c == None:
      self.x, self.h = abs(x), abs(h)
      self.surface = self.s = 1.0/2 * self.x * self.h
      self.a = self.b = self.c = None
      self.perimeter = self.p = None
      self.rc = self.ri = None
    else:
      self.a, self.b, self.c = abs(x), abs(h), abs(c)
      self.perimeter = self.p = self.a + self.b + self.c
      hp = self.p / 2.0
      self.surface = self.s = sqrt(hp * abs(hp - self.a) * abs(hp - self.b) * abs(hp - self.c) )
      self.rc = (self.a * self.b * self.c) / (4.0 * self.s)
      self.ri = (2.0 * self.s) / (self.a + self.b + self.c)
      self.x = self.h = None

  def __name__(self):
    return 'triangle'

#------------------------------------------------------------------------------
class cube(shape):
  """
  cube(x)

  Implmentation d'un cube.
  Un cube est dfinit par son cot.

  longeur : x
  surface : surface, s
  volume : volume, v
  sommet : sommet
  face : face
  arete : arete
  perimetre developpe : perimeter, p
  """

  def __init__(self, x):
    self.x = abs(x)
    self.sommet = 8
    self.face = 6
    self.arete = 12
    self.surface = self.s = self.face * square(self.x).area
    self.volume = self.v = self.x ** 3
    self.perimeter = self.p = self.x * 14

  def __name__(self):
    return 'cube'

#------------------------------------------------------------------------------
class pyramid(shape):
  """
  pyramid(x, y, h), pyramid(b, h)

  Implmentation d'une pyramide.
  Une pyramide est dfinie par :
   - la surface de sa base et sa hauteur
   - la longueur et la largeur de sa base et sa hauteur

  Attributs disponibles :
  base (surface) : basearea, base, b
  longeur : x
  largeur : y
  hauteur : height, h
  volume : volume, v
  """

  def __init__(self, x, y, h=None):
    if h == None :
      self.b = self.basearea = self.base = abs(x)
      self.h = self.height = abs(y)
      self.volume = self.v = 1.0/3 * self.b * self.h
      self.x = self.y = None
    else :
      self.x, self.y = x, y
      self.b = self.basearea = self.base = abs(x) * abs(y)
      self.h = self.height = abs(h)
      self.volume = self.v = 1.0/3 * self.b * self.h

  def __name__(self):
    return 'pyramid'

#------------------------------------------------------------------------------
class cone(shape):
  """
  cone(r, h)

  Implmentation d'un cone circulaire.
  Un cne est dfinit par le rayon de sa base et sa hauteur.

  Attributs disponibles :
  rayon : radius, r
  diametre : diameter, d
  hauteur : height, h
  base (surface) : basearea, base, b
  surface : surface, s
  volume : volume, v
  """

  def __init__(self, r, h):
    self.r = self.radius = abs(r)
    self.h = self.height = abs(h)
    self.diameter = self.d = 2*self.r
    self.b = self.basearea = self.base = circle(r=self.r).a
    self.volume = self.v = 1.0/3 * self.b * self.h
    self.surface = self.s = pi * self.r * hypot(self.h, self.r)

  def __name__(self):
    return 'cone'

#------------------------------------------------------------------------------
class prism(shape):
  """
  prism(x, y, h), prism(b, h)

  Implmentation d'un prisme droit rectangulaire.
  Un  prisme droit est dfinit par :
   - la surface de sa base et sa hauteur
   - la longueur et la largeur de sa base et sa hauteur

  Attributs disponibles :
  base (surface) : basearea, base, b
  longeur : x
  largeur : y
  hauteur : height, h
  volume : volume, v
  """

  def __init__(self, x, y, h=None):
    if h == None :
      self.b = self.basearea = self.base = abs(x)
      self.h = self.height = abs(y)
      self.volume = self.v = self.b * self.h
      self.x = self.y = None
    else :
      self.x, self.y = x, y
      self.b = self.basearea = self.base = abs(x) * abs(y)
      self.h = self.height = abs(h)
      self.volume = self.v = self.b * self.h

  def __name__(self):
    return 'prism'

#------------------------------------------------------------------------------
class polyhedron(shape):
  """
  polyhedron(a, n)

  Implmentation d'un polyhdre rgulier.
  Un polyhdre rgulier est dfinit par la mesure d'un cot a et le nombre de faces n.

  Tetradre :
    sommet = 4
    cot = 3
    arte = 6
    surface = 1.73 * a ** 2
    volume = 0.12 * a ** 3

  Hexadre :
    sommet = 8
    cot = 4
    arte = 12
    surface = 6 * a ** 2
    volume = a ** 3

  Octadre :
    sommet = 6
    cot = 3
    arte = 12
    surface = 3.46 * a ** 2
    volume = 0.47 * a ** 3

  Dodcadre :
    sommet = 20
    cot = 5
    arte = 30
    surface = 20.6 * a ** 2
    volume = 7.66 * a ** 3

  Icosadre :
    sommet = 12
    cot = 3
    arte = 30
    surface = 8.66 * a ** 2
    volume = 2.18 * a ** 3

  Attributs disponibles :
  faces : face
  cots (par face) : cote
  sommets : sommet
  artes : arete
  surface : area
  volume : volume
  """

  def __init__(self, a, n):
    self.face = n
    if n == 4:
      self.sommet = 4
      self.cote = 3
      self.arete = 6
      self.area = 1.73 * a ** 2
      self.volume = 0.12 * a ** 3
    elif n == 6:
      self.sommet = 8
      self.cote = 4
      self.arete = 12
      self.area = 6 * a ** 2
      self.volume = a ** 3
    elif n == 8:
      self.sommet = 6
      self.cote = 3
      self.arete = 12
      self.area = 3.46 * a ** 2
      self.volume = 0.47 * a ** 3
    elif n == 12:
      self.sommet = 20
      self.cote = 5
      self.arete = 30
      self.area = 20.6 * a ** 2
      self.volume = 7.66 * a ** 3
    elif n == 20:
      self.sommet = 12
      self.cote = 3
      self.arete = 30
      self.area = 8.66 * a ** 2
      self.volume = 2.18 * a ** 3
    else:
      self.sommet = self.cote = self.arete = self.area = self.volume = 0

  def __name__(self):
    return 'polyhedron'
