# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##########################################
#            CONSOLE                      
##########################################
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode

# NOTES :
# Un fichier .geo est essentiellement un fichier python (structure par du XML).
# C'est potentiellement dangereux (en particulier sous Windows), car certains modules python (comme os, sys, ...), et certaines fonctions (write...) peuvent endommager le systeme d'exploitation.
# Une bonne partie du travail de ce module consiste donc a securiser l'environnement d'execution.
# Comme je suis loin d'etre un expert, il est possible (voire probable) que des failles subsistent neanmoins.
# Merci de m'en informer.


    
#  Il faudra donner a l'utilisateur la possibilite d'autoriser la macro ponctuellement, ou meme de changer le parametre de securite


from LIB import *




class Commande:
    def __init__(self, parent):
        self.parent = parent
        self.historique = self.parent.historique # historique des commandes
        # Liste des commandes d'affichage (gerees par le canevas).
        self.commandes_canvas = [u"zoomer(", u"orthonormer(", u"zoom_auto(", u"codage(", u"reperage(", u"graduations(", u"reperage_et_graduations(", u"effacer_traces(", u"relier(", u"modifier_quadrillage("]
        # Liste des commandes de manipulation de feuilles (gere par le module actif)
        self.commandes_panel = [u"creer_feuille(", u"charger_feuille(", u"rafraichir_feuille(", u"fermer_feuille("]
        #self.macro = ":"
        
        self.compteur = 0 # s'incrmente  chaque action.
        # cela permet aux autres parties du programme de savoir si quelque chose s'est pass.

        # sert essentiellement au debogage :
        self.afficher = "&"
        # ':&"bonjour"' est un equivalent de ':print "bonjour"' par exemple.
        
        
        
    
    def executer_code_externe(self, commande, abreviations = True, mode = wxgeo.NORMAL):
        u"Excute du code provenant d'une source externe (de l'utilisateur en gnral, ou d'un fichier, etc.)."
        self.mode = mode
        self.commande = self.commande_originale = commande.strip() # enleve les espaces, les retours a la ligne, etc...
        
        
        if self.commande:
            self.parent.canvas.regenerer_liste = True
                        
            macro = (self.commande[0] ==":")

            if macro and param.securite:
                if param.debug: print u"REQUETE:" + self.commande
                print(u"Commande non excute.")
                warning(u"Les paramtres de scurit n'autorisent pas l'excution de macros")
                return


            if macro:  self.commande = self.commande[1:]
            
            if self.commande[0] == ":" and param.debug:
                exec(self.commande[1:]) # execution brute ("de bas niveau") pour le deboguage
                return



            afficher = self.debute_par(self.afficher)
            if afficher:  self.commande = self.commande[1:]
            
            
            
            
            if abreviations:
                self.transformer()
            # transforme (A B) en Droite(A,B), [A B] en Segment(A,B), et |A B| en Variable(Segment(A,B)) (longueur de [AB])


            
            self.archiver = True # archiver l'etat actuel de la feuille
            
            if macro:               
                # Les racourcis suivants sont utilisables dans une macro :
                # ! pour self.parent.
                # !! pour self.parent.canvas.
                # !n pour self.parent.parent.onglet(n).
                # !n! pour self.parent.parent.onglet(n).canvas.
                # !: pour self.parent.actuelle.
                # !:: pour self.parent.actuelle.objets.
                if abreviations:
                    self.commande = self.commande.replace(u"!::", u"self.parent.actuelle.objets.")
                    self.commande = self.commande.replace(u"!:", u"self.parent.actuelle.")
                    liste = re.split(u"(![!0-9]*)", self.commande)
                    self.commande = u""
                    for i in range(len(liste)):
                        if i%2:
                            self.commande += u"self.parent."
                            if liste[i] == u"!!":
                                self.commande += u"canvas."
                            elif liste[i] <> u"!":
                                a, b, c = re.split(u"([0-9]+)", liste[i])
                                self.commande += u"parent.onglet(" + b + ")."
                                if c == u"!":
                                    self.commande += u"canvas."
                        else:
                            self.commande += liste[i]
                        
            else:
                # NOTE CONCERNANT LA SECURITE :
                # Le format d'encodage de sauvegarde est du code Python (!).
                # Les lignes commencant par ":" sont des macros.
                # Les autres lignes subissent un traitement pour les rendre inoffensives.
                #
                # Une instruction standard ne doit pas avoir accs  n'importe quoi,
                # et en particulier pas  la feuille elle-mme, mais seulement aux objets de la feuille.
                #
                # Toutes les fonctions sensibles susceptibles d'tre accessibles comportent un "__".
                # On remplace donc les occurences successives de "_" par un seul "_".
                # La concatenation pourrait eventuellement poser un probleme,
                # d'ou la necessite d'empecher l'execution d'eval et d'exec.
                #
                # L'utilisation de __self__. permet de passer par la methode __setattr__() de feuille.objets.
                # Or, cette methode a ete modifiee, il est donc important de passer par elle,
                # et de ne pas se contenter d'une affectation ordinaire (A=Point(1,2))
                # 
                # Il est important d'empcher l'excution de la fonction eval(),
                # qui permet de contourner certaines limitations (voir plus bas).
                #
                # L'instruction exec() ne peut pas etre desactive aussi facilement,
                # il faut s'arranger pour qu'elle provoque une erreur de syntaxe.
                # Exemple de code problematique : exec("print 3;exec ('K = __'+'feuille__')")
                # On remplace donc ";" par ","
                
                for code in self.commande.split(): # suppression de l'instruction exec
                    if code.startswith("exec") and (code == "exec" or code.startswith("exec'") or code.startswith("exec\"") or code.startswith("exec(")):
                        if param.debug: print u"REQUETE:" + self.commande
                        print(u"Commande non excute.")
                        warning(u"L'instruction 'exec' est interdite.")
                        return

                self.commande = self.commande.replace(";", ",")
                #~ self.commande = regexp("[_]+", self.commande, "'_'")
                self.commande = regsub("[_]+", self.commande, "_")
                for methode in ('func_globals', 'im_self'):
                    if self.commande.find("." + methode) != -1:
                        raise RuntimeError, "la methode '" + methode + "' est interdite."
                
                #if self.parent.actuelle:
                    #try:
                    #    self.parent.canvas.editeur.close() # evite d'enregistrer l'etat avec un nom d'objet en cours d'edition.
                    #except:
                    #    print_error()

#                dictionnaire = self.parent.actuelle and self.parent.actuelle.objets.__dict__ or {}
                dictionnaire = self.parent.actuelle and self.parent.actuelle.objets or {}
#                if self.parent.actuelle:
#                    dictionnaire["Objet"].__feuille__ = self.parent.actuelle
                #~ if self.parent.actuelle:
                    #~ dictionnaire.update(pause = self.parent.actuelle.objets.pause, effacer = self.parent.actuelle.objets.effacer)
                if self.debute_par(self.commandes_canvas):
                    self.prefixe(u"__canvas__.")
                    #dictionnaire = {u"__canvas__" : self.parent.canvas}
                    dictionnaire.update(__canvas__ = self.parent.canvas)
                elif self.debute_par(self.commandes_panel):
                    self.archiver = False # ne pas archiver l'etat actuel de la feuille (commandes non annulables)
                    self.prefixe(u"__parent__.")
                    #dictionnaire = {u"__parent__" : self.parent}
                    dictionnaire.update(__parent__ = self.parent)
                    # Pour ces commandes, il n'y a pas forcement de feuille actuelle.
                    # Il faut donc faire un dictionnaire personnalise.
                    # Attention a bien mettre __self__ et non self (pour ne pas qu'on puisse y acceder.
                    # Voir la note plus haut a ce sujet ("__" est remplace par "_")
                #~ else:
                    #~ if self.commande.startswith(u"del "):
                        #~ self.commande = self.commande[4:] + u".supprimer()"
                        #~ # Il aurait ete plus agreable de definir une fonction __del__ pour la classe objet.
                        #~ # Mais cette fonction est appelee lorsque la derniere reference est detruite, et en pratique,
                        #~ # c'est tres aleatoire...

            
            if afficher: self.prefixe(u"print ")
            

            if param.debug: print u"REQUETE:" + self.commande
            
            
            if macro:
                self.execution()
            else:
                def avertissement(*arg, **kw): print u"Instruction interdite en mode securis."
                fonctions_interdites = ["eval", "compile", "execfile", "open", "write", "getattr", "setattr"]
                securite = {}
                for fonction in fonctions_interdites:
                    securite[fonction] = avertissement
                self.execution(securite, dictionnaire, evaluer_si_possible = True)


    executer = executer_code_externe
    
    
    


    def executer_code_interne(self, commande, mode = wxgeo.NORMAL):
        u"Excute du code produit par le programme, ie. scuris et correctement format."
        self.mode = mode
        self.commande = self.commande_originale = commande
        self.parent.canvas.regenerer_liste = True
        self.archiver = True # archiver l'etat actuel de la feuille
        #if self.parent.actuelle:
        #    try:
        #        self.parent.canvas.editeur.close() # evite d'enregistrer l'etat avec un nom d'objet en cours d'edition.
        #    except:
        #        print_error()
        dictionnaire = self.parent.actuelle and self.parent.actuelle.objets or {}
#        if self.parent.actuelle:
#            dictionnaire["Objet"].__feuille__ = self.parent.actuelle
#        dictionnaire = self.parent.actuelle and self.parent.actuelle.objets.__dict__ or {}
        if self.debute_par(self.commandes_canvas):
            self.prefixe(u"__canvas__.")
            #dictionnaire = {u"__canvas__" : self.parent.canvas}
            dictionnaire.update(__canvas__ = self.parent.canvas)
        elif self.debute_par(self.commandes_panel):
            self.archiver = False # ne pas archiver l'etat actuel de la feuille (commandes non annulables)
            self.prefixe(u"__parent__.")
            #dictionnaire = {u"__parent__" : self.parent}
            dictionnaire.update(__parent__ = self.parent)
            # Pour ces commandes, il n'y a pas forcement de feuille actuelle.
            # Il faut donc faire un dictionnaire personnalise.
            # Attention a bien mettre __self__ et non self (pour ne pas qu'on puisse y acceder.
            # Voir la note plus haut a ce sujet ("__" est remplace par "_")
        #~ else:
            #~ if self.commande.startswith(u"del "):
                #~ self.commande = self.commande[4:] + u".supprimer()"
                #~ # Il aurait ete plus agreable de definir une fonction __del__ pour la classe objet.
                #~ # Mais cette fonction est appelee lorsque la derniere reference est detruite, et en pratique,
                #~ # c'est tres aleatoire...

        if param.debug: print u"REQUETE:" + self.commande
        
        
        self.execution({}, dictionnaire)





    def execution(self, securite = None, dictionnaire = None, evaluer_si_possible = False):
        geler_affichage = self.parent.canvas.geler_affichage
        try:
            if self.mode is wxgeo.RAPIDE:
                self.parent.canvas.geler_affichage = True
            if securite is None:
                feuille = self.parent.actuelle # alias pratiques
                if feuille is not None:
                    objets = feuille.objets
                exec(self.commande.encode(param.encodage))

            else: # commande scurise
                if evaluer_si_possible:
                    try:
                        dictionnaire["_"] = eval(self.commande.encode(param.encodage), securite, dictionnaire)
                    except SyntaxError:
                        exec(self.commande.encode(param.encodage), securite, dictionnaire)
                else:
                    exec(self.commande.encode(param.encodage), securite, dictionnaire)
            self.historique.enregistrer(self.commande_originale, self.archiver)
            if self.parent.actuelle:
                if not self.parent.actuelle.sauvegarde["modifie"] and self.archiver: # les commandes qui ne demandent pas d'archivage ne ncessitent pas non plus d'tre suivis d'une sauvegarde (exemple: charger_feuille())
                    self.parent.actuelle.sauvegarde["modifie"] = True # une modification a t faite depuis la dernire sauvegarde
                    self.parent.rafraichir_titre()  # rajoute une "*" dans le titre de la fentre
                if self.mode is wxgeo.NORMAL:
                    self.actualiser()
            if self.mode is wxgeo.RAPIDE:
                self.parent.canvas.geler_affichage = geler_affichage
                
        except:
            self.historique.enregistrer("#" + self.commande_originale, self.archiver)
            #if param.debug: raise
            if self.mode is wxgeo.RAPIDE:
                self.parent.canvas.geler_affichage = geler_affichage
            raise



    def actualiser(self):
        u"""Mises  jour diverses aprs l'excution d'une commande.
        
        Lorsque des commandes sont effectues en mode RAPIDE, cette mthode doit tre lance manuellement aprs la dernire commande."""
        self.parent.canvas.detecter() # redtection des objets  proximit du pointeur
        pertes = gc.collect() # gestion corrective des pertes de mmoire de geolib (vassaux non supprims)
        if pertes > 150 and param.debug:
            print u"Warning: %s rfrences circulaires." %pertes



    def debute_par(self, liste):
        return filter(lambda x: self.commande.startswith(x), liste)



    def prefixe(self, texte):
        self.commande = texte + self.commande




    def transformer(self):
        #~ # traduction de M', M", M''' en M_prime, M_seconde, M_tierce
        #~ flag = None # indique si on est dans une chane
        #~ precedent = commande = ""
        #~ i = 0
        #~ for caractere in self.commande:
            #~ i += 1
            #~ if caractere in "'\"":
                #~ if flag is None: # on n'est pas  l'intrieur d'une chane
                    #~ if precedent.isalnum() or precedent in "_'": # ce ne peut pas tre le dbut d'une chane
#~ #                        unicod = False
                        
#~ #                        if precedent == "u": # cas unicode. L'algorithme est loin d'tre parfait...
#~ #                        # (j'ai supprim le cas "r", les raw_string ne sont donc pas reconnues - trop compliqu).
#~ #                            if len(commande)>1 and commande[-2].isalnum(): # ce n'est pas un 'u' isol avant la chane (donc pas un unicode)
#~ #                                pass
#~ #                            elif caractere not in self.commande[i:]: # pas le dbut d'une chane unicode non plus, car le guillemet ne se retrouve plus aprs.
#~ #                                pass
#~ #                            else:
#~ #                                unicod = True
#~ #                        if unicod:
#~ #                            flag = caractere
#~ #                            commande += caractere
#~ #                        elif caractere == "'": # ajout d'un guillemet simple
                        #~ if caractere == "'": # ajout d'un guillemet simple
                            #~ if commande.endswith("_prime"):
                                #~ commande = commande[:-6] + "_seconde"
                            #~ elif commande.endswith("_seconde"):
                                #~ commande = commande[:-8] + "_tierce"
                            #~ else:
                                #~ commande += "_prime"
                        #~ else: # ajout de guillemets doubles
                            #~ if commande.endswith("_prime"):
                                #~ commande = commande[:-6] + "_tierce"
                            #~ else:
                                #~ commande += "_seconde"
                    #~ else: # on entre dans une chane
                        #~ flag = caractere
                        #~ commande += caractere
                #~ else:
                    #~ if flag == caractere: # on sort d'une chane
                        #~ flag = None
                    #~ commande += caractere
            #~ else:
                #~ commande += caractere
            #~ precedent = caractere
        #~ self.commande = commande

        # traduction de M`, M``, M``` en M_prime_, M_prime__prime_, M_prime__prime__prime_, etc.
        self.commande = self.commande.replace("`", "_")

        # transforme (A B) en Droite(A,B), [A B] en Segment(A,B), et |A B| en Variable(Segment(A,B)) (longueur de [AB])
        liste = re.split("([[(|][ ]*[A-Za-z_][A-Za-z0-9_]*[ ]+[A-Za-z_][A-Za-z0-9_]*[ ]*[])|])", self.commande)
        texte = ""
        for i in range(len(liste)):
            s = liste[i]
            if i%2:
                # morceau de chaine a traiter ("(A B)", "[A B]" ou "|A B|")
                ajouts = {"(" : ("Droite(", ")"), "[" : ("Segment(", ")"), "|" : ("Variable(Segment(", "))")}[s[0]]
                s = ajouts[0] + s[1:-1].strip().replace(" ", ",", 1) + ajouts[1]
            texte += s
        self.commande = texte

        # transforme (2 3), ou (2;3) en Point(2,3)
        liste = re.split("([(][ ]*[-]?[0-9.]+[ ]*[; ][ ]*[-]?[0-9.]+[ ]*[)])", self.commande)
        texte = ""
        for i in range(len(liste)):
            s = liste[i]
            if i%2:
                # morceau de chaine a traiter ("(2 3)", ou "(2;3)")
                s = "Point(" + s[1:-1].strip().replace(";", " ").replace(" ", ",", 1) + ")"
            texte += s
        self.commande = texte


        # Les expressions mathematiques style "$\alpha > \beta$" de matplotlib doivent etre rentrees comme raw_string,
        # sinon, le parser de matplotlib plante.
        #~ self.commande = regexp("""[^r]["][$][^$"]+[$]["]""", " " + self.commande, "x[0] + 'r' + x[1:].replace('\\\\\\\\','\\\\')")[1:] # avec ""
        #~ self.commande = regexp("""[^r]['][$][^$"]+[$][']""", " " + self.commande, "x[0] + 'r' + x[1:].replace('\\\\\\\\','\\\\')")[1:] # avec ''
        self.commande = regsub("""[^r]["][$][^$"]+[$]["]""", " " + self.commande, lambda x:x[0] + 'r' + x[1:].replace('\\\\','\\'))[1:] # avec ""
        self.commande = regsub("""[^r]['][$][^$"]+[$][']""", " " + self.commande, lambda x:x[0] + 'r' + x[1:].replace('\\\\','\\'))[1:] # avec ''
